/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#ifndef FILE_HANDLER_H

#define FILE_HANDLER_H
#include <strstream.h>
#include <fstream.h>
#include <qlist.h>
#include "chord.h"
#define LINE_LENGTH 1024

class NClef;
class NSign;
class NKeySig;
class NVoice;
class NStaff;
class NChord;
class NMainFrameWidget;
class mupWrn;
class mupFatl;

#define ERR_NOTE_COUNT	     1
#define ERR_LYRICS_COUNT     2
#define ERR_SLUR	     3
#define ERR_NOTE_128         4
#define ERR_IRREGULAER       5
#define ERR_NOTE_MIXED_TIES  6
#define ERR_LONELY_TRILL     7
#define ERR_BARTYPES	     8
#define ERR_TEMPO_PLACEMENT  9
#define ERR_VOL_PLACEMENT   10
#define ERR_PROG_PLACEMENT  11
#define ERR_NO_CLEF	    12
#define ERR_TOO_MANY_VOICES	    13
#define ERR_VAR_TRILLS      14
#define ERR_DRUM_STAFF      15
#define ERR_MULTIREST	    16


class badmeasure {
	public:
		badmeasure(int k, int t, int m, int r, int s) {
			kind = k; track = t; measure = m; realcount = r; shouldbe = s;
		}
		int kind;
		int track, measure;
		int realcount, shouldbe;
};

class pending_elements {
	public:
		NSign *pending_volsig, *pending_program_change;
};

class NFileHandler {
	public:
		NFileHandler();
		bool readStaffs(const char *fname, QList<NVoice> *voilist, QList<NStaff> *stafflist, NMainFrameWidget *mainWidget);
		bool writeStaffs(QString fname, QList<NStaff> *stafflist, NMainFrameWidget *mainWidget, bool showErrors);
	private:
		void pitchOut( const NNote *note, NClef *ac_clef, bool with_tie);
		int writeStaffUntilBar(int staff_nr, NVoice *voi, bool first, int multirestlength);
		void writeVoiceElemsTill(int staff_nr, int voice_nr, NVoice *voi, int stopTime, int multirestlength);
		void writeScoreInfo(int voice_nr, NVoice *voi, bool firstcall);
		void writeTempoSig(NSign *temposig);
		void writeVolSig(int staff_nr, NSign *volsig);
		void writeProgramChange(int staff_nr, NSign *pChange);
		int determineMultiRest(QList<NStaff> *stafflist);
		bool divide_multi_rest(int staff_nr, int voice_nr, int multirestlength);
		QString lyrics2MUP(QString *lyrics);
		ofstream out_;
		char s_[LINE_LENGTH];
		char buffer_[128], ornamentbuffer_[128], dynamicsbuffer_[128];
		char lyricsbuffer_[NUM_LYRICS][128];
		pending_elements *pending_elems_;
		int **pending_multi_rests_;
		NSign *pending_tempo_sig_;
		ostrstream *os_;
		ostrstream *ornaments_;
		ostrstream *dynamics_;
		ostrstream *lyricsLine_[NUM_LYRICS];
		int lnr_;
		ifstream in_;
		int staffCount_;
		bool musicmode_;
		bool hasClef_;
		int bar_nr_;
		int countof128th_;
		int curr_num_, curr_denom_;
		bool drum_problem_written_;
		mupWrn* mupWarn_;
		mupFatl* mupFatl_;
		QList<badmeasure> badlist_;
		QList<badmeasure> fatallist_;
		QRegExp newLines_;
		
};

#endif // FILE_HANDLER_H
