/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kstddirs.h>
#include <kkeydialog.h>
#if KDE_VERSION >= 220
#include <ktip.h>
#endif
#include <qpainter.h>
#include <qslider.h>
#include <qtabwidget.h>
#include <kprogress.h>
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include "mainframewidget.h"
#include "configuredialog.h"
#include "resource.h"
#include "scaleedit_impl.h"
#include "midimapper.h"
#include "transpainter.h"
#include "clef.h"
#include "scoreinfo.h"
#include "sign.h"
#include "keysig.h"
#include "timesig.h"
#include "keyoffs.h"
#include "filehandler.h"
#include "midiexport.h"
#include "musixtex.h"
#include "pmxexport.h"
#include "lilyexport.h"
#include "dbufferwidget.h"
#include "numberdisplay.h"
//#include "clefdialog.h"
#include "staff.h"
#include "lyrics.h"
#include "zoomselection.h"
#include "uiconnect.h"
#include "voicedialog.h"
#include "notesel.h"
#ifdef WITH_TSE3
#include "tse3handler.h"
#endif
#include <kstdaction.h>
#include <klocale.h>
#include <kwin.h>
#include <kfiledialog.h>
// KToggleAction has setChecked, isChecked, and activate()
// for what QPushButton calls setOn, isOn, and toggle()
#define setOn setChecked
#define isOn isChecked
#define toggle activate

#define PUSHBUTTONWIDTH 40
#define PUSHBUTTONHEIGHT 30

// Not needed anymore
#define MENUBARHEIGHT 0
#define TOOLBARHEIGHT 0

#define QBHEIGHT 20
#define QBWIDTH 40
#define PUSH_DIST 2
#define SCROLLBARHEIGHT 14
#define Y_SCROLL_DIST 100
#define BORDER 4
#define CUR_DIST 10


#define LINE_WIDTH 1
#define MID_LINE_WIDTH 1
#define RIGHT_BORDER 10
#define LISTWIDTH 250
#define KEYOFFSWIDTH 25
#define LIST_DIST 0
#define KEY_OFFS_LABEL_WIDTH 20
#define KEY_OFFS_LABEL_HEIGHT 20
#define KEY_OFFS_LABEL_DIST -20
#define RBORDER 10
#define Y_STAFF_BASE 40

#define SMALL_X_SCROLL 200
#define SMALL_X_SENS_DIST 150

#define SMALL_X_REC_SCROLL 35
#define SMALL_X_REC_SENS_DIST 30

#define STAFF_PORTION 2

#define OVER 0

#define OVERLENTH_CHECK_INTERVAL 1000

//export pages:

#define TOOL_ELEMENT_WIDTH	200
#define TOOL_ELEMENT_HEIGHT	70

char *NMainFrameWidget::keySigTab_[13] =
{I18N_NOOP("C Major; a minor"),           I18N_NOOP("F Major; d minor"),
 I18N_NOOP("B flat Major; g minor"),      I18N_NOOP("E flat Major; c minor"),
 I18N_NOOP("A flat Major; f minor"),      I18N_NOOP("D flat Major; b minor"),
 I18N_NOOP("G flat Major; e flat minor"), I18N_NOOP("G Major; e minor"),
 I18N_NOOP("D Major; b minor"),           I18N_NOOP("A Major; f sharp minor"),
 I18N_NOOP("E Major; c sharp minor"),     I18N_NOOP("B Major; g sharp minor"),
 I18N_NOOP("F sharp Major; d sharp minor")
};

NMainFrameWidget::NMainFrameWidget (QObject *actObj, bool inPart, QWidget *parent, const char *name) :
    QWidget (parent, name) {
	int i;

	inPart_ = inPart;
#ifdef WITH_TSE3
	tse3Handler_ = new NTSE3Handler(this);
	connect(tse3Handler_, SIGNAL(endRecorded(bool)), this, SLOT(completeRecording(bool)));
	connect(tse3Handler_, SIGNAL(endTSE3toScore(bool)), this, SLOT(completeTSE3toScore(bool)));
#endif
	genPropDialog_ = new propFrm(this);
	lyricsDialog_ = new lyricsFrm( this );
	// ----------- KDE user interface -------------
	// Create one action for each menu entry, with text, icon and keyboard shortcut
	// The layout of the menus is described in noteeditui.rc
	// The name of the action is what is matched against the actions in the XML file.
	m_actionCollection = actObj;
	KStdAction::open( this, SLOT(fileOpen()), actionCollection() );
	KStdAction::openNew( this, SLOT(newPaper()), actionCollection() );
	KStdAction::save( this, SLOT(fileSave()), actionCollection() );
	KStdAction::saveAs( this, SLOT(fileSaveAs()), actionCollection() );
	KStdAction::undo( this, SLOT(undo()), actionCollection() );
	KStdAction::redo( this, SLOT(redo()), actionCollection() );
	KStdAction::zoomIn( this, SLOT(zoomIn()), actionCollection() );
	KStdAction::zoomOut( this, SLOT(zoomOut()), actionCollection() );
	KStdAction::preferences(this, SLOT(configure()), actionCollection() );
        m_recentFilesAction = KStdAction::openRecent( this, SLOT(fileOpenRecent(const KURL &)), actionCollection() );
        m_recentFilesAction->loadEntries( KGlobal::config() );
	new KAction( i18n("Export &MIDI..."), 0, this, SLOT(exportMidi()), actionCollection(), "export_midi" );
	new KAction( i18n("Export Musi&XTeX..."), 0, this, SLOT(exportMusiXTeX()), actionCollection(), "export_musixtex" );
	new KAction( i18n("Export &PMX..."), 0, this, SLOT(exportPMX()), actionCollection(), "export_pmx" );
	new KAction( i18n("Output Params..."), 0, this, SLOT(setOutputParam()), actionCollection(), "set_params" );
	lilyPort_ = new KAction( i18n("Export &LilyPond..."), 0, this, SLOT(exportLilyPond()), actionCollection(), "export_lily" );
	new KAction( i18n("Score in&formation..."), "readme", 0, this, SLOT(scoreInfo()), actionCollection(), "score_information" );
	new KAction( i18n("&Key configuration"), 0, this, SLOT(keyConfig()), actionCollection(), "keyconfig" );
	new KAction( i18n("Quit"), "exit", 0, this, SLOT(quitDialog()), actionCollection(), "quit" );
	new KAction( i18n("&Close all"), 0, this, SLOT(closeAllWindows()), actionCollection(), "close_all" );
	KToggleAction *actionToggleBarNumbers = new KToggleAction
	  (i18n("Show &bar numbers"), 0, this, SLOT(toggleBarNumbers()), actionCollection(), "view_show_bar_numbers");
	actionToggleBarNumbers->setChecked(NResource::showStaffNrs_);
	KToggleAction *actionToggleStaffNames = new KToggleAction
	  (i18n("Show &staff names"), 0, this, SLOT(toggleStaffNames()), actionCollection(), "view_show_staff_names");
	actionToggleStaffNames->setChecked(NResource::showStaffNames_);
	KToggleAction *actionToggleAuxLines = new KToggleAction
	  (i18n("Show &aux lines"), 0, this, SLOT(toggleAuxLines()), actionCollection(), "view_show_aux_lines");
	actionToggleAuxLines->setChecked(NResource::showAuxLines_);
	KToggleAction *actionToggleDrumToolBar = new KToggleAction
	  (i18n("Show &drum tools"), 0, this, SLOT(toggleDrumUp()), actionCollection(), "view_drum_tools");
	actionToggleDrumToolBar->setChecked(NResource::showDrumToolbar_);
	new KAction( i18n("Reduce &accidentals..."), 0, this, SLOT(redAccidentals()), actionCollection(), "tools_accidentals" );
	new KAction( i18n("&Collect notes..."), 0, this, SLOT(collChords()), actionCollection(), "tools_collect" );
	new KAction( i18n("Set all to sharp"), 0, this, SLOT(setAllSharp()), actionCollection(), "tools_set_sharp" );
	new KAction( i18n("Set all to flat"), 0, this, SLOT(setAllFlat()), actionCollection(), "tools_set_flat" );
	new KAction( i18n("&Transpose..."),  0, this, SLOT(transposeDialog()), actionCollection(), "tools_transpose" );
	new KAction( i18n("&Change Clef..."),  0, this, SLOT(changeClefDialog()), actionCollection(), "tools_change_clef" );
	criticalButtons_.append
		(new KAction( i18n("&Clef..."), "cleficon", 0, this, SLOT(clefDialog()), actionCollection(), "insert_clef"));
	criticalButtons_.append
		(new KAction( i18n("&Key signature..."), "keyicon", 0, this, SLOT(keyDialog()), actionCollection(), "insert_keysig" ));
	criticalButtons_.append
		(new KAction( i18n("&RepeatOpen"), "repopen", 0, this, SLOT(insertRepeatOpen()), actionCollection(), "insert_repeatopen"));
	criticalButtons_.append
		(new KAction( i18n("RepeatOpen&Close"), "repopenclose", 0, this, SLOT(insertRepeatOpenClose()), actionCollection(), "insert_repeatopenclose"));
	criticalButtons_.append
		(new KAction( i18n("R&epeatClose"), "repclose", 0, this, SLOT(insertRepeatClose()), actionCollection(), "insert_repeatclose"));
	criticalButtons_.append
		(new KAction( i18n("SpecialEnding&1"), 0, this, SLOT(insertspecEnding1()), actionCollection(), "insert_specialending1" ));
	criticalButtons_.append
		(new KAction( i18n("SpecialEnding&2"), 0, this, SLOT(insertspecEnding2()), actionCollection(), "insert_specialending2" ));
	criticalButtons_.append
		(new KAction( i18n("T&ime signature..."), "timeicon", 0, this, SLOT(timesigDialog()), actionCollection(), "insert_timesig" ));
	criticalButtons_.append
		(new KAction( i18n("Te&mpo signature..."), 0, this, SLOT(tempoSigDialog()), actionCollection(), "insert_temposig" ));
	criticalButtons_.append
		(new KAction( i18n("&Volume change..."), "speaker", 0, this, SLOT(volChangeDialog()), actionCollection(), "insert_volumechange"));
	criticalButtons_.append
		(new KAction( i18n("V&oice change..."), "voichange", 0, this, SLOT(voiceChangeDialog()), actionCollection(), "insert_voicechange"));
	criticalButtons_.append
		(new KAction( i18n("&Line..."), 0, this, SLOT(insertLine()), actionCollection(), "insert_line" )); // HINT: The associated function is located in mainframewidget2.cpp
	new KAction( i18n("&Multi rest..."),  0, this, SLOT(multiRestDialog()), actionCollection(), "insert_multi_rest" );
	new KAction( i18n("&New"), "filenew", 0, this, SLOT(newStaff()), actionCollection(), "staff_new" );
	new KAction( i18n("&Delete"), "editdelete", 0, this, SLOT(deleteStaff()), actionCollection(), "staff_delete" );
	new KAction( i18n("&Move..."), "editcopy", 0, this, SLOT(staffMoveDialog()), actionCollection(), "staff_move" );
	new KAction( i18n("M&ute staffs..."), 0, this, SLOT(muteDialog()), actionCollection(), "staff_mute" );
	new KAction( i18n("&Voices..."), 0, this, SLOT(voiceDialog()), actionCollection(), "voice_dialog" );
	new KAction( i18n("&Staff properties..."), "configure", 0, this, SLOT(setStaffProperties()), actionCollection(), "staff_properties" );
	criticalButtons_.append
		(new KAction( i18n("&Lyrics..."), "history", 0, this, SLOT(showLyricsDialog()), actionCollection(), "staff_lyrics" ));
#ifdef WITH_TSE3
	new KAction( i18n("&Import recording"), "folder_sound", 0, this, SLOT(importRecording()), actionCollection(), "staff_importrec" );
#endif
	new KAction( i18n("&MultiStaff"),  0, this, SLOT(multiStaffDialog()), actionCollection(), "edit_multistaff" );
	new KAction( i18n("&Goto"), "goto", 0, this, SLOT(gotoDialog()), actionCollection(), "edit_goto" );
	criticalButtons_.append
		(new KAction( i18n("&AutoBar"),  0, this, SLOT(autoBar()), actionCollection(), "edit_autobar" ));
	new KAction( i18n("Auto&beam..."),  0, this, SLOT(autoBeamDialog()), actionCollection(), "edit_autobeam" );
	new KAction( i18n("&Cleanup rests..."), 0, this, SLOT(cleanRestsDialog()), actionCollection(), "edit_cleanuprests" );
	// TODO add Redo/Cut/Copy/Paste to the Edit menu
#ifdef WITH_TSE3
	new KAction( i18n("&Create TSE3 song"), "Wizard", 0, this, SLOT(createTSE3()), actionCollection(), "tse3_createsong" );
	new KAction( i18n("&Play TSE3 song"), "1rightarrow", 0, this, SLOT(playSong()), actionCollection(), "tse3_playsong" );
	new KAction( i18n("&Read TSE3 song"), "filefind", 0, this, SLOT(readTSE3()), actionCollection(), "tse3_readsong" );
	new KAction( i18n("&Write TSE3 song"), "filesaveas", 0, this, SLOT(writeTSE3()), actionCollection(), "tse3_writesong" );
	new KAction( i18n("MIDI &out"), "midi", 0, this, SLOT(TSE3MidiOut()), actionCollection(), "tse3_midiout" );
	new KAction( i18n("MIDI &in"), "fileopen", 0, this, SLOT(TSE3MidiIn()), actionCollection(), "tse3_midiin" );
	new KAction( i18n("&Filter dialog..."), "filter", 0, this, SLOT(TSE3Filter()), actionCollection(), "tse3_filterdlg" );
	new KAction( i18n("In&formation..."), "help", 0, tse3Handler_, SLOT(printSongInformation()), actionCollection(), "tse3_information" );
	new KAction( i18n("&Merge..."), "view_right", 0, tse3Handler_, SLOT(TSE3Merge()), actionCollection(), "tse3_merge" );
	new KAction( i18n("TSE3 --> &Score"), 0, this, SLOT(TSE3toScore()), actionCollection(), "tse3_score" );
#endif
	new KAction( i18n("&New window"), "window_new", 0, this, SLOT(openNewWindow()), actionCollection(), "window_new_window" );
	new KAction( i18n("Tip of the &day"), "idea", 0, this, SLOT(showTipOfTheDay()), actionCollection(), "help_tipoftheday" );


	// ------------ Actions for toolbar buttons, KDE interface --------
	note_buttons_[0] = new KToggleAction(i18n("breve"), "breve", 0, actionCollection(), "breve");
	note_buttons_[1] = new KToggleAction(i18n("Full note"), "fullnote", 0, actionCollection(), "full");
	note_buttons_[2] = new KToggleAction(i18n("Half note"), "halfnote", 0, actionCollection(), "half");
	note_buttons_[3] = new KToggleAction(i18n("Quarter note"), "quarternote", 0, actionCollection(), "quarter");
	note_buttons_[4] = new KToggleAction(i18n("Eighth note"), "eightnote", 0, actionCollection(), "n8");
	note_buttons_[5] = new KToggleAction(i18n("Sixteenth note"), "sixteenthnote", 0, actionCollection(), "n16");
	note_buttons_[6] = new KToggleAction(i18n("32nd note"), "32ndnote", 0, actionCollection(), "n32");
	note_buttons_[7] = new KToggleAction(i18n("64th note"), "64thnote", 0, actionCollection(), "n64");
	note_buttons_[8] = new KToggleAction(i18n("128th note"), "128thnote", 0, actionCollection(), "n128");

	for ( i = 0; i < COUNT_CHORDBUTTONS; ++i)
	    note_buttons_[i]->setExclusiveGroup( "notegrp" );

	note_dymmy_ = new KToggleAction(QString::null, 0, actionCollection(), "notedummy");
	note_dymmy_->setExclusiveGroup( "notegrp" );


	dotbutton_ = new KToggleAction(i18n("Dotted note"), "dottednote", 0, actionCollection(), "dot");
	dotbutton_->setExclusiveGroup("dotgrp");
	ddotbutton_ = new KToggleAction(i18n("DoubleDotted note"), "doubledottednote", 0, actionCollection(), "ddot");
	ddotbutton_->setExclusiveGroup("dotgrp");
	tiebutton_ = new KToggleAction(i18n("Tie"), "tiednote", 0, actionCollection(), "tie");
	beambutton_ = new KToggleAction(i18n("Beam"), "beamednote", 0, actionCollection(), "beam");
	slurbutton_ = new KToggleAction(i18n("Slur"), "slurednote", 0, actionCollection(), "slur");
	tripletbutton_ = new KToggleAction(i18n("Triplet"), "tripletnotes", 0, actionCollection(), "triplet");
	hiddenrestbutton_ = new KToggleAction(i18n("Hidden rests"), "hiddenrest", 0, actionCollection(), "hrest");

	staccatobutton_ = new KToggleAction(i18n("Staccato"), "staccatonote", 0, actionCollection(), "staccato");
	staccatobutton_->setExclusiveGroup("accgrp");
	sforzatobutton_ = new KToggleAction(i18n("Sforzato"), "sforzatonote", 0, actionCollection(), "sforzato");
	sforzatobutton_->setExclusiveGroup("accgrp");
	portatobutton_ = new KToggleAction(i18n("Portato"), "portatonote", 0, actionCollection(), "portato");
	portatobutton_->setExclusiveGroup("accgrp");
	fermatebutton_ = new KToggleAction(i18n("Fermate"), "fermatenote", 0, actionCollection(), "fermate");
	fermatebutton_->setExclusiveGroup("accgrp");
	strong_pizzicatobutton_ = new KToggleAction(i18n("Strong pizzicato"), "strpizzicatonote", 0, actionCollection(), "strong_pizzicato");
	strong_pizzicatobutton_->setExclusiveGroup("accgrp");
	sforzandobutton_ = new KToggleAction(i18n("Sforzando"), "sforzandonote", 0, actionCollection(), "sforzando");
	sforzandobutton_->setExclusiveGroup("accgrp");

	offs_buttons_[0] = new KToggleAction(i18n("Sharp"), "cross", 0, actionCollection(), "sharp");
	offs_buttons_[0]->setExclusiveGroup("offsgrp");
	offs_buttons_[1] = new KToggleAction(i18n("Flat"), "flat", 0, actionCollection(), "flat");
	offs_buttons_[1]->setExclusiveGroup("offsgrp");
	offs_buttons_[2] = new KToggleAction(i18n("Natural"), "natur", 0, actionCollection(), "natural");
	offs_buttons_[2]->setExclusiveGroup("offsgrp");
	offs_buttons_[3] = new KToggleAction(i18n("Double sharp"), "doublecross", 0, actionCollection(), "dsharp");
	offs_buttons_[3]->setExclusiveGroup("offsgrp");
	offs_buttons_[4] = new KToggleAction(i18n("Double flat"), "doubleflat", 0, actionCollection(), "dflat");
	offs_buttons_[4]->setExclusiveGroup("offsgrp");


	playbutton_ = new KToggleAction(i18n("Play"), "1rightarrow", 0, actionCollection(), "play");

	// plug undo here

	kbbutton_ = new KToggleAction(i18n("Keyboard") /* ????? */, "kbicon", 0, actionCollection(), "kb");
	kbInsertButton_ = new KToggleAction(i18n("Keyboard insert") /* ????? */, "redkbicon", 0, actionCollection(), "kbi");
#ifdef WITH_TSE3
	recordButton_ = new KToggleAction(i18n("Record"), "recordicon", 0, actionCollection(), "rec");
#endif
	editbutton_ = new KToggleAction(i18n("Edit mode"), "editicon", 0, actionCollection(), "edit");
	stemUpbutton_ = new KToggleAction(i18n("Stem up"), "stemupicon", 0, actionCollection(), "stemUp");
	stemDownbutton_ = new KToggleAction(i18n("Stem down"), "stemdownicon", 0, actionCollection(), "stemDown");

	crossDrumBu_ = new KToggleAction(i18n("drum1"), "perccross", 0, actionCollection(), "cross_drum");
	crossDrumBu_->setExclusiveGroup( "note_body_group" );
	cross2DrumBu = new KToggleAction(i18n("drum2"), "perccross2", 0, actionCollection(), "cross2_drum");
	cross2DrumBu->setExclusiveGroup( "note_body_group" );
	crossCricDrumBu_ = new KToggleAction(i18n("drum3"), "perccrosscirc", 0, actionCollection(), "cross_cric_drum");
	crossCricDrumBu_->setExclusiveGroup( "note_body_group" );
	rectDrumBu_ = new KToggleAction(i18n("drum4"), "percrect", 0, actionCollection(), "rect_drum");
	rectDrumBu_->setExclusiveGroup( "note_body_group" );
	triaDrumBu_ = new KToggleAction(i18n("drum5"), "perctrian", 0, actionCollection(), "tria_drum");
	triaDrumBu_->setExclusiveGroup( "note_body_group" );

	connect(note_buttons_[0], SIGNAL(toggled(bool)), this, SLOT(setToDFull(bool)));
	connect(note_buttons_[1], SIGNAL(toggled(bool)), this, SLOT(setToFull(bool)));
	connect(note_buttons_[2], SIGNAL(toggled(bool)), this, SLOT(setToHalf(bool)));
	connect(note_buttons_[3], SIGNAL(toggled(bool)), this, SLOT(setToQuarter(bool)));
	connect(note_buttons_[4], SIGNAL(toggled(bool)), this, SLOT(setToN8(bool)));
	connect(note_buttons_[5], SIGNAL(toggled(bool)), this, SLOT(setToN16(bool)));
	connect(note_buttons_[6], SIGNAL(toggled(bool)), this, SLOT(setToN32(bool)));
	connect(note_buttons_[7], SIGNAL(toggled(bool)), this, SLOT(setToN64(bool)));
	connect(note_buttons_[8], SIGNAL(toggled(bool)), this, SLOT(setToN128(bool)));

	connect(offs_buttons_[0], SIGNAL(toggled(bool)), this, SLOT(setCross(bool)));
	connect(offs_buttons_[1], SIGNAL(toggled(bool)), this, SLOT(setFlat(bool)));
	connect(offs_buttons_[2], SIGNAL(toggled(bool)), this, SLOT(setNatur(bool)));
	connect(offs_buttons_[3], SIGNAL(toggled(bool)), this, SLOT(setDCross(bool)));
	connect(offs_buttons_[4], SIGNAL(toggled(bool)), this, SLOT(setDFlat(bool)));
	connect(&autoscrollTimer_, SIGNAL(timeout()), this, SLOT(autoscroll()));

	connect(playbutton_, SIGNAL(toggled(bool)), this, SLOT(playAll(bool)));
	connect(stemUpbutton_, SIGNAL(toggled(bool)), this, SLOT(setStemUp(bool)));
	connect(stemDownbutton_, SIGNAL(toggled(bool)), this, SLOT(setStemDown(bool)));

	connect(dotbutton_, SIGNAL(toggled(bool)), this, SLOT(setDotted(bool)));
	connect(ddotbutton_, SIGNAL(toggled(bool)), this, SLOT(setDDotted(bool)));
	connect(editbutton_, SIGNAL(toggled(bool)), this, SLOT(setEditMode(bool)));
	connect(kbbutton_, SIGNAL(toggled(bool)), this, SLOT(setKbMode(bool)));
	connect(kbInsertButton_, SIGNAL(toggled(bool)), this, SLOT(setKbInsertMode(bool)));
#ifdef WITH_TSE3
	connect(recordButton_, SIGNAL(toggled(bool)), this, SLOT(TSE3record(bool)));
#endif
	connect(tiebutton_, SIGNAL(toggled(bool)), this, SLOT(setTied(bool)));
	connect(staccatobutton_, SIGNAL(toggled(bool)), this, SLOT(setStaccato(bool)));
	connect(sforzatobutton_, SIGNAL(toggled(bool)), this, SLOT(setSforzato(bool)));
	connect(portatobutton_, SIGNAL(toggled(bool)), this, SLOT(setPortato(bool)));
	connect(strong_pizzicatobutton_, SIGNAL(toggled(bool)), this, SLOT(setStrong_pizzicato(bool)));
	connect(sforzandobutton_, SIGNAL(toggled(bool)), this, SLOT(setSforzando(bool)));
	connect(fermatebutton_, SIGNAL(toggled(bool)), this, SLOT(setFermate(bool)));
	connect(beambutton_, SIGNAL(toggled(bool)), this, SLOT(setBeamed(bool)));
	connect(slurbutton_, SIGNAL(toggled(bool)), this, SLOT(setSlured(bool)));
	connect(tripletbutton_, SIGNAL(toggled(bool)), this, SLOT(setTriplet(bool)));
	connect(hiddenrestbutton_, SIGNAL(toggled(bool)), this, SLOT(setHidden(bool)));
	connect(crossDrumBu_, SIGNAL(toggled(bool)), this, SLOT(setCrossBody(bool)));
	connect(cross2DrumBu, SIGNAL(toggled(bool)), this, SLOT(setCross2Body(bool)));
	connect(crossCricDrumBu_, SIGNAL(toggled(bool)), this, SLOT(setCrossCircBody(bool)));
	connect(rectDrumBu_, SIGNAL(toggled(bool)), this, SLOT(setRectBody(bool)));
	connect(triaDrumBu_, SIGNAL(toggled(bool)), this, SLOT(setTrianBody(bool)));
	editMode_ = false;
	notePart_ = new NDbufferWidget(this, "notepart");
	notePart_->setCursor( QCursor(crossCursor) );
	scrollx_ = new QScrollBar(QScrollBar::Horizontal, this, "scrollx");
	connect(scrollx_, SIGNAL(valueChanged(int)), this , SLOT(xscrollFromWidget(int)));
	scrolly_ = new QScrollBar(QScrollBar::Vertical, this, "scrolly");
	connect(scrolly_, SIGNAL(valueChanged(int)), this , SLOT(yscroll(int)));
	width_ = height_ = 0;
	paperScrollWidth_ = paperScrollHeight_ = 0;
	paperWidth_ = paperHeight_ = 0;
	tempo_ = DEFAULT_TEMPO * INTERNAL_LENGTH_FAC;
	lastBarNr_ = 0;
	nextStaffElemToBePainted_ = 0;
	fhandler_ = new NFileHandler();
	lilyexport_ = new NLilyExport();
	exportDialog_ = new exportFrm( this );
	setEdited(false);
	NVoice::resetUndo();
	actualOffs_ = UNDEFINED_OFFS;
	leftx_ = 0;
	topy_ = 0;
	voiceNr_ = 0;
	lastYHeight_ = 0;
	lastXpos_ = oldLastXpos_ = 0;
	staffList_.append(currentStaff_ = new NStaff(Y_STAFF_BASE +  NResource::overlength_, 0, 0, this));
	voiceList_.append(currentVoice_ = currentStaff_->getVoiceNr(0));
	enableCriticalButtons(true);
	main_props_.voiceDisplay =
	voiceDisplay_ = new NNumberDisplay(0, currentStaff_->voiceCount(), i18n("voice sel."), 0, actionCollection(), "voicenumber");
	connect(voiceDisplay_, SIGNAL(valChanged(int)), this, SLOT(changeActualVoice(int)));
	currentStaff_->setActual(true);
	lastYHeight_ = 50 + (4+LINE_OVERFLOW/2)*LINE_DIST;
	staffCount_ = 1;
	selectedSign_ = 0;
	playing_ = false;
	changeZoomValue(NResource::defZoomval_);
	computeMidiTimes(false);
	reposit();

	listDialog_ = new listFrm( this );

	zoomselect_ = new NZoomSelection(i18n("Zoom"), 0, actionCollection(), "zoomval", this);

	multistaffDialog_ = new staffFrm( this );

	staffPropFrm_ = new staffPropFrm( this );
	cleanUpRestsDialog_ = new smallestRestFrm( this );

	clefDialog_ = new staffelFrm( this );

	keyDialog_ = new QDialog();
	keyDialog_->setCaption(kapp->makeStdCaption(i18n("Key")));
	keyList_ = new QListBox(keyDialog_);
	keyOkButton_ = new QPushButton(i18n("&OK"), keyDialog_);
	keyOkButton_->setFocus();
	keyCancButton_ = new QPushButton(i18n("&Cancel"), keyDialog_);
	offs_list_[0] = new NKeyOffs("C", 0, keyDialog_, "Coffs");
	offs_list_[1] = new NKeyOffs("D", 1, keyDialog_, "Doffs");
	offs_list_[2] = new NKeyOffs("E", 2, keyDialog_, "Eoffs");
	offs_list_[3] = new NKeyOffs("F", 3, keyDialog_, "Foffs");
	offs_list_[4] = new NKeyOffs("G", 4, keyDialog_, "Goffs");
	offs_list_[5] = new NKeyOffs("A", 5, keyDialog_, "Aoffs");
	offs_list_[6] = new NKeyOffs("B", 6, keyDialog_, "Boffs");

	crosslabel_ = new QLabel(keyDialog_);
	flatlabel_ = new QLabel(keyDialog_);
	naturlabel_ = new QLabel(keyDialog_);

	crosslabel_->setPixmap(*NResource::crossIcon_);
	flatlabel_->setPixmap(*NResource::flatIcon_);
	naturlabel_->setPixmap(*NResource::naturIcon_);

	connect(keyList_, SIGNAL(highlighted(int)), this, SLOT(changeKey(int)));
	connect(keyOkButton_, SIGNAL(clicked()), this, SLOT(setInsertionKey()));
	connect(keyCancButton_, SIGNAL(clicked()), keyDialog_, SLOT(hide()));

	for (i = 0; i < 13; ++i) keyList_->insertItem(i18n(keySigTab_[i]));

	scaleFrm_ = new scaleFrm( this );
	volChangeDialog_ = new volumeFrm( this );

/*
	timesigDialog_ = new staffelFrm( this );
*/
	timesigDialog_ = new timesigDiaFrm ( this );

	keys_ = new KAccel(this);

	keys_->insertItem( i18n( "move up" ), "KEmoveup",  Key_Up);
	keys_->connectItem( "KEmoveup", this, SLOT( KE_moveUp() ) );
	keys_->insertItem( i18n( "move down" ), "KEmovedown",  Key_Down);
	keys_->connectItem( "KEmovedown", this, SLOT( KE_moveDown() ) );
	keys_->insertItem( i18n( "move left" ), "KEmoveleft",  Key_Left);
	keys_->connectItem( "KEmoveleft", this, SLOT( KE_moveLeft() ) );
	keys_->insertItem( i18n( "move start" ), "KEmovestart",  "Alt+Left");
	keys_->connectItem( "KEmovestart", this, SLOT( KE_moveStart() ) );
	keys_->insertItem( i18n( "move right" ), "KEmoveright",  Key_Right);
	keys_->connectItem( "KEmoveright", this, SLOT( KE_moveRight() ) );
	keys_->insertItem( i18n( "move end" ), "KEmoveend",  "Alt+Right");
	keys_->connectItem( "KEmoveend", this, SLOT( KE_moveEnd() ) );
	keys_->insertItem( i18n( "delete" ), "KEdelete",  Key_Delete);
	keys_->connectItem( "KEdelete", this, SLOT( KE_delete() ) );
	keys_->insertItem( i18n( "toggle edit" ), "KEedit",  Key_Insert);
	keys_->connectItem( "KEedit", this, SLOT( KE_edit() ) );
	keys_->insertItem( i18n( "insert note" ), "KEreturn",  Key_Return);
	keys_->connectItem( "KEreturn", this, SLOT( KE_return() ) );
	keys_->insertItem( i18n( "Full note" ), "KE1", Key_1);
	keys_->connectItem( "KE1", this, SLOT( KE_1() ) );
	keys_->insertItem( i18n( "Half note" ), "KE2", Key_2);
	keys_->connectItem( "KE2", this, SLOT( KE_2() ) );
	keys_->insertItem( i18n( "set triplet" ), "KE3", Key_3);
	keys_->connectItem( "KE3", this, SLOT( KE_3() ) );
	keys_->insertItem( i18n( "Quarter note" ), "KE4", Key_4);
	keys_->connectItem( "KE4", this, SLOT( KE_4() ) );
	keys_->insertItem( i18n( "Eighth note" ), "KE5", Key_5);
	keys_->connectItem( "KE5", this, SLOT( KE_5() ) );
	keys_->insertItem( i18n( "Sixteenth note" ), "KE6", Key_6);
	keys_->connectItem( "KE6", this, SLOT( KE_6() ) );
	keys_->insertItem( i18n( "32nd note" ), "KE7", Key_7);
	keys_->connectItem( "KE7", this, SLOT( KE_7() ) );
	keys_->insertItem( i18n( "64th note" ), "KE8", Key_8);
	keys_->connectItem( "KE8", this, SLOT( KE_8() ) );
	keys_->insertItem( i18n( "128th note" ), "KE9", Key_9);
	keys_->connectItem( "KE9", this, SLOT( KE_9() ) );
	keys_->insertItem( i18n( "set tied" ), "KEtie", Key_QuoteLeft);
	keys_->connectItem( "KEtie", this, SLOT( KE_tie() ) );
	keys_->insertItem( i18n( "set tied" ), "KEtie1", "Alt+AsciiTilde");
	keys_->connectItem( "KEtie1", this, SLOT( KE_tie() ) );
	keys_->insertItem( i18n( "dot" ), "KEdot", Key_Period);
	keys_->connectItem( "KEdot", this, SLOT( KE_dot() ) );
	keys_->insertItem( i18n( "flat" ), "KEflat", Key_B);
	keys_->connectItem( "KEflat", this, SLOT( KE_flat() ) );
	keys_->insertItem( i18n( "flat" ), "KEflat1", Key_Minus);
	keys_->connectItem( "KEflat1", this, SLOT( KE_flat() ) );
	keys_->insertItem( i18n( "sharp" ), "KEsharp", Key_NumberSign);
	keys_->connectItem( "KEsharp", this, SLOT( KE_sharp() ) );
	keys_->insertItem( i18n( "sharp" ), "KEsharp1", Key_Plus);
	keys_->connectItem( "KEsharp1", this, SLOT( KE_sharp() ) );
	keys_->insertItem( i18n( "natural" ), "KEnatural", Key_N);
	keys_->connectItem( "KEnatural", this, SLOT( KE_natural() ) );
	keys_->insertItem( i18n( "natural" ), "KEnatural1", "Shift+Equal");
	keys_->connectItem( "KEnatural1", this, SLOT( KE_natural() ) );
	keys_->insertItem( i18n( "set bar at" ), "KEbar", Key_Bar);
	keys_->connectItem( "KEbar", this, SLOT( KE_bar() ) );
	keys_->insertItem( i18n( "delete" ), "KEbackspace", Key_Backspace);
	keys_->connectItem( "KEbackspace", this, SLOT( KE_backspace() ) );
	keys_->insertItem( i18n( "set bar after" ), "KEtab", Key_Tab);
	keys_->connectItem( "KEtab", this, SLOT( KE_tab() ) );
	keys_->insertItem( i18n( "insert rest" ), "KEspace", Key_Space);
	keys_->connectItem( "KEspace", this, SLOT( KE_space() ) );
	keys_->insertItem( i18n( "toggle beam" ), "KEunderscore", "Shift+Underscore");
	keys_->connectItem( "KEunderscore", this, SLOT( KE_underscore() ) );
	keys_->insertItem( i18n( "keyboard insert mode" ), "KEkeybordInsert", Key_K);
	keys_->connectItem( "KEkeybordInsert", this, SLOT( KE_keybordInsert() ) );

/*------------------------- "note" keys -----------------------------------------------*/

	keys_->insertItem( i18n( "pitch C" ), "KEpitchC", "C");
	keys_->connectItem( "KEpitchC", this, SLOT( KE_pitch_C() ) );
	keys_->insertItem( i18n( "pitch D" ), "KEpitchD", "D");
	keys_->connectItem( "KEpitchD", this, SLOT( KE_pitch_D() ) );
	keys_->insertItem( i18n( "pitch E" ), "KEpitchE", "E");
	keys_->connectItem( "KEpitchE", this, SLOT( KE_pitch_E() ) );
	keys_->insertItem( i18n( "pitch F" ), "KEpitchF", "F");
	keys_->connectItem( "KEpitchF", this, SLOT( KE_pitch_F() ) );
	keys_->insertItem( i18n( "pitch G" ), "KEpitchG", "G");
	keys_->connectItem( "KEpitchG", this, SLOT( KE_pitch_G() ) );
	keys_->insertItem( i18n( "pitch A" ), "KEpitchA", "A");
	keys_->connectItem( "KEpitchA", this, SLOT( KE_pitch_A() ) );
	keys_->insertItem( i18n( "pitch B" ), "KEpitchB", "B");
	keys_->connectItem( "KEpitchB", this, SLOT( KE_pitch_B() ) );

	keys_->readSettings();

	connect(&timer_, SIGNAL(timeout()), this, SLOT(playNext()));
	x0_ = x1_ = y0_  = 0;
	note_buttons_[3]->setOn(true);
	main_props_.directPainter->setPaintDevice(notePart_);
	help_x0_ = -1;
	keyLine_ = NULL_LINE;
	keyOffs_ = 0;

	// the following code prepares the possibility of tool elements for note attribute modification.
	char *enabled = "enabled";
	toolContainer_ = new QFrame( this, "", WStyle_NoBorder );
	toolContainer_->resize( TOOL_ELEMENT_WIDTH + 8, TOOL_ELEMENT_HEIGHT );
	toolContainer_->setLineWidth( 2 );
	toolContainer_->setFrameShadow( QFrame::Raised );
	toolContainer_->setFrameShape( QFrame::WinPanel );
	tabWid_ = new QTabWidget( toolContainer_ );
	tabWid_->setGeometry( 2, 2, TOOL_ELEMENT_WIDTH + 4, TOOL_ELEMENT_HEIGHT);
	toolContainer_->hide();

	trillLengthBase_ = new QFrame( toolContainer_ );
	trillLengthBase_->setGeometry( QRect(6, 2, TOOL_ELEMENT_WIDTH, 40 ) );
	trillEnabled_ = new QCheckBox( enabled, trillLengthBase_ );
	trillEnabled_->setChecked( true );
	trillEnabled_->setGeometry( QRect( 2, 2, 100, 16 ) );
	trillLength_ = new QSlider( QSlider::Horizontal, trillLengthBase_ );
	trillLength_->setGeometry( QRect( 2, 18, TOOL_ELEMENT_WIDTH - 4, 20 ) );
	trillLength_->setMinValue( 0 );
	trillLength_->setMaxValue( 30 );

	connect( trillLength_, SIGNAL( valueChanged( int ) ), this, SLOT( trillLengthChanged( int ) ) );
	connect( trillEnabled_, SIGNAL( clicked() ), this, SLOT( trillDisabled() ) );
	
	tabWid_->insertTab( trillLengthBase_, "Trill" );
	
	dynamicBase_ = new QFrame( toolContainer_ );
	dynamicBase_->setGeometry( QRect(6, 2, TOOL_ELEMENT_WIDTH, 40 ) );
	dynamicPos_ = new QSlider( QSlider::Horizontal, dynamicBase_ );
	dynamicPos_->setGeometry( QRect( 2, 18, TOOL_ELEMENT_WIDTH - 4,20 ) );
	dynamicPos_->setMinValue( 0 );
	dynamicPos_->setMaxValue( 6000 );
	dynamicDisable_ = new QCheckBox( enabled, dynamicBase_ );
	dynamicDisable_->setGeometry( QRect( 2, 2, 100, 16 ) );
	dynamicDisable_->setChecked( true );
	dynamicAlignment_ = new QCheckBox( "turn", dynamicBase_ );
	dynamicAlignment_->setGeometry( QRect( TOOL_ELEMENT_WIDTH - 52, 2, 50, 16 ) );
	dynamicAlignment_->setChecked( true );

	connect( dynamicPos_, SIGNAL( valueChanged(int) ), this, SLOT(dynamicPosChanged( int ) ) );
	connect( dynamicDisable_, SIGNAL( clicked() ), this, SLOT( dynamicKill() ) );
	connect( dynamicAlignment_, SIGNAL( clicked() ), this, SLOT( dynamicSwitch() ) );

	tabWid_->insertTab( dynamicBase_, "Crescendo" );
	
	stopList_.setAutoDelete(false);
	nextEvents_.setAutoDelete(false);
	currentEvents_.setAutoDelete(false);
	setSaveWidth(160);
	setSaveHeight(250);
	setParamsEnabled(false);

}

NMainFrameWidget::~NMainFrameWidget() {
        m_recentFilesAction->saveEntries( KGlobal::config() );
	synchronizeRecentFiles();
#ifdef WITH_TSE3
	delete tse3Handler_;
#endif
	delete genPropDialog_;
	delete notePart_;
	delete fhandler_;
	delete lilyexport_;
	delete exportDialog_;
	delete listDialog_;
	delete zoomselect_;
	delete multistaffDialog_;
	delete staffPropFrm_;
	delete clefDialog_;
	delete keyDialog_;
	delete cleanUpRestsDialog_;
	delete volChangeDialog_;
	delete lyricsDialog_;
	delete timesigDialog_;
	delete scaleFrm_;
	delete playbutton_;
	delete stemUpbutton_;
	delete stemDownbutton_;
	delete dotbutton_;
	delete tiebutton_;
	delete beambutton_;
	delete slurbutton_;
	delete tripletbutton_;
	delete editbutton_;
	staffList_.setAutoDelete(true);
	staffList_.clear();
	delete staccatobutton_;
	delete portatobutton_;
	delete strong_pizzicatobutton_;
	delete sforzatobutton_;
	delete trillEnabled_;
	delete trillLength_;
	delete trillLengthBase_;
	delete dynamicPos_;
	delete dynamicDisable_;
	delete dynamicAlignment_;
	delete dynamicBase_;
	delete tabWid_;
	delete toolContainer_;

}

void NMainFrameWidget::synchronizeRecentFiles() {
	NMainWindow *window;
	for (window = NResource::windowList_.first(); window; window = NResource::windowList_.next()) {
		window->mainFrameWidget()->reloadRecentFileList();
	}
}

void NMainFrameWidget::reloadRecentFileList() {
        m_recentFilesAction->loadEntries( KGlobal::config() );
}

void NMainFrameWidget::setEdited(bool edited = true) {
	editiones_ = edited;
	if (inPart_) return;
	static_cast<KMainWindow *>(parentWidget())
	->setCaption(actualFname_, edited);
}

void NMainFrameWidget::scoreInfo() {
	ScoreInfoDialog scoreInfoDialog(this);
	scoreInfoDialog.exec();
}

void NMainFrameWidget::configure() {
	ConfigureDialog configureDialog(this);
	configureDialog.exec();
}

void NMainFrameWidget::plugButtons(KToolBar *toolbar) {
	int i;
	for ( i = 0; i < COUNT_CHORDBUTTONS; ++i) {
		note_buttons_[i]->plug(toolbar);
	}
	playbutton_->plug(toolbar);
	stemUpbutton_->plug(toolbar);
	stemDownbutton_->plug(toolbar);
	dotbutton_->plug(toolbar);
	tiebutton_->plug(toolbar);
	beambutton_->plug(toolbar);
	slurbutton_->plug(toolbar);
	tripletbutton_->plug(toolbar);
	for (i = 0; i < 5; ++i) {
		offs_buttons_[i]->plug(toolbar);
	}
	editbutton_->plug(toolbar);
}

void NMainFrameWidget::unPlugButtons(KToolBar *toolbar) {
	int i;
	for ( i = 0; i < COUNT_CHORDBUTTONS; ++i) {
		note_buttons_[i]->unplug(toolbar);
	}
	playbutton_->unplug(toolbar);
	stemUpbutton_->unplug(toolbar);
	stemDownbutton_->unplug(toolbar);
	dotbutton_->unplug(toolbar);
	tiebutton_->unplug(toolbar);
	beambutton_->unplug(toolbar);
	slurbutton_->unplug(toolbar);
	tripletbutton_->unplug(toolbar);
	for (i = 0; i < 5; ++i) {
		offs_buttons_[i]->unplug(toolbar);
	}
	editbutton_->unplug(toolbar);
}

void NMainFrameWidget::paintNew() {
	repaint();
}
void NMainFrameWidget::repaintAll() {
	repaint(0, 0, paperWidth_, paperHeight_, false);
}

void NMainFrameWidget::cleanupSelections() {
	if (NResource::staffSelAutobeam_) delete [] NResource::staffSelAutobeam_;
	NResource::staffSelAutobeam_ = 0;
	if (NResource::staffSelAutobar_) delete [] NResource::staffSelAutobar_;
	NResource::staffSelAutobar_ = 0;
	if (NResource::staffSelMute_) delete [] NResource::staffSelMute_;
	NResource::staffSelMute_ = 0;
}

void NMainFrameWidget::enableCriticalButtons(bool enable) {
	KAction *action;

	for (action = criticalButtons_.first(); action; action = criticalButtons_.next()) {
		action->setEnabled(enable);
	}
}

void NMainFrameWidget::processMouseEvent ( QMouseEvent * evt)  {
#define TRANSX(x) (((int) ((float) (x)/main_props_.zoom + 0.5))+leftx_-LEFT_RIGHT_BORDER)
#define TRANSY(y) (((int) ((float) (y)/main_props_.zoom + 0.5))+topy_-TOP_BOTTOM_BORDER)
#define RETRANSY(y) ((int) (main_props_.zoom * ((y)-topy_+TOP_BOTTOM_BORDER) + 0.5))
#define TRANS_RECT(x, y) QRect((int) ((float) (x)/main_props_.zoom + 0.5)+leftx_-LEFT_RIGHT_BORDER, \
			       (int) ((float) (y)/main_props_.zoom + 0.5)+topy_-TOP_BOTTOM_BORDER, \
			       (int) (2*main_props_.zoom + 0.5), (int) (2*main_props_.zoom + 0.5))
#define TRANS_RECT0(x, y) QRect((int) ((x)/main_props_.zoom + 0.5), \
				 (int) ((float) (y)/main_props_.zoom + 0.5)+topy_-TOP_BOTTOM_BORDER, \
			       (int) (2*main_props_.zoom + 0.5), (int) (2*main_props_.zoom + 0.5))

	toolContainer_->hide();

	int val, state, newXpos;
	bool playable;
	QRect p, p0;
	NVoice *voice_elem;
	NStaff *staff_elem, *source_staff;
	int i;
	QList<NMusElement> *clipboard;
	if (playing_) return;
	keyLine_ = NULL_LINE;
	if ((evt->state() & QEvent::MouseButtonPress)) return;
	switch (evt->button()) {
		case LeftButton:
			if (NResource::windowWithSelectedRegion_) {
				if (NResource::windowWithSelectedRegion_ != this) {
					NResource::windowWithSelectedRegion_->repaintAll();
				}
			}
			NResource::windowWithSelectedRegion_ = 0;
			if (NResource::staffSelMulti_) {
				delete [] NResource::staffSelMulti_;
				NResource::staffSelMulti_ = 0;
				NResource::numOfMultiStaffs_ = 0;
				
			}
			if (evt->type() == QEvent::MouseButtonDblClick) {
				selectWholeStaff();
				return;
			}
 			p = TRANS_RECT(evt->x(), evt->y());
			p0 = TRANS_RECT0(evt->x(), evt->y());
			if ((val = checkAllStaffs(p, p0, &state, &playable)) > 0) {
				if (editMode_) {
					if (playable) {
						main_props_.actualLength = val;
						setButton(NResource::noteLength2Button_(val));
						stateButtonChange(state);
					}
					else {
						setButton(-1);
						stateButtonChange(0);
						main_props_.actualLength = -1;
					}
				}
			}
			else if (editMode_) {
				setButton(-1);
				stateButtonChange(0);
				main_props_.actualLength = -1;
			}
			x0_ = TRANSX(xori_ = evt->x());
			currentStaff_->getActualVoice()->findStartElemAt(x0_);
			y0_ = currentStaff_->getBase();
			repaintAll();
			break;
		case MidButton:
				if (evt->state() == ControlButton) {
					p0 = TRANS_RECT0(evt->x(), evt->y());
					checkStaffIntersection(p0);
 					p = QRect(TRANSX(evt->x()), TRANSY(evt->y())-600, 2, 1200);
					if (currentVoice_->insertNewNoteAt(TRANSY(evt->y()), p, actualOffs_)) {
						setEdited();
					}
					computeMidiTimes(false);
					reposit();
				}
				else if (selectedSign_) {
	 				p = TRANS_RECT(evt->x(), evt->y());
					p0 = TRANS_RECT0(evt->x(), evt->y());
					checkStaffIntersection(p0);
					switch (selectedSign_) {
						case  T_KEYSIG:
						case T_CLEF:
						case T_TIMESIG:
						case TEMPO_SIGNATURE:
						case VOLUME_SIG:
						case MULTIREST:
						case PROGRAM_CHANGE:
							currentVoice_->insertTmpElemAtPosition(TRANSX(evt->x()), tmpElem_);
							setEdited();
							break;
						case DYNAMIC:
						case LNTRILL:
						case TRILL:
							int state;
							bool playable;
							setEdited();
							voice_elem = voiceList_.first();
							for (; voice_elem; voice_elem = voiceList_.next()) 
							    if (voice_elem->checkElement(TRANS_RECT(evt->x(), evt->y()), &state, &playable) > 0)
    								goto next;
							KMessageBox::sorry(this, i18n("Please choose a note!"), kapp->makeStdCaption(i18n("???")));
							selectedSign_ = 0;
							return;
							next:
							switch(selectedSign_) {
							    case TRILL:	voice_elem->pubAddUndoElement();  
									if( voice_elem->getCurrentElement()->trill_ ) {
									      if( voice_elem->getCurrentElement()->trill_ < 0 )
									        voice_elem->getCurrentElement()->trill_ =
										    -voice_elem->getCurrentElement()->trill_;
									  }
									else 
									  voice_elem->getCurrentElement()->trill_ = 4;
									break;
							    case LNTRILL: voice_elem->pubAddUndoElement();
									  if( voice_elem->getCurrentElement()->trill_ ) {
										if( voice_elem->getCurrentElement()->trill_ > 0 )
										    voice_elem->getCurrentElement()->trill_ =
											-voice_elem->getCurrentElement()->trill_;
									  }
									else
									    voice_elem->getCurrentElement()->trill_ = -3;
									break;
							    case DYNAMIC: voice_elem->pubAddUndoElement();
									  voice_elem->getCurrentElement()->dynamic_ = 10;
									  voice_elem->getCurrentElement()->dynamicAlign_ = true;
							}
							repaintAll();
							selectedSign_ = 0;
							manageToolElement();
							return;
						default:
							currentVoice_->insertAtPosition(T_SIGN, TRANSX(evt->x()), TRANSY(evt->y()),
							        selectedSign_, 0, tmpElem_);
							setEdited();
							break;
					}
					setEdited();
					selectedSign_ = 0;
					computeMidiTimes(false);
					reposit();
					repaint();
					newXpos = currentVoice_->getCurrentElement()->getXpos()+
						currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
					if (newXpos + SMALL_X_SENS_DIST > leftx_ + paperScrollWidth_) {
						scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
					}
				}
				else if (NResource::windowWithSelectedRegion_) {
					p0 = TRANS_RECT0(evt->x(), evt->y());
					checkStaffIntersection(p0);
					if (NResource::numOfMultiStaffs_) {
						if (!NResource::isGrabbed_) {
							NResource::windowWithSelectedRegion_->grabElementsAccording();
							NResource::isGrabbed_ = true;
						}
						for (i = 0, staff_elem = staffList_.first(); staff_elem && i < NResource::numOfMultiStaffs_;
							 staff_elem = staffList_.next() , i++) {
							if (NResource::staffSelMulti_[i]) {
								if (i >= NResource::windowWithSelectedRegion_->staffList_.count()) break;
								source_staff = NResource::windowWithSelectedRegion_->staffList_.at(i);
								staff_elem->pasteAtPosition(TRANSX(evt->x()), source_staff);
							}
						}
					}
					else {
						if (!NResource::isGrabbed_) {
							NResource::voiceWithSelectedRegion_->getStaff()->grabElements(NResource::voiceWithSelectedRegion_);
							NResource::isGrabbed_ = true;
						}
						setButton(-1);
						currentStaff_->pasteAtPosition(TRANSX(evt->x()), NResource::voiceWithSelectedRegion_->getStaff());
					}
					setEdited();
					computeMidiTimes(false);
					reposit();
				}
				else {
					if (main_props_.actualLength > 0) {
						p0 = TRANS_RECT0(evt->x(), evt->y());
						checkStaffIntersection(p0);
						currentVoice_->insertAtPosition(T_CHORD, TRANSX(evt->x()), TRANSY(evt->y()),
								main_props_.actualLength, actualOffs_);
						setEdited();
						computeMidiTimes(NResource::automaticBarInsertion_);
						reposit();
						newXpos = currentVoice_->getCurrentElement()->getXpos()+
								currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
						if (newXpos + SMALL_X_SENS_DIST > leftx_ + paperScrollWidth_) {
								scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
						}
					}
				}
			break;
		case RightButton:
				  if (evt->state() == ControlButton) {
					if (currentVoice_->deleteAtPosition(TRANSY(evt->y()-OVER))) {
						setEdited();
					}
					computeMidiTimes(false);
					reposit();
				  }
				  else {
					if (main_props_.actualLength > 0 && main_props_.actualLength < 3*256) {
						p0 = TRANS_RECT0(evt->x(), evt->y());
					  	checkStaffIntersection(p0);
						currentVoice_->insertAtPosition(T_REST, TRANSX(evt->x()), TRANSY(evt->y()),
								main_props_.actualLength, 0);
						setEdited();
						computeMidiTimes(false);
						reposit();
						newXpos = currentVoice_->getCurrentElement()->getXpos()+
							currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
						if (newXpos + SMALL_X_SENS_DIST > leftx_ + paperScrollWidth_) {
							scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
						}
					}
				  }
			break;
	}
}

void NMainFrameWidget::autoscroll() {
#define SENS_DIST 50
#define AUTO_SCROLL_DIST (width_ / 2)
	bool scrolled = true;
	int scrolldist;
	x1_ = TRANSX(cursor().pos().x());
	if (x1_ < leftx_ + SENS_DIST) {
		scrolldist = leftx_ > AUTO_SCROLL_DIST ? AUTO_SCROLL_DIST : leftx_;
		if (scrolldist) {
			scrollx_->setValue(leftx_ - scrolldist);
			x1_ -= scrolldist;
		}
		else {
			autoscrollTimer_.stop();
			scrolled = false;
		}
	}
	else if (x1_ > leftx_ + (int) ((float) width_ / main_props_.zoom)  - SENS_DIST) {
		scrolldist = leftx_ < lastXpos_ + AUTO_SCROLL_DIST ? AUTO_SCROLL_DIST : 0;
		if (scrolldist) {
			scrollx_->setValue(leftx_ + scrolldist);
			x1_ += AUTO_SCROLL_DIST;
		}
		else {
			autoscrollTimer_.stop();
			scrolled = false;
		}
	}
	else {
		autoscrollTimer_.stop();
		scrolled = false;
	}
	if (scrolled) {
		selRect_ = QRect(x1_ > x0_ ? x0_ : x1_, y0_, abs(x0_ - x1_), 4*LINE_DIST);
		repaint();
	}
}

void NMainFrameWidget::processMoveEvent( QMouseEvent * evt)  {
#define MINDIST 10
#define MINREGION 8
#define AUTOSCROLL_RATE 500
	if (abs(evt->x()-xori_) < MINDIST) return;
	x1_ = TRANSX(xori_ = evt->x());
	if (abs(x1_ - x0_) < MINREGION) return;
	NResource::windowWithSelectedRegion_ = this;
	NResource::voiceWithSelectedRegion_ = currentStaff_->getActualVoice();
	NResource::isGrabbed_ = false;
	NResource::voiceWithSelectedRegion_->trimmRegion(&x0_, &x1_);
	if (x1_ < leftx_ + SENS_DIST || x1_ > leftx_ + (int) ((float) width_ / main_props_.zoom) - SENS_DIST) {
		if (!autoscrollTimer_.isActive()) {
			autoscrollTimer_.start(AUTOSCROLL_RATE);
		}
	}
	selRect_ = QRect(x1_ > x0_ ? x0_ : x1_, y0_, abs(x0_ - x1_), 4*LINE_DIST);
	repaint();
}

void NMainFrameWidget::selectWholeStaff() {
	if (!currentStaff_->trimmRegionToWholeStaff(&x0_, &x1_)) return;
	NResource::windowWithSelectedRegion_ = this;
	NResource::voiceWithSelectedRegion_ = currentStaff_->getActualVoice();
	NResource::isGrabbed_ = false;
	y0_ = currentStaff_->getBase();
	selRect_ = QRect(x0_, y0_ , x1_ - x0_, 4*LINE_DIST);
	repaint();
}
	

void NMainFrameWidget::deleteBlock() {
	int i;
	NStaff *staff_elem;
	if (selRect_.width() < 10) return;
	if (NResource::numOfMultiStaffs_) {
		for (i = 0, staff_elem = staffList_.first(); staff_elem && i < NResource::numOfMultiStaffs_; staff_elem = staffList_.next(), i++) {
			if (NResource::staffSelMulti_[i]) {
				staff_elem->deleteBlocksAccording();
			}
		}
	}
	else {
		NResource::voiceWithSelectedRegion_->getStaff()->deleteBlock(NResource::voiceWithSelectedRegion_);
	}
	computeMidiTimes(false);
	reposit();
	setEdited();
	NResource::windowWithSelectedRegion_ = 0;
	repaint();
}

/*------------------------- reaction on QWidget events -------------------------------*/

void NMainFrameWidget::resizeEvent ( QResizeEvent *evt ) {
	if( !toolContainer_->isHidden() )
	    toolContainer_->move( width() - toolContainer_->width(), height() - toolContainer_->height() );
	if (!inPart_) setDrumToolbar();
	width_ = evt->size().width(); height_ = evt->size().height();
	scrollx_->setGeometry(BORDER, height()-2*BORDER-SCROLLBARHEIGHT,
			width() - 2*BORDER, SCROLLBARHEIGHT);
	setScrollableNotePage();
}


#ifdef XXX
void NMainFrameWidget::keyPressEvent ( QKeyEvent * evt) {
	QPoint curpos;
	int ydist, newXpos;
	NMusElement *elem;
	if (playing_) return;
	switch (evt->key ()) {
		case Key_Up:	if (NResource::allowKeyboardInsert_) {
				curpos = notePart_->mapFromGlobal(cursor().pos());
				if (keyLine_ == NULL_LINE) {
					ydist = TRANSY(curpos.y()) - currentStaff_->getBase();
					keyLine_ = ydist / LINE_DIST;
					keyOffs_ = 0;
				}
				if (keyOffs_) {
					keyOffs_ = 0;
				}
				else {
					keyOffs_ = 1;	
					keyLine_--;
				}
				ydist = keyLine_ * LINE_DIST + keyOffs_ * (LINE_DIST / 2 + 1);
				curpos.setY(RETRANSY(currentStaff_->getBase()+ydist));
				cursor().setPos(notePart_->mapToGlobal(curpos));
			     }
			     else {
				moveUp();
			     }
			     break;
		case Key_Down:	if (NResource::allowKeyboardInsert_) {
				curpos = notePart_->mapFromGlobal(cursor().pos());
				if (keyLine_ == NULL_LINE) {
					ydist = TRANSY(curpos.y()) - currentStaff_->getBase();
					keyLine_ = ydist / LINE_DIST;
					keyOffs_ = 0;
				}
				if (keyOffs_) {
					keyOffs_ = 0;
					++keyLine_;
				}
				else {
					keyOffs_ = 1;
				}
				ydist = keyLine_ * LINE_DIST + keyOffs_ * (LINE_DIST / 2 + 1);
				curpos.setY(RETRANSY(currentStaff_->getBase()+ydist));
				cursor().setPos(notePart_->mapToGlobal(curpos));
			      }
			      else {
				moveDown();
			      }
			      break;
		case Key_Left: if (evt->state() & AltButton) {
				  scrollx_->setValue(0);
				  break;
			       }
			       prevElement();
			       if ((elem = currentVoice_->getCurrentElement()) == 0) break;
			       if (!NResource::allowKeyboardInsert_) {
				       newXpos = currentVoice_->getCurrentElement()->getXpos();
				       if (newXpos - SMALL_X_SENS_DIST< leftx_) {
						scrollx_->setValue(leftx_ - SMALL_X_SCROLL < 0 ? 0 : leftx_ - SMALL_X_SCROLL);
				       }
				       break;
			       }
			       curpos = notePart_->mapFromGlobal(cursor().pos());
			       newXpos = currentVoice_->getCurrentElement()->getXpos();
			       if (newXpos - SMALL_X_SENS_DIST< leftx_) {
					scrollx_->setValue(leftx_ - SMALL_X_SCROLL < 0 ? 0 : leftx_ - SMALL_X_SCROLL);
			       }
			       curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
			       cursor().setPos(notePart_->mapToGlobal(curpos));
			       break;
		case Key_Right: if (evt->state() & AltButton) {
					newXpos = lastXpos_ - width_;
					if (newXpos < 0) newXpos = 0;
					scrollx_->setValue(newXpos);
				}
			       nextElement();
			       if ((elem = currentVoice_->getCurrentElement()) == 0) break;
			       if (!NResource::allowKeyboardInsert_) {
					newXpos = currentVoice_->getCurrentElement()->getXpos();
						currentVoice_->getCurrentElement()->getBbox()->width();
					if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
						scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
					}
					break;
			       }
			       curpos = notePart_->mapFromGlobal(cursor().pos());
			       newXpos = currentVoice_->getCurrentElement()->getXpos()+
						currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
			       if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
					scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
			       }
			       curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
			       cursor().setPos(notePart_->mapToGlobal(curpos));
			       break;
		case Key_Delete: deleteElem(false);
			       if (!NResource::allowKeyboardInsert_) break;
			       if ((elem = currentVoice_->getCurrentElement()) == 0) break;
			       curpos = notePart_->mapFromGlobal(cursor().pos());
			       curpos.setX((int) ((elem->getXpos()+ elem->getBbox()->width()+CUR_DIST) * main_props_.zoom));
			       cursor().setPos(notePart_->mapToGlobal(curpos));
			       break;
		case Key_Insert: editbutton_->toggle(); break;
		case Key_Return: if (!NResource::allowKeyboardInsert_ || main_props_.actualLength <= 0) break;
				curpos = notePart_->mapFromGlobal(cursor().pos());
				if (evt->state() == ControlButton) {
					if (currentVoice_->insertNewNoteAtCurrent(TRANSY(curpos.y()), actualOffs_)) {
						setEdited();
					}
				}
				else {
					currentVoice_->insertAtPosition(T_CHORD, TRANSX(curpos.x()), TRANSY(curpos.y()),
						main_props_.actualLength, actualOffs_);
					setEdited();
				}
				computeMidiTimes(false);
				reposit();
				if ((elem = currentVoice_->getCurrentElement()) == 0) break;
				newXpos = currentVoice_->getCurrentElement()->getXpos()+
						currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
				if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
					scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
				}
				curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
				cursor().setPos(notePart_->mapToGlobal(curpos));
				break;
		case Key_1: note_buttons_[1]->setOn(true);
			    setToFull(true);
			    break;
		case Key_2: note_buttons_[2]->setOn(true);
			    setToHalf(true);
			    break;
		case Key_3: tripletbutton_->toggle(); break;
		case Key_4: note_buttons_[3]->setOn(true);
			    setToQuarter(true);
			    break;
		case Key_5: note_buttons_[4]->setOn(true);
			    setToN8(true);
			    break;
		case Key_6: note_buttons_[5]->setOn(true);
			    setToN16(true);
			    break;
		case Key_7: note_buttons_[6]->setOn(true);
			    setToN32(true);
			    break;
		case Key_8: note_buttons_[7]->setOn(true);
			    setToN64(true);
			    break;
		case Key_9: note_buttons_[8]->setOn(true);
			    setToN128(true);
			    break;
		case Key_QuoteLeft:
		case Key_AsciiTilde: tiebutton_->toggle(); break;
		case Key_Period: dotbutton_->toggle(); break;
		case Key_B:
		case Key_Minus: offs_buttons_[1]->setOn(true);
			    setFlat(true);
			    break;
		case Key_NumberSign:
		case Key_Plus: offs_buttons_[0]->setOn(true);
			    setCross(true);
			    break;
		case Key_N:
		case Key_Equal: offs_buttons_[2]->setOn(true);
			    setNatur(true);
			    break;
		case Key_Bar: if (!currentVoice_->isFirstVoice()) break;
				currentVoice_->insertBarAt(cursor().pos().x()-geometry().left());
				computeMidiTimes(false);
				setEdited();
				reposit();
				repaint();
				curpos = notePart_->mapFromGlobal(cursor().pos());
				if ((elem = currentVoice_->getCurrentElement()) == 0) break;
				curpos.setX((int) ((currentVoice_->getCurrentElement()->getXpos()+
					currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST) * main_props_.zoom));
				cursor().setPos(notePart_->mapToGlobal(curpos));
				break;
		case Key_Backspace: if (NResource::windowWithSelectedRegion_) deleteBlock();
				    else deleteElem(true);
				    if (!NResource::allowKeyboardInsert_) break;
			            if ((elem = currentVoice_->getCurrentElement()) == 0) break;
			            curpos = notePart_->mapFromGlobal(cursor().pos());
			            curpos.setX((int) ((elem->getXpos()+ elem->getBbox()->width()+CUR_DIST-leftx_) * main_props_.zoom));
			            cursor().setPos(notePart_->mapToGlobal(curpos));
				    break;
		case Key_Tab:   if (!currentVoice_->isFirstVoice()) break;
				currentVoice_->insertAfterCurrent(T_SIGN, SIMPLE_BAR);
				computeMidiTimes(false);
				reposit();
				if (!NResource::allowKeyboardInsert_) break;
				if ((elem = currentVoice_->getCurrentElement()) == 0) break;
			        curpos = notePart_->mapFromGlobal(cursor().pos());
				newXpos = currentVoice_->getCurrentElement()->getXpos()+
						currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
				if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
					scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
				}
				curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
				cursor().setPos(notePart_->mapToGlobal(curpos));
				break;
		case Key_Space: if (!NResource::allowKeyboardInsert_ || main_props_.actualLength <= 0) break;
				curpos = notePart_->mapFromGlobal(cursor().pos());
				currentVoice_->insertAtPosition(T_REST, TRANSX(curpos.x()), TRANSY(curpos.y()),
					main_props_.actualLength, actualOffs_);
				setEdited();
				computeMidiTimes(false);
				reposit();
				if ((elem = currentVoice_->getCurrentElement()) == 0) break;
				newXpos = currentVoice_->getCurrentElement()->getXpos()+
						currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
				if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
					scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
				}
				curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
				cursor().setPos(notePart_->mapToGlobal(curpos));
				break;
		case Key_Underscore: beambutton_->toggle();
				break;
	}
}
#else
/*--------------------------- reaction on Ok button of the dialogs above ----------------*/

void NMainFrameWidget::KE_moveUp() {
	if (playing_) return;
	QPoint curpos;
	int ydist;
	if (NResource::allowKeyboardInsert_) {
		curpos = notePart_->mapFromGlobal(cursor().pos());
		if (keyLine_ == NULL_LINE) {
			ydist = TRANSY(curpos.y()) - currentStaff_->getBase();
			keyLine_ = ydist / LINE_DIST;
			keyOffs_ = 0;
		}
		if (keyOffs_) {
			keyOffs_ = 0;
		}
		else {
			keyOffs_ = 1;	
			keyLine_--;
		}
		ydist = keyLine_ * LINE_DIST + keyOffs_ * (LINE_DIST / 2 + 1);
		curpos.setY(RETRANSY(currentStaff_->getBase()+ydist));
		cursor().setPos(notePart_->mapToGlobal(curpos));
		return;
	}
	moveUp();
}

void NMainFrameWidget::KE_moveDown() {
	if (playing_) return;
	QPoint curpos;
	int ydist;
	if (NResource::allowKeyboardInsert_) {
		curpos = notePart_->mapFromGlobal(cursor().pos());
		if (keyLine_ == NULL_LINE) {
			ydist = TRANSY(curpos.y()) - currentStaff_->getBase();
			keyLine_ = ydist / LINE_DIST;
			keyOffs_ = 0;
		}
		if (keyOffs_) {
			keyOffs_ = 0;
			++keyLine_;
		}
		else {
			keyOffs_ = 1;
		}
		ydist = keyLine_ * LINE_DIST + keyOffs_ * (LINE_DIST / 2 + 1);
		curpos.setY(RETRANSY(currentStaff_->getBase()+ydist));
		cursor().setPos(notePart_->mapToGlobal(curpos));
		return;
	}
	moveDown();
}

void NMainFrameWidget::KE_moveLeft() {
	if (playing_) return;
	NMusElement *elem;
	int newXpos;
	QPoint curpos;
	prevElement();
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	if (!NResource::allowKeyboardInsert_) {
		newXpos = currentVoice_->getCurrentElement()->getXpos();
		if (newXpos - SMALL_X_SENS_DIST< leftx_) {
				scrollx_->setValue(leftx_ - SMALL_X_SCROLL < 0 ? 0 : leftx_ - SMALL_X_SCROLL);
		}
		return;
	}
	curpos = notePart_->mapFromGlobal(cursor().pos());
	newXpos = currentVoice_->getCurrentElement()->getXpos();
	if (newXpos - SMALL_X_SENS_DIST< leftx_) {
		scrollx_->setValue(leftx_ - SMALL_X_SCROLL < 0 ? 0 : leftx_ - SMALL_X_SCROLL);
	}
	curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)),
	cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_moveStart() {
	if (playing_) return;
	scrollx_->setValue(0);
}
void NMainFrameWidget::KE_moveEnd() {
	if (playing_) return;
	int newXpos;
	newXpos = lastXpos_ - width_;
	if (newXpos < 0) newXpos = 0;
	scrollx_->setValue(newXpos);
}

void NMainFrameWidget::KE_moveRight() {
	if (playing_) return;
	NMusElement *elem;
	int newXpos;
	QPoint curpos;

	nextElement();
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	if (!NResource::allowKeyboardInsert_) {
		newXpos = currentVoice_->getCurrentElement()->getXpos();
		currentVoice_->getCurrentElement()->getBbox()->width();
		if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
			scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
		}
		return;
	}
	curpos = notePart_->mapFromGlobal(cursor().pos());
	newXpos = currentVoice_->getCurrentElement()->getXpos()+
			currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
	if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
		scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
	}
	curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)), cursor().setPos(notePart_->mapToGlobal(curpos));
}
void NMainFrameWidget::KE_delete() {
	if (playing_) return;
	NMusElement *elem;
	QPoint curpos;
	deleteElem(false);
	if (!NResource::allowKeyboardInsert_) return;
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	curpos = notePart_->mapFromGlobal(cursor().pos());
	curpos.setX((int) ((elem->getXpos()+ elem->getBbox()->width()+CUR_DIST) * main_props_.zoom));
	cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_edit() {
	if (playing_) return;
	editbutton_->toggle();
}

void NMainFrameWidget::KE_return() {
	if (playing_) return;
	NMusElement *elem;
	int newXpos;
	QPoint curpos;
	if (!NResource::allowKeyboardInsert_ || main_props_.actualLength <= 0) return;
	curpos = notePart_->mapFromGlobal(cursor().pos());
/*
	if (evt->state() == ControlButton) {
		if (currentVoice_->insertNewNoteAtCurrent(TRANSY(curpos.y()), actualOffs_)) {
			setEdited();
		}
	}
	else {
*/
		currentVoice_->insertAtPosition(T_CHORD, TRANSX(curpos.x()), TRANSY(curpos.y()),
			main_props_.actualLength, actualOffs_);
		setEdited();
/*
	}
*/
	computeMidiTimes(true);
	reposit();
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	newXpos = currentVoice_->getCurrentElement()->getXpos()+
	          currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
	if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
			scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
	}
	curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)), cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_1() {
	if (playing_) return;
	note_buttons_[1]->setOn(true);
	setToFull(true);
}
void NMainFrameWidget::KE_2() {
	if (playing_) return;
	note_buttons_[2]->setOn(true);
	setToHalf(true);
}
void NMainFrameWidget::KE_3() {
	if (playing_) return;
	tripletbutton_->toggle();
}
void NMainFrameWidget::KE_4() {
	if (playing_) return;
	note_buttons_[3]->setOn(true);
	setToQuarter(true);
}
void NMainFrameWidget::KE_5() {
	if (playing_) return;
	note_buttons_[4]->setOn(true);
	setToN8(true);
}
void NMainFrameWidget::KE_6() {
	if (playing_) return;
	note_buttons_[5]->setOn(true);
	setToN16(true);
}
void NMainFrameWidget::KE_7() {
	if (playing_) return;
	note_buttons_[6]->setOn(true);
	setToN32(true);
}
void NMainFrameWidget::KE_8() {
	if (playing_) return;
	note_buttons_[7]->setOn(true);
	setToN64(true);
}
void NMainFrameWidget::KE_9() {
	if (playing_) return;
	note_buttons_[8]->setOn(true);
	setToN128(true);
}
void NMainFrameWidget::KE_tie() {
	if (playing_) return;
	tiebutton_->toggle();
}
void NMainFrameWidget::KE_dot() {
	if (playing_) return;
	dotbutton_->toggle();
}
void NMainFrameWidget::KE_flat() {
	if (playing_) return;
	offs_buttons_[1]->setOn(true);
	setFlat(true);
}
void NMainFrameWidget::KE_sharp() {
	if (playing_) return;
	offs_buttons_[0]->setOn(true);
	setCross(true);
}
void NMainFrameWidget::KE_natural() {
	if (playing_) return;
	offs_buttons_[2]->setOn(true);
	setNatur(true);
}
void NMainFrameWidget::KE_bar() {
	if (playing_) return;
	if (!currentVoice_->isFirstVoice()) return;
	QPoint curpos;
	NMusElement *elem;
	currentVoice_->insertBarAt(cursor().pos().x()-geometry().left());
	computeMidiTimes(false);
	setEdited();
	reposit();
	repaint();
	curpos = notePart_->mapFromGlobal(cursor().pos());
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	curpos.setX((int) ((currentVoice_->getCurrentElement()->getXpos()+
			currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST) * main_props_.zoom));
	cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_backspace() {
	if (playing_) return;
	QPoint curpos;
	NMusElement *elem;
	int newXpos;
	if (NResource::windowWithSelectedRegion_) deleteBlock();
	else deleteElem(true);
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	newXpos = elem->getXpos();
	if (newXpos - SMALL_X_SENS_DIST< leftx_) {
		scrollx_->setValue(leftx_ - SMALL_X_SCROLL < 0 ? 0 : leftx_ - SMALL_X_SCROLL);
	}
	if (!NResource::allowKeyboardInsert_) return;
	curpos = notePart_->mapFromGlobal(cursor().pos());
	curpos.setX((int) ((elem->getXpos()+ elem->getBbox()->width()+CUR_DIST-leftx_) * main_props_.zoom));
	cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_tab() {
	if (playing_) return;
	if (!currentVoice_->isFirstVoice()) return;
	NMusElement *elem;
	QPoint curpos;
	int newXpos;
	currentVoice_->insertAfterCurrent(T_SIGN, SIMPLE_BAR);
	computeMidiTimes(false);
	reposit();
	if (!NResource::allowKeyboardInsert_) return;
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	curpos = notePart_->mapFromGlobal(cursor().pos());
	newXpos = currentVoice_->getCurrentElement()->getXpos()+
			currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
	if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
		scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
	}
	curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)), cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_space() {
	if (playing_) return;
	int newXpos;
	if (!NResource::allowKeyboardInsert_ || main_props_.actualLength <= 0) return;
	NMusElement *elem;
	QPoint curpos;
	curpos = notePart_->mapFromGlobal(cursor().pos());
	currentVoice_->insertAtPosition(T_REST, TRANSX(curpos.x()), TRANSY(curpos.y()),
			main_props_.actualLength, actualOffs_);
	setEdited();
	computeMidiTimes(false);
	reposit();
	if ((elem = currentVoice_->getCurrentElement()) == 0) return;
	newXpos = currentVoice_->getCurrentElement()->getXpos()+
			currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
	if (newXpos + SMALL_X_SENS_DIST> leftx_ + paperScrollWidth_) {
			scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
	}
	curpos.setX((int) ((newXpos-leftx_) * main_props_.zoom)), cursor().setPos(notePart_->mapToGlobal(curpos));
}

void NMainFrameWidget::KE_underscore() {
	if (playing_) return;
	beambutton_->toggle();
}
void NMainFrameWidget::KE_keybordInsert() {
	if (playing_) return;
	if (NResource::allowKeyboardInsert_) {
		NResource::allowKeyboardInsert_ = false;
	}
	else {
		NResource::allowKeyboardInsert_ = true;
	}
}

void NMainFrameWidget::KE_pitch_C() {
	pitchToLine(0);
}
void NMainFrameWidget::KE_pitch_D() {
	pitchToLine(1);
}
void NMainFrameWidget::KE_pitch_E() {
	pitchToLine(2);
}
void NMainFrameWidget::KE_pitch_F() {
	pitchToLine(3);
}
void NMainFrameWidget::KE_pitch_G() {
	pitchToLine(4);
}
void NMainFrameWidget::KE_pitch_A() {
	pitchToLine(5);
}
void NMainFrameWidget::KE_pitch_B() {
	pitchToLine(6);
}
#endif

void NMainFrameWidget::pitchToLine(int pitchNumber) {
	if (playing_) return;
	int halfLines, offs;
	int newXpos;
	int status;
	NChord *newchord;
	QPoint curpos;
	int ydist;
	curpos = notePart_->mapFromGlobal(cursor().pos());
	if (keyLine_ == NULL_LINE) {
		ydist = TRANSY(curpos.y()) - currentStaff_->getBase();
		keyLine_ = ydist / LINE_DIST;
		keyOffs_ = 0;
	}
	halfLines = currentStaff_->findLineOf(pitchNumber, 8 - (2*keyLine_ - keyOffs_), TRANSX(curpos.x()));
	keyLine_ = (8 - halfLines) / 2;
	keyOffs_ = (8 - halfLines) % 2;
	ydist = keyLine_ * LINE_DIST + keyOffs_ * (LINE_DIST / 2 + 1);
	if (kbbutton_->isOn()) {
		offs = currentStaff_->actualKeysig_.computeOffs(halfLines);
		if (NResource::allowInsertEcho_) {
			NResource::mapper_->playImmediately(&(currentStaff_->actualClef_), 
				halfLines, offs, currentStaff_->getVoice(), currentStaff_->transpose_);
		}
		if (kbInsertButton_->isOn()) {
			status = 0;
			if (main_props_.tied) status |= STAT_TIED;
			if (main_props_.staccato) status |= STAT_STACC;
			if (main_props_.sforzato) status |= STAT_SFORZ;
			if (main_props_.portato) status |= STAT_PORTA;
			if (main_props_.strong_pizzicato) status |= STAT_STPIZ;
			if (main_props_.sforzando) status |= STAT_SFZND;
			if (main_props_.fermate) status |= STAT_FERMT;
			status |= (main_props_.dotcount & DOT_MASK);
			status |= (main_props_.noteBody & BODY_MASK);
			newchord = new  NChord(&main_props_, currentStaff_->getStaffPropsAddr(), halfLines, offs, main_props_.actualLength, currentVoice_->stemPolicy_, status);
			if (!currentVoice_->insertAfterCurrent(newchord)) return;
			setEdited();
			computeMidiTimes(true);
			reposit();
			newXpos = currentVoice_->getCurrentElement()->getXpos()+ currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
			if (newXpos + SMALL_X_SENS_DIST > leftx_ + paperScrollWidth_) {
				scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
				return;
			}
			repaint();
		}
		return;
	}
	if (!NResource::allowKeyboardInsert_) return;
	curpos.setY(RETRANSY(currentStaff_->getBase()+ydist));
	cursor().setPos(notePart_->mapToGlobal(curpos));
}


void NMainFrameWidget::paintNextStaff() {
	if (nextStaffElemToBePainted_->getBase() > boty_ ) {
		nextStaffElemToBePainted_ = 0;
		staffPainting_ = false;
		return;
	}
	while (nextStaffElemToBePainted_->getBase() < topy_) {
		if (staffList_.find(nextStaffElemToBePainted_) == -1) {
			printf("staff not found\n");fflush(stdout);
		}
		nextStaffElemToBePainted_ = staffList_.next();
		if (!nextStaffElemToBePainted_) {
			staffPainting_ = false;
			return;
		}
	}
	nextStaffElemToBePainted_->draw(newLeft_, newRight_);
	if (staffList_.find(nextStaffElemToBePainted_) == -1) {
		printf("staff not found\n");fflush(stdout);
	}
	nextStaffElemToBePainted_ = staffList_.next();
	staffPainting_ = false;
}

void NMainFrameWidget::addVoice(NVoice *voice, int actualVoiceNr, int numVoices) {
	voiceList_.append(voice);
	voiceDisplay_->setMax(numVoices);
	voiceDisplay_->setVal(actualVoiceNr+1);
	currentVoice_ = voice;
	enableCriticalButtons(false);
	setEdited();
}

void NMainFrameWidget::removeVoice(NVoice *voice, NVoice *newCurrentVoice, int actualVoiceNr, int numVoices) {
	if (voiceList_.find(voice) == -1) {
		NResource::abort("NMainFrameWidget::removeVoice: internal error");
	}
	voiceList_.remove();
	currentVoice_ = newCurrentVoice;
	enableCriticalButtons(currentVoice_->isFirstVoice());
	voiceDisplay_->setMax(numVoices);
	voiceDisplay_->setVal(actualVoiceNr+1);
	setEdited();
}


void NMainFrameWidget::paintEvent( QPaintEvent * ) {
	NStaff *staff_elem;

	if (playing_) {
		notePart_->flip();
		return;
	}
	main_props_.p->begin( notePart_->acShowPixmap());
	main_props_.p->setBrush(NResource::backgroundBrush);
	main_props_.p->setPen(NResource::noPen);
	main_props_.p->setPen(NResource::blackPen); //  color of border around drawing area
	main_props_.p->drawRect(0, 0, paperWidth_, paperHeight_);
	main_props_.p->end();
	if (NResource::windowWithSelectedRegion_ == this) {
		main_props_.tp->beginTranslated( );
		main_props_.tp->fillRect(selRect_, NResource::selectionBackgroundBrush);
		main_props_.tp->end();
	}
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		if (staff_elem->getBase() < topy_ || staff_elem->getBase() > boty_) continue;
		staff_elem->draw(leftx_, leftx_ + (int) ((float) nettoWidth_ / main_props_.zoom));
	}
	notePart_->setMouseTracking(false);
	restoreOverlengthAera();
	notePart_->flip();
	if (leftx_ + lastXpos_ < (int) ((float) nettoWidth_ / main_props_.zoom)) {
		scrollx_->setRange ( 0, 0);
	}
	else if (abs(oldLastXpos_ - lastXpos_) > (int) ((float) nettoWidth_ / main_props_.zoom) / 4 || lastXpos_ >= (int) ((float) nettoWidth_ / main_props_.zoom) - SMALL_X_SCROLL) {
		scrollx_->setRange ( 0, lastXpos_);
	}
	notePart_->setMouseTracking(NResource::showAuxLines_);
}

/*-------------------------- reaction on pushbutton events ----------------------------- */

void NMainFrameWidget::setToDFull(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*256;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToFull(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*128;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}

}
void NMainFrameWidget::setToHalf(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*64;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToQuarter(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*32;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToN8(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*16;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToN16(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*8;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToN32(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*4;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToN64(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*2;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}
void NMainFrameWidget::setToN128(bool on) {
	if (playing_) return;
	if (!on) {
		main_props_.actualLength = -1;
		return;
	}
	main_props_.actualLength = 3*1;
	if (editMode_) {
		currentVoice_->changeActualChord();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::setCrossBody(bool on) {
	if (playing_) return;
	if (on) {
		main_props_.noteBody = STAT_BODY_CROSS;
	}
	else {
		main_props_.noteBody &= (~STAT_BODY_CROSS);
	}
	if (editMode_) {
		currentVoice_->changeBodyOfActualElement();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::setCross2Body(bool on) {
	if (playing_) return;
	if (on) {
		main_props_.noteBody = STAT_BODY_CROSS2;
	}
	else {
		main_props_.noteBody &= (~STAT_BODY_CROSS2);
	}
	if (editMode_) {
		currentVoice_->changeBodyOfActualElement();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::setCrossCircBody(bool on) {
	if (playing_) return;
	if (on) {
		main_props_.noteBody = STAT_BODY_CIRCLE_CROSS;
	}
	else {
		main_props_.noteBody &= (~STAT_BODY_CIRCLE_CROSS);
	}
	if (editMode_) {
		currentVoice_->changeBodyOfActualElement();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::setRectBody(bool on) {
	if (playing_) return;
	if (on) {
		main_props_.noteBody = STAT_BODY_RECT;
	}
	else {
		main_props_.noteBody &= (~STAT_BODY_RECT);
	}
	if (editMode_) {
		currentVoice_->changeBodyOfActualElement();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::setTrianBody(bool on) {
	if (playing_) return;
	if (on) {
		main_props_.noteBody = STAT_BODY_TRIA;
	}
	else {
		main_props_.noteBody &= (~STAT_BODY_TRIA);
	}
	if (editMode_) {
		currentVoice_->changeBodyOfActualElement();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
	if (NResource::windowWithSelectedRegion_) {
		NResource::windowWithSelectedRegion_ = 0;
		repaint();
	}
}

void NMainFrameWidget::toggleDrumUp() {
	NResource::showDrumToolbar_ = not NResource::showDrumToolbar_;
	setDrumToolbar();
}

void NMainFrameWidget::setDrumToolbar() {
	NMainWindow *mainWindow = static_cast<NMainWindow *>(parentWidget());
	KToolBar * tb = static_cast<KToolBar *>(mainWindow->child("drum_toolbar", "KToolBar" ) );
        if (NResource::showDrumToolbar_)  {
		tb->show();
	}
	else {
		tb->hide();
		main_props_.noteBody &= (~BODY_MASK);
	}
}


void NMainFrameWidget::playAll(bool on) {
	NMidiEventStr *m_events;
	int min_time = (1 << 30);
	int last_time;
	int start_time;
	NVoice *voice_elem;
	NStaff *staff_elem;
	int midipos;

	if (playing_) {
		playStop_ = true;
		return;
	}
	if (!on) return;
	if (NResource::mapper_->isInUse_) {
		KMessageBox::sorry(this, i18n("MIDI mapper is already in use!"), kapp->makeStdCaption(i18n("Play")));
		playButtonReset();
		return;
	}
	notesToPlay_ = 0;
	playStop_ = false;

	notePart_->setMouseTracking(false);
	restoreOverlengthAera();
	currentEvents_.clear();
	nextEvents_.clear();
	stopList_.clear();
	NResource::mapper_->openDevice();
	start_time = currentVoice_->getMidiTime();
	midipos = currentVoice_->getMidiPos();
	turnOverOffset_ = (int) ((double) NResource::turnOverPoint / main_props_.zoom);
	if (midipos < leftx_ || midipos > leftx_ + paperScrollWidth_) {
		scrollx_->setValue(currentVoice_->getMidiPos());
	}
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		staff_elem->startPlaying(start_time);
	}
	repaint();
	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		if (voice_elem->muted_) continue;
		m_events = voice_elem->getNextMidiEvent(0, false);
		if (m_events) {
			currentEvents_.append(m_events);
			++notesToPlay_;
		}
		if (m_events && m_events->ev_time < min_time) min_time = m_events->ev_time;
	}
	if (!notesToPlay_) {
		for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
			voice_elem->stopPlaying();
		}
		playButtonReset();
		KMessageBox::sorry(this, i18n("nothing to play"), kapp->makeStdCaption(i18n("Play")));
		notePart_->setMouseTracking(NResource::showAuxLines_);
		return;
	}
	preparePixmaps();

	last_time = myTime_ = min_time;
	for (m_events = currentEvents_.first(); m_events; m_events = currentEvents_.next()) {
		if (m_events->ev_time == min_time)  {
			m_events->from->skipChord();
			m_events->notehalt->ev_time = min_time + m_events->length;
			stopList_.append(m_events->notehalt);
		}
	}
	min_time = (1 << 30);
	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		if (voice_elem->muted_) continue;
		m_events = voice_elem->getNextMidiEvent(myTime_+1, false);
		if (m_events) {
			nextEvents_.append(m_events);
		}
		if (m_events && m_events->ev_time < min_time) min_time = m_events->ev_time;
	}
	for (m_events = stopList_.first(); m_events; m_events = stopList_.next()) {
		nextEvents_.append(m_events);
		if (m_events->ev_time < min_time) min_time = m_events->ev_time;
	}
	notesToPlay_ = 0;
	for (m_events = nextEvents_.first(); m_events; m_events = nextEvents_.next()) {
		if (m_events->ev_time == min_time)  {
			if (m_events->midi_cmd == MNOTE_OFF) {
				stopList_.find(m_events);
				stopList_.remove();
			}
			else  {
				m_events->from->skipChord();
				m_events->notehalt->ev_time = min_time + m_events->length;
				stopList_.append(m_events->notehalt);
			}
			++notesToPlay_;
		}
	}
	playing_ = true;
#ifdef WITH_TSE3
	kbbutton_->setOn(false);
#endif
	myTime_ = min_time;
	nextToPlay_ = &nextEvents_;
	nextToSearch_ = &currentEvents_;
	NResource::mapper_->isInUse_ = true;
	NResource::mapper_->setPaintDevice(notePart_);
	NResource::mapper_->play_list(&currentEvents_, last_time);
	timer_.start((int) ((double) ((myTime_-last_time) * SECONDS_PER_MINUTE * INTERNAL_LENGTH_FAC ) / (MYPPQN * tempo_)), true);
	tempofactor_ = 1;
	staffPainting_ = false;
}

void NMainFrameWidget::setDotted(bool dotted) {
	if (playing_) return;
	main_props_.dotcount = dotted ? 1 : 0;
	if (editMode_) {
		currentVoice_->setDotted();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setDDotted(bool ddotted) {
	if (playing_) return;
	main_props_.dotcount = ddotted ? 2 : 0;
	if (editMode_) {
		currentVoice_->setDotted();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setTied(bool tied) {
	if (playing_) return;
	main_props_.tied = tied;
	if (editMode_) {
		currentVoice_->setTied();
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setBeamed(bool beamed) {
	if (!beamed) {
		currentVoice_->breakBeames();
		repaint();
		setEdited();
		return;
	}
	if (!NResource::windowWithSelectedRegion_) return;
	NResource::voiceWithSelectedRegion_->setBeamed();
	if (!editMode_) {
		beambutton_->setOn(false);
	}
	repaint();
	setEdited();
}

void NMainFrameWidget::setStaccato(bool val)         { this->forceAccent(STAT_STACC, val); }
void NMainFrameWidget::setSforzato(bool val)         { this->forceAccent(STAT_SFORZ, val); }
void NMainFrameWidget::setPortato(bool val)          { this->forceAccent(STAT_PORTA, val); }
void NMainFrameWidget::setStrong_pizzicato(bool val) { this->forceAccent(STAT_STPIZ, val); }
void NMainFrameWidget::setSforzando(bool val)        { this->forceAccent(STAT_SFZND, val); }
void NMainFrameWidget::setFermate(bool val)          { this->forceAccent(STAT_FERMT, val); }

void NMainFrameWidget::setHidden(bool val) {
	main_props_.hidden = val;
}

void NMainFrameWidget::forceAccent(unsigned int acc, bool val) {
	if (playing_) return;
	main_props_.staccato = main_props_.sforzato = main_props_.portato = 
	main_props_.strong_pizzicato = main_props_.sforzando = main_props_.fermate = false;
	switch (acc){
		case STAT_STACC: main_props_.staccato         = val; break;
		case STAT_SFORZ: main_props_.sforzato         = val; break;
		case STAT_PORTA: main_props_.portato  	      = val; break;
		case STAT_STPIZ: main_props_.strong_pizzicato = val; break;
		case STAT_SFZND: main_props_.sforzando        = val; break;
		case STAT_FERMT: main_props_.fermate	      = val; break;
	}
	if (editMode_) {
		currentVoice_->setAccent(acc);
		setEdited();
		repaint();
	}
}

void NMainFrameWidget::manageToolElement( ) {

    int elcnt = 0;
    NMusElement *elem;

    elem = currentVoice_->getCurrentElement();
    if( elem && elem->trill_ ) {
	bool isneg = false;
	if( currentVoice_->getCurrentElement()->trill_ < 0 ) {
	    currentVoice_->getCurrentElement()->trill_ = -currentVoice_->getCurrentElement()->trill_;
	    isneg = true;
	}
	trillLength_->setValue( currentVoice_->getCurrentElement()->trill_ );

	if( isneg )
	    currentVoice_->getCurrentElement()->trill_ = -currentVoice_->getCurrentElement()->trill_;
	++elcnt;
	tabWid_->setTabEnabled( trillLengthBase_, true );
    }
    else
        tabWid_->setTabEnabled( trillLengthBase_, false );
    

    if( elem && elem->dynamic_ ) {
	dynamicPos_->setValue( currentVoice_->getCurrentElement()->dynamic_ );
	tabWid_->setTabEnabled( dynamicBase_, true );
	++elcnt;
    }
    else 
      tabWid_->setTabEnabled( dynamicBase_, false );

    if( elcnt ) {
	toolContainer_->move(width()-toolContainer_->width(), height()-TOOL_ELEMENT_HEIGHT);
	toolContainer_->show();
    }
    else
	toolContainer_->hide();
    
}

void NMainFrameWidget::setSlured(bool slured) {
	if (!slured) {
		currentVoice_->resetSlured();
		repaint();
		setEdited();
		return;
	}
	if (!NResource::windowWithSelectedRegion_) return;
	NResource::voiceWithSelectedRegion_->setSlured();
	repaint();
	setEdited();
}


void NMainFrameWidget::setTriplet(bool triplet) {
	if (!triplet) {
		currentVoice_->breakTriplet();
		computeMidiTimes(false);
		reposit();
		repaint();
		setEdited();
		return;
	}
	if (!NResource::windowWithSelectedRegion_) return;
	NResource::voiceWithSelectedRegion_->setTriplet();
	computeMidiTimes(false);
	reposit();
	repaint();
	setEdited();
}

void NMainFrameWidget::changeActualVoice(int voiceNr) {
	if (voiceNr < 0 || voiceNr > currentStaff_->voiceCount()) {
		NResource::abort("changeActualVoice: internal error");
	}
	currentVoice_ = currentStaff_->changeActualVoice(voiceNr-1);
	NResource::windowWithSelectedRegion_ = 0;
	repaint();
	enableCriticalButtons(currentVoice_->isFirstVoice());
}

void NMainFrameWidget::setCross(bool on) {
	if (playing_) return;
	actualOffs_ = on ? 1 : UNDEFINED_OFFS;
	if (editMode_) {
		currentVoice_->changeActualOffs(actualOffs_);
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setFlat(bool on) {
	if (playing_) return;
	actualOffs_ = on ? -1 : UNDEFINED_OFFS;
	if (editMode_) {
		computeMidiTimes(false);
		currentVoice_->changeActualOffs(actualOffs_);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setDCross(bool on) {
	if (playing_) return;
	actualOffs_ = on ? 2 : UNDEFINED_OFFS;
	if (editMode_) {
		currentVoice_->changeActualOffs(actualOffs_);
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setDFlat(bool on) {
	if (playing_) return;
	actualOffs_ = on ? -2 : UNDEFINED_OFFS;
	if (editMode_) {
		computeMidiTimes(false);
		currentVoice_->changeActualOffs(actualOffs_);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setNatur(bool on) {
	if (playing_) return;
	actualOffs_ = on ? 0 : UNDEFINED_OFFS;
	if (editMode_) {
		currentVoice_->changeActualOffs(actualOffs_);
		computeMidiTimes(false);
		reposit();
		setEdited();
	}
}

void NMainFrameWidget::setStemUp(bool on) {
	if (on && stemDownbutton_->isOn()) {
		stemDownbutton_->setOn(false);
	}
	if (playing_) return;
	if (!on) {
		main_props_.actualStemDir = STEM_DIR_AUTO;
		return;
	}
	main_props_.actualStemDir = STEM_DIR_UP;
	if (editMode_) {
		currentVoice_->changeActualStem();
		setEdited();
	}
	repaint();
}

void NMainFrameWidget::setStemDown(bool on) {
	if (on && stemUpbutton_->isOn()) {
		stemUpbutton_->setOn(false);
	}
	if (playing_) return;
	if (!on) {
		main_props_.actualStemDir = STEM_DIR_AUTO;
		return;
	}
	main_props_.actualStemDir = STEM_DIR_DOWN;
	if (editMode_) {
		currentVoice_->changeActualStem();
		setEdited();
	}
	repaint();
}

// Seems unused
/*void NMainFrameWidget::offsButtonsSet(int nr) {
	if (nr < 2 && nr >= 0) {
		offsgrp_->setButton(nr);
	}
}*/

void NMainFrameWidget::setEditMode(bool on) {
	editMode_ = on;
	int state, val, i;
	bool playable;
	if (on) {
		val = currentVoice_->getElemState(&state, &playable);
		if (playable) {
			setButton(NResource::noteLength2Button_(val));
			stateButtonChange(state);
		}
	}
	else {
		stemUpbutton_->setOn(false);
		stemDownbutton_->setOn(false);
		main_props_.actualStemDir = STEM_DIR_AUTO;
		for (i = 0; i < 5; ++i) {
			offs_buttons_[i]->setOn(false);
		}
		actualOffs_ = UNDEFINED_OFFS;
	}
}

void NMainFrameWidget::setKbMode(bool on) {
	if (on) {
#ifdef WITH_TSE3
		NResource::mapper_->setEchoChannel(currentStaff_->getChannel(), currentStaff_->getVoice());
		connect(&midiInTimer_, SIGNAL(timeout()), this, SLOT(readNotesFromMidiMapper()));
		midiInTimer_.start(20, false);
#endif
	}
	else {
#ifdef WITH_TSE3
		disconnect(&midiInTimer_, SIGNAL(timeout()), this, SLOT(readNotesFromMidiMapper()));
		midiInTimer_.stop();
#endif
		if (kbInsertButton_->isOn()) {
			kbInsertButton_->setOn(false);
		}
	}
}

void NMainFrameWidget::setKbInsertMode(bool on) {
	if (on) {
		if (!kbbutton_->isOn()) {
#ifdef WITH_TSE3
			NResource::mapper_->setEchoChannel(currentStaff_->getChannel(), currentStaff_->getVoice());
			connect(&midiInTimer_, SIGNAL(timeout()), this, SLOT(readNotesFromMidiMapper()));
			midiInTimer_.start(20, false);
#endif
			kbbutton_->setOn(true);
		}
	}
}

void NMainFrameWidget::readNotesFromMidiMapper() {
#ifdef WITH_TSE3
	NChord *newchord;
	unsigned int status;
	NMusElement *curElem;
	int line, offs, *pitch;
	int newXpos;
	QList<int> *pitches;

	pitches = NResource::mapper_->readEvents();
	if (!pitches) return;

	if (main_props_.actualLength < 0) {
		delete pitches;
		return;
	}
	if (!kbInsertButton_->isOn()) {
		delete pitches;
		return;
	}
	pitch = pitches->first();
	if ((curElem = currentStaff_->getVoiceNr(0)->getCurrentPosition())) {
		currentStaff_->getVoiceNr(0)->validateKeysig(-1, curElem->getXpos());
	}
	else {
		currentStaff_->getVoiceNr(0)->validateKeysig(-1, 200);
	}
	currentStaff_->actualClef_.midi2Line(*pitch, &line, &offs, &(currentStaff_->actualKeysig_));
	status = 0;
	if (main_props_.tied) status |= STAT_TIED;
	if (main_props_.staccato) status |= STAT_STACC;
	if (main_props_.sforzato) status |= STAT_SFORZ;
	if (main_props_.portato) status |= STAT_PORTA;
	if (main_props_.strong_pizzicato) status |= STAT_STPIZ;
	if (main_props_.sforzando) status |= STAT_SFZND;
	if (main_props_.fermate) status |= STAT_FERMT;
	status |= (main_props_.dotcount & DOT_MASK);
	status |= (main_props_.noteBody & BODY_MASK);
	newchord = new  NChord(&main_props_, currentStaff_->getStaffPropsAddr() ,line, offs, main_props_.actualLength, 
				currentVoice_->stemPolicy_, status);
	for (pitch = pitches->next(); pitch; pitch = pitches->next()) {
		currentStaff_->actualClef_.midi2Line(*pitch, &line, &offs, &(currentStaff_->actualKeysig_));
		newchord->insertNewNote(line, offs, currentVoice_->stemPolicy_, status);
	}
	delete pitches;
	if (!currentVoice_->insertAfterCurrent(newchord)) return;
	setEdited();
	computeMidiTimes(NResource::automaticBarInsertion_);
	reposit();
	newXpos = currentVoice_->getCurrentElement()->getXpos()+ currentVoice_->getCurrentElement()->getBbox()->width()+CUR_DIST;
	if (newXpos + SMALL_X_SENS_DIST > leftx_ + paperScrollWidth_) {
		scrollx_->setValue(leftx_ + SMALL_X_SCROLL);
		return;
	}
	repaint();
#endif
}

void NMainFrameWidget::restoreOverlengthAera() {
	int y, i;
	if (help_x0_ >= 0) {
		main_props_.directPainter->beginTranslated();
		main_props_.directPainter->setPen(NResource::whitePen);
		main_props_.directPainter->setRasterOp(XorROP);
		for (i = 0, y = help_y_; i < num_help_lines_; ++i, y += LINE_DIST) {
			main_props_.directPainter->drawLine(help_x0_, y, help_x1_, y);
		}
		main_props_.directPainter->end();
		help_x0_ = -1;
	}
}

void NMainFrameWidget::grabElementsAccording() {
	NStaff *staff_elem;
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		staff_elem->grabElementsAccording();
	}
}

QList<NMusElement> *NMainFrameWidget::getClipBoard(int clipBoardNr) {
	voiceList_.at(clipBoardNr);
	return voiceList_.current()->getClipBoard();
}

void NMainFrameWidget::checkOverlength(QMouseEvent *evt) {
#define HELP_LINE_LENGTH 10
	int y, i;
	restoreOverlengthAera();
	if (!NResource::showAuxLines_) return;
	QPoint cursor_pos = evt->pos();
	y = TRANSY(cursor_pos.y());
	if (currentStaff_->getBase() + 4 * LINE_DIST < y) {
		help_x1_ = TRANSX(cursor_pos.x());
		help_x0_ = help_x1_ - HELP_LINE_LENGTH;
		help_y_ = currentStaff_->getBase() + 5 * LINE_DIST;
		num_help_lines_ = (y - (currentStaff_->getBase() + 4 * LINE_DIST)) / LINE_DIST;
		if (num_help_lines_ >= (LINE_OVERFLOW / 2)) num_help_lines_ = (LINE_OVERFLOW / 2);
		main_props_.directPainter->beginTranslated();
		main_props_.directPainter->setPen(NResource::whitePen);
		main_props_.directPainter->setRasterOp(XorROP);
		for (i = 0, y = help_y_; i < num_help_lines_; ++i, y += LINE_DIST) {
			main_props_.directPainter->drawLine(help_x0_, y, help_x1_, y);
		}
		main_props_.directPainter->end();
	}
	else if (currentStaff_->getBase() > y) {
		help_x1_ = TRANSX(cursor_pos.x());
		help_x0_ = help_x1_ - HELP_LINE_LENGTH;
		num_help_lines_ = (currentStaff_->getBase() - y) / LINE_DIST;
		if (num_help_lines_ >= (LINE_OVERFLOW / 2)) num_help_lines_ = (LINE_OVERFLOW / 2);
		help_y_ = currentStaff_->getBase() - num_help_lines_ * LINE_DIST;
		main_props_.directPainter->beginTranslated();
		main_props_.directPainter->setPen(NResource::whitePen);
		main_props_.directPainter->setRasterOp(XorROP);
		for (i = 0, y = help_y_; i < num_help_lines_; ++i, y += LINE_DIST) {
			main_props_.directPainter->drawLine(help_x0_, y, help_x1_, y);
		}
		main_props_.directPainter->end();
	}
}

/*--------------------------- reaction on menu events -----------------------------------*/

void NMainFrameWidget::newPaper() {
	if (editiones_) {
		switch (KMessageBox::warningYesNoCancel
		         (this,
		          i18n("Your document contains unsaved changes.\n"
		               "Do you want to save your changes or discard them?"),
		          kapp->makeStdCaption(i18n("New")),
		          i18n("&Save"),
		          i18n("&Discard")
		         )
		       ) {
			case KMessageBox::Cancel: return;
			case KMessageBox::No:     break;
			default:
				fileSave();
				break;
		}
	}
	voiceList_.setAutoDelete(false);
	voiceList_.clear();
	staffList_.setAutoDelete(true);
	staffList_.clear();
	staffList_.setAutoDelete(false);
	currentStaff_ = staffList_.first();
	staffList_.append(currentStaff_ = new NStaff(Y_STAFF_BASE +  NResource::overlength_, 0, 0, this));
	voiceList_.append(currentVoice_ = currentStaff_->getVoiceNr(0));
	lastYHeight_ = voiceList_.last()->getStaff()->getBase()+voiceList_.last()->getStaff()->underlength_;
	enableCriticalButtons(true);
	voiceDisplay_->setMax(currentStaff_->voiceCount());
	voiceDisplay_->setVal(1);
	staffCount_ = staffList_.count();
	currentStaff_->setChannel(0);
	currentStaff_->changeVoice(0);
	currentStaff_->setActual(true);
	currentStaff_->setBase( NResource::overlength_  + Y_STAFF_BASE);
	actualFname_ = QString();
	setCaption("noteedit");
	emit caption("noteedit");
	tempo_ = INTERNAL_LENGTH_FAC * DEFAULT_TEMPO;
	NVoice::resetUndo();
	NResource::windowWithSelectedRegion_ = 0;
	scrollx_->setValue(0); // includes repaint()
	currentStaff_->setVolume(80);
	reposit();
	setScrollableNotePage();
	repaint();
	setEdited(false);
	scTitle_.truncate(0);
	scSubtitle_.truncate(0);
	scAuthor_.truncate(0);
	scLastAuthor_.truncate(0);
	scComment_.truncate(0);
	setSaveWidth(160);
	setSaveHeight(250);
	setParamsEnabled(false);
}

const char* noteedit_file_pattern = "*.not|NoteEdit (*.not)\n*|All Files (*)";
const char* midi_file_pattern = "*.mid|MIDI (*.mid)\n*|All Files (*)";
const char* tse3_file_pattern = "*.tse3|TSE3 (*.tse3)\n*|All Files (*)";

void NMainFrameWidget::fileOpen() {
	if (playing_) return;

	if (editiones_) {
		switch (KMessageBox::warningYesNoCancel
		         (this,
		          i18n("Your document contains unsaved changes.\n"
		               "Do you want to save your changes or discard them?"),
		          kapp->makeStdCaption(i18n("Open")),
		          i18n("&Save"),
		          i18n("&Discard")
		         )
		       ) {
			case KMessageBox::Cancel: return;
			case KMessageBox::No:     break;
		  default:
				fileSave();
				break;
		}
	}
	QString fileName = KFileDialog::getOpenFileName( QString::null, noteedit_file_pattern, this );
	if (!fileName.isNull() ) {
		loadFile( fileName );
		KURL url;
		url.setPath( fileName );
		m_recentFilesAction->addURL( url );
        	m_recentFilesAction->saveEntries( KGlobal::config() );
		synchronizeRecentFiles();
	}
}

bool NMainFrameWidget::loadFile( const QString & fileName )
{
	NVoice *voice_elem;
#ifdef WITH_TSE3
	kbbutton_->setOn(false);
#endif
	if (readStaffs(fileName)) {
		actualFname_ = fileName;
		parentWidget()->setCaption(actualFname_);
		tempo_ = DEFAULT_TEMPO * INTERNAL_LENGTH_FAC;
		setScrollableNotePage();
		NResource::windowWithSelectedRegion_ = 0;
		arrangeStaffs();
		reposit();
		for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
			voice_elem->correctReadTrillsAndDynamics();
		}
		scrollx_->setValue(0); // includes repaint()
		setEdited(false);
		stemUpbutton_->setOn(false);
		stemDownbutton_->setOn(false);
		main_props_.actualStemDir = STEM_DIR_AUTO;
		return true;
	}
	return false;
}

// Called when an item is selected from the "recent files" submenu
void NMainFrameWidget::fileOpenRecent( const KURL & u )
{
	if (!testEditiones()) return;
	ASSERT(u.isLocalFile());
	loadFile( u.path() );
	m_recentFilesAction->addURL( u );
        m_recentFilesAction->saveEntries( KGlobal::config() );
	synchronizeRecentFiles();
}

void NMainFrameWidget::showLyricsDialog() {
	if (playing_) return;
	currentVoice_->copyLyricsToEditor();
	lyricsDialog_->boot();
	currentVoice_->updateLyrics();
	setEdited();
	reposit();
	repaint();
}

void NMainFrameWidget::fileSave() {
	if (actualFname_.isNull())
		fileSaveAs();
	else
		writeStaffs(actualFname_);
}



void NMainFrameWidget::fileSaveAs() {
	QString fileName = this->checkFileName(KFileDialog::getSaveFileName( QString::null, noteedit_file_pattern, this ), ".not");

	if (!fileName.isNull() ) {
		writeStaffs(fileName);
		actualFname_ = fileName;
		emit caption(actualFname_);
		KURL url;
		url.setPath( fileName );
		m_recentFilesAction->addURL( url );
        	m_recentFilesAction->saveEntries( KGlobal::config() );
		synchronizeRecentFiles();
	}
	repaint();
}

void NMainFrameWidget::exportMidi() {	this->exportManager( MIDI_PAGE ); }
void NMainFrameWidget::exportMusiXTeX() { this->exportManager( MUSIX_PAGE ); }
void NMainFrameWidget::exportPMX() {	this->exportManager( PMX_PAGE ); }
void NMainFrameWidget::exportLilyPond() {	this->exportManager( LILY_PAGE ); };
void NMainFrameWidget::setOutputParam() {	this->exportManager( PARAM_PAGE ); };

void NMainFrameWidget::exportManager( int type ) {

    if (playing_) return;

    exportDialog_->card->setCurrentPage( type );
    exportDialog_->initialize( &staffList_, &voiceList_, actualFname_);
    exportDialog_->boot();

}

void NMainFrameWidget::exportMusixTeXImm() {
	NResource::staffSelExport_ = 0;
	exportDialog_->texMusixbm->setChecked(false);
	exportDialog_->texMusixadd->setChecked(true);
	exportDialog_->texWidth->setValue( 170 );
	exportDialog_->texHeight->setValue( 250 );
	exportDialog_->texTop->setValue( -24 );
	exportDialog_->texLeft->setValue( -10 );
	exportDialog_->texSize->setChecked(false);
	exportDialog_->texBar->setChecked(true);
	exportDialog_->texTies->setChecked(false);
        NMusiXTeX mt;
	QRegExp notSuff(".not$");
	QString fname(actualFname_);
	fname.replace(notSuff, ".tex");
	mt.exportStaffs(fname, &staffList_, exportDialog_, this);
}

void NMainFrameWidget::exportLilyPondImm() {
	NResource::staffSelExport_ = 0;
	exportDialog_->lilyWidth->setValue( 160 );
	exportDialog_->lilyHeight->setValue( 250 );
	exportDialog_->lilyVoice->setChecked(false);
	exportDialog_->lilyBeam->setChecked(false);
	exportDialog_->lilyTies->setChecked(false);
	exportDialog_->lilyTies->setChecked(false);
	exportDialog_->lilyStem->setChecked(false);
	exportDialog_->lilyVol->setCurrentItem(1);
	exportDialog_->lilyMeasure->setChecked(true);
	NLilyExport le;
	QRegExp notSuff(".not$");
	QString fname(actualFname_);
	fname.replace(notSuff, ".ly");
	le.exportStaffs(fname, &staffList_, exportDialog_, this);
}

void NMainFrameWidget::zoomIn() {
	zoomselect_->zoomIn();
}

void NMainFrameWidget::zoomOut() {
	zoomselect_->zoomOut();
}

void NMainFrameWidget::openNewWindow() {
	NMainWindow *newWindow = new NMainWindow;
	if ((NResource::lastWindowX_ += WINDOWXY_INCR) > MAXWINDOWXY) NResource::lastWindowX_ = 0;
	if ((NResource::lastWindowY_ += WINDOWXY_INCR) > MAXWINDOWXY) NResource::lastWindowY_ = 0;
	newWindow->setGeometry( NResource::lastWindowX_, NResource::lastWindowY_, START_WIDTH, START_HEIGHT);
	newWindow->show();
}

bool NMainFrameWidget::testEditiones() {
	if (editiones_) {
		switch (KMessageBox::warningYesNoCancel
		         (this,
		          i18n("Your document contains unsaved changes.\n"
		               "Do you want to save your changes or discard them?"),
		          kapp->makeStdCaption(i18n("Close")),
		          i18n("&Save"),
		          i18n("&Discard")
		         )
	         ) {
			case KMessageBox::Cancel: return false;
			case KMessageBox::No:     return true;
		  default:
				fileSave();
				break;
		}
	}
	return true;
}

void NMainFrameWidget::keyConfig() {
	KKeyDialog::configureKeys( keys_ );
}

void NMainFrameWidget::quitDialog() {
	if (playing_) return;
	if (!testEditiones()) return;
	if (NResource::windowList_.count() > 1) {
		NMainWindow *mainWindow = static_cast<NMainWindow *>(parentWidget());
		NResource::windowList_.removeRef(mainWindow);
		mainWindow->close(true);
	}
	else {
		NMainWindow *mainWindow = static_cast<NMainWindow *>(parentWidget());
		delete NResource::nresourceobj_;
		mainWindow->close(true);
		qApp->quit();
	}
}

void NMainFrameWidget::closeAllWindows() {
	NMainFrameWidget *frameWindow;
	NMainWindow *mainWindow;
	if (playing_) return;
	if (KMessageBox::warningYesNo
	     (this,
	      i18n("Do you really want to close all windows?"),
	      kapp->makeStdCaption(i18n("Close all")),
	      i18n("&Close all")
	     )
	    != KMessageBox::Yes
		 ) return;
	while (!NResource::windowList_.isEmpty()) {
		mainWindow = NResource::windowList_.first();
		frameWindow = (NMainFrameWidget *) mainWindow->centralWidget();
		if (!frameWindow->testEditiones()) return;
		NResource::windowList_.removeRef(mainWindow );
		mainWindow->close(true);
	}
	delete NResource::nresourceobj_;
	qApp->quit();
}

void NMainFrameWidget::toggleBarNumbers() {
	NResource::showStaffNrs_ = not NResource::showStaffNrs_;
}

void NMainFrameWidget::toggleStaffNames() {
	NResource::showStaffNames_ = not NResource::showStaffNames_;
}

void NMainFrameWidget::toggleAuxLines() {
	NResource::showAuxLines_ = not NResource::showAuxLines_;
}

void NMainFrameWidget::gotoDialog() {
	if (playing_) return;
	scaleFrm_->desc->setText(i18n("<center>Please choose the target point!</center>"));
	scaleFrm_->chkbox->hide();
	scaleFrm_->scal_ed->setAll(0, lastBarNr_ - 1, 0);
	scaleFrm_->setCaption(kapp->makeStdCaption(i18n("Goto")));
	scaleFrm_->boot( &staffList_, scrollx_ );
}

void NMainFrameWidget::muteDialog() {
	NStaff *staff_elem;
	int i;
	if (NResource::staffSelMute_) delete [] NResource::staffSelMute_;
	NResource::staffSelMute_ = new bool[staffList_.count()];
	for (i = 0, staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next(), i++) {
		NResource::staffSelMute_[i] = staff_elem->getMuted();
	}
	multistaffDialog_->boot( &staffList_, STAFF_ID_MUTE );
	for( i = 0, staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next(), i++ )
	    staff_elem->setMuted(NResource::staffSelMute_[i]);
}

void NMainFrameWidget::voiceDialog() {
	VoiceDialog voicedialog(this, staffList_.find(currentStaff_), &staffList_);
	voicedialog.exec();
}

void NMainFrameWidget::setStaffProperties() {
	staffPropFrm_->boot( currentStaff_ );
}

void NMainFrameWidget::autoBar() {
	if( NResource::staffSelAutobar_ ) delete [] NResource::staffSelAutobar_;
	NResource::staffSelAutobar_ = 0;
	multistaffDialog_->boot( &staffList_, STAFF_ID_AUTOBAR );
	if( !NResource::staffSelAutobar_ )
	    return;
	NStaff *staff;
	int i;
	for (i = 0, staff = staffList_.first(); staff; staff = staffList_.next(), i++)
	    if (NResource::staffSelAutobar_[i]) 
		staff->autoBar();
	computeMidiTimes(false);
	reposit();
	repaint();
	setEdited();
}

void NMainFrameWidget::doAutoBeam() {
	if (NResource::staffSelAutobeam_) delete [] NResource::staffSelAutobeam_;
	NResource::staffSelAutobeam_ = 0;
	multistaffDialog_->boot( &staffList_, STAFF_ID_AUTOBEAM );
	NStaff *staff;
	bool includeRests;
	int i;

	includeRests = scaleFrm_->chkbox->isChecked();
	for (i = 0, staff = staffList_.first(); staff; staff = staffList_.next(), i++) {
		if (NResource::staffSelAutobeam_[i]) staff->autoBeam(scaleFrm_->scal_ed->getValue(), includeRests);
	}
	reposit();
	repaint();
	NResource::progress_->hide();
	setEdited();
}

void NMainFrameWidget::generateClef(int type, int shift) {
	if (playing_) return;
	selectedSign_ = T_CLEF;
	tmpElem_ = new NClef(currentVoice_->getMainPropsAddr(), currentStaff_->getStaffPropsAddr(), type, shift);
}

void NMainFrameWidget::performClefChange(int type, int shift) {
	if (playing_) return;
	currentStaff_->performClefChange(type, shift);
	reposit();
	repaint();
}

void NMainFrameWidget::clefDialog() {
	clefDialog_->boot( IS_CLEF );
}

void NMainFrameWidget::changeClefDialog() {
	clefDialog_->boot( IS_CLEF_DISTANCE );
	setEdited();
}

void NMainFrameWidget::redAccidentals() {
	if (playing_) return;
	currentStaff_->setHalfsAccordingKeySig();
	setEdited();
	reposit();
	repaint();

}
void NMainFrameWidget::collChords() {
	if (playing_) return;
	currentStaff_->collChords();
	setEdited();
	computeMidiTimes(false);
	reposit();
	repaint();
}

void NMainFrameWidget::setAllSharp() {
	if (playing_) return;
	currentVoice_->setHalfsTo(STAT_CROSS);
	setEdited();
	reposit();
	repaint();
}

void NMainFrameWidget::setAllFlat() {
	if (playing_) return;
	currentVoice_->setHalfsTo(STAT_FLAT);
	setEdited();
	reposit();
	repaint();
}

void NMainFrameWidget::insertRepeatOpen() {
	if (playing_) return;
	selectedSign_ = REPEAT_OPEN;
}

void NMainFrameWidget::insertRepeatOpenClose() {
	if (playing_) return;
	selectedSign_ = REPEAT_OPEN_CLOSE;
}


void NMainFrameWidget::insertRepeatClose() {
	if (playing_) return;
	selectedSign_ = REPEAT_CLOSE;
}

void NMainFrameWidget::insertspecEnding1() {
	if (playing_) return;
	selectedSign_ = SPECIAL_ENDING1;
}

void NMainFrameWidget::insertspecEnding2() {
	if (playing_) return;
	selectedSign_ = SPECIAL_ENDING2;
}

void NMainFrameWidget::keyDialog() {
	int i, h;
	keyOkButton_->setDown(false);
	keyCancButton_->setDown(false);
	keyDialog_->setCaption(kapp->makeStdCaption(i18n("Key")));
	keyDialog_->setGeometry(40, 40, LISTWIDTH + LIST_DIST + 7*KEYOFFSWIDTH + RIGHT_BORDER, 20 * (10+2) + PUSHBUTTONHEIGHT);
	keyDialog_->setMinimumSize(LISTWIDTH + LIST_DIST + 7*KEYOFFSWIDTH + RIGHT_BORDER, 20 * (10+2) + PUSHBUTTONHEIGHT);
	keyDialog_->setMaximumSize(LISTWIDTH + LIST_DIST + 7*KEYOFFSWIDTH + RIGHT_BORDER, 20 * (10+2) + PUSHBUTTONHEIGHT);
	keyList_->setGeometry(10, 10, LISTWIDTH-50, 20 * 10);
	tmpKeysig_ = new NKeySig(currentVoice_->getMainPropsAddr(), currentStaff_->getStaffPropsAddr());
	keyList_->setCurrentItem(0);
	for (i = 0; i < 7; ++i) {
		offs_list_[i]->set(STAT_NATUR);
		offs_list_[i]->setKeysigObj(tmpKeysig_);
		offs_list_[i]->setGeometry(LISTWIDTH + LIST_DIST+i*KEYOFFSWIDTH, 10, KEYOFFSWIDTH, keyList_->height()/2);
	}
	h = (keyList_->height()/2 - KEY_OFFS_UP_DIST-KEY_OFFS_BOTTOM_DIST) / 3;
	crosslabel_->setGeometry(LISTWIDTH + KEY_OFFS_LABEL_DIST, 10 +  KEY_OFFS_UP_DIST, KEY_OFFS_LABEL_WIDTH, KEY_OFFS_LABEL_HEIGHT);
	flatlabel_->setGeometry(LISTWIDTH + KEY_OFFS_LABEL_DIST, 10 + KEY_OFFS_UP_DIST + h, KEY_OFFS_LABEL_WIDTH, KEY_OFFS_LABEL_HEIGHT);
	naturlabel_->setGeometry(LISTWIDTH + KEY_OFFS_LABEL_DIST, 10 + KEY_OFFS_UP_DIST + 2*h, KEY_OFFS_LABEL_WIDTH, KEY_OFFS_LABEL_HEIGHT);
	keyOkButton_->setGeometry(40, keyList_->height() + 20, PUSHBUTTONWIDTH, PUSHBUTTONHEIGHT);
	keyCancButton_->setGeometry(40 + 3*PUSHBUTTONWIDTH, keyList_->height() + 20, 2*PUSHBUTTONWIDTH, PUSHBUTTONHEIGHT);
	keyDialog_->show();
}

/*
void NMainFrameWidget::timesigDialog() {
	tmpTimeSig_ = new NTimeSig(currentVoice_->getMainPropsAddr(), currentStaff_->getStaffPropsAddr());
	int value = timesigDialog_->boot( IS_TIME );
	tmpTimeSig_->setSignature( ( value % 24 ) + 1, 1 << ( ( ( value + 1 ) / 25 ) + 1 ) );
	tmpElem_ = tmpTimeSig_;
	selectedSign_ = T_TIMESIG;
}
*/

void NMainFrameWidget::timesigDialog() {
	 timesigDialog_->showDialog();
}

void NMainFrameWidget::setTempTimesig(int num, int dom) {
	tmpTimeSig_ = new NTimeSig(currentVoice_->getMainPropsAddr(), currentStaff_->getStaffPropsAddr());
	tmpTimeSig_->setSignature(num, dom);
	tmpElem_ = tmpTimeSig_;
	selectedSign_ = T_TIMESIG;
}
bool NMainFrameWidget::paramsEnabled() {return exportDialog_->paramsEnabled();}
int NMainFrameWidget::getSaveWidth() {return exportDialog_->getSaveWidth();}
bool NMainFrameWidget::withMeasureNums() {return exportDialog_->withMeasureNums();}
int NMainFrameWidget::getSaveHeight() {return exportDialog_->getSaveHeight();}
void NMainFrameWidget::setParamsEnabled(bool ok) {exportDialog_->setEnabled(ok);}
void NMainFrameWidget::setSaveWidth(int width)  {exportDialog_->setSaveWidth(width);}
void NMainFrameWidget::setSaveHeight(int height) {exportDialog_->setSaveHeight(height);}
void NMainFrameWidget::setWithMeasureNums(bool with) {exportDialog_->setWithMeasureNums(with);}

void NMainFrameWidget::newStaff() {
	int staffYpos = 0;
	NVoice *voice;
	if (playing_) return;
	currentStaff_->setActual(false);
	staffList_.append(currentStaff_  = new NStaff(staffYpos+NResource::underlength_, staffCount_ % 16, 0, this));
	voiceList_.append(currentVoice_ = currentStaff_->getVoiceNr(0));
	enableCriticalButtons(true);
	voiceDisplay_->setMax(currentStaff_->voiceCount());
	voiceDisplay_->setVal(1);
	currentStaff_->setActual(true);
	arrangeStaffs();
	++staffCount_;
	reposit();
	setScrollableNotePage();
	repaint();
	setEdited();
	cleanupSelections();
}

void NMainFrameWidget::deleteStaff() {
	int i;
	NStaff *staff_elem;
	if (playing_) return;
	if (staffCount_ == 1) {
		KMessageBox::sorry
		  (this,
		   i18n("Can't delete last staff."),
		   kapp->makeStdCaption(i18n("Delete Staff"))
		  );
		return;
	}
	if (KMessageBox::warningYesNo
	     (0,
	      i18n("This deletes current voice! Are you sure?"),
	      kapp->makeStdCaption(i18n("Delete Staff")),
	      i18n("&Delete")
	     )
	    != KMessageBox::Yes
	   ) return;
	if (staffList_.find(currentStaff_) < 0) {
		NResource::abort("NMainFrameWidget::deleteStaff: internal error", 1);
	}
	staffList_.remove();
	currentStaff_->updateVoiceList(&voiceList_);
	delete currentStaff_;
	staffCount_--;
	if ((currentStaff_ = staffList_.current()) == 0) {
		NResource::abort("NMainFrameWidget::deleteStaff: internal error", 2);
	}
	if ((currentVoice_ = currentStaff_->getVoiceNr(0)) == 0) {
		NResource::abort("NMainFrameWidget::deleteStaff: internal error", 3);
	}
	enableCriticalButtons(currentVoice_->isFirstVoice());
	arrangeStaffs();
	voiceDisplay_->setMax(currentStaff_->voiceCount());
	voiceDisplay_->setVal(1);
	currentStaff_->setActual(true);
	reposit();
	setScrollableNotePage();
	repaint();
	setEdited();
	cleanupSelections();
}

void NMainFrameWidget::staffMoveDialog() {

	listDialog_->boot
	  (0, LIST_MOVE_STAFF, kapp->makeStdCaption(i18n("Staff moving")),
	   i18n("Please choose the target position:"), &staffList_
	  );
	if( !listDialog_->succ_ )
	    return;

	int cur_idx;
	NStaff *staff_elem;
	lastYHeight_ = 0;
	char *err = "moveStaff: internal error";
	if (staffList_.find(currentStaff_) == -1) {
		NResource::abort(err, 1);
	}
	cur_idx = staffList_.at();
	if (listDialog_->choice->currentItem() == cur_idx) return;
	staffList_.remove();
	if (listDialog_->choice->currentItem() == staffCount_ - 1) {
		staffList_.append(currentStaff_);
	}
	else {
		staffList_.insert(listDialog_->choice->currentItem(), currentStaff_);
	}
	arrangeStaffs();
	setEdited();
	reposit();
	repaint();
}

void NMainFrameWidget::cancelMultiStaff() {
	if (NResource::staffSelMulti_) delete [] NResource::staffSelMulti_;
	NResource::staffSelMulti_ = 0;
	NResource::numOfMultiStaffs_ = 0;
}

void NMainFrameWidget::multiStaffDialog() {
	if (NResource::staffSelMulti_) delete [] NResource::staffSelMulti_;
	NResource::staffSelMulti_ = 0;
	NResource::numOfMultiStaffs_ = staffList_.count();
	multistaffDialog_->boot( &staffList_, STAFF_ID_MULTI );
}

void NMainFrameWidget::tempoSigDialog() {
	scaleFrm_->desc->setText(i18n("<center>Set tempo:</center>"));
	scaleFrm_->scal_ed->setAll(40, 300, 100);
	scaleFrm_->chkbox->hide();
	scaleFrm_->setCaption(kapp->makeStdCaption(i18n("Set new tempo")));
	scaleFrm_->ok->setText(i18n("Set new temp&o"));
	if( scaleFrm_->boot( &main_props_, currentStaff_, currentVoice_, &tmpElem_, TEMPO_SIGNATURE ) )
	    selectedSign_ = TEMPO_SIGNATURE;
}

void NMainFrameWidget::multiRestDialog() {
	scaleFrm_->desc->setText(i18n("<center>Set count of Measures:</center>"));
	scaleFrm_->scal_ed->setAll(1, 200, 2);
	scaleFrm_->chkbox->hide();
	scaleFrm_->setCaption(kapp->makeStdCaption(i18n("multi rest")));
	scaleFrm_->ok->setText(i18n("Set count of m&easures"));
	if( scaleFrm_->boot( &main_props_, currentStaff_, currentVoice_, &tmpElem_, MULTIREST ) )
	    selectedSign_ = MULTIREST;
}

void NMainFrameWidget::cleanRestsDialog() {
	if( cleanUpRestsDialog_->boot() )
	    this->cleanupRests();
}

void NMainFrameWidget::volChangeDialog() {
	if( volChangeDialog_->boot() )
	    insVolChange();
}

void NMainFrameWidget::autoBeamDialog() {
	scaleFrm_->chkbox->show();
	scaleFrm_->desc->setText(i18n("<center>How many notes:</center>"));
	scaleFrm_->scal_ed->setMinVal( 2 );
	scaleFrm_->scal_ed->setMaxVal( 16 );
	scaleFrm_->scal_ed->setStartVal( 4 );
	scaleFrm_->setCaption(kapp->makeStdCaption(i18n("Autobeam")));
	scaleFrm_->ok->setText(i18n("&Start"));
	if ( ! scaleFrm_->boot() )
	    return;

	doAutoBeam();

}

void NMainFrameWidget::transposeDialog() {
	NStaff *staff_elem;
	int i, semitones;

	scaleFrm_->chkbox->hide();
	scaleFrm_->desc->setText(i18n("<center>Semitones:</center>"));
	scaleFrm_->scal_ed->setMinVal( -12 );
	scaleFrm_->scal_ed->setMaxVal( 12 );
	scaleFrm_->scal_ed->setStartVal( 0 );
	scaleFrm_->setCaption(kapp->makeStdCaption(i18n("Transpose")));
	scaleFrm_->ok->setText(i18n("&Transpose"));
	if (NResource::numOfMultiStaffs_) {
		if (NResource::windowWithSelectedRegion_ == 0 || NResource::windowWithSelectedRegion_ == this) {
			semitones = scaleFrm_->boot();
			for (i = 0, staff_elem = staffList_.first(); staff_elem && i < NResource::numOfMultiStaffs_;
				 staff_elem = staffList_.next() , i++) {
				if (NResource::staffSelMulti_[i]) {
					staff_elem->transpose (semitones);
				}
			}
			setEdited();
		}
	}
	else {
		currentStaff_->transpose( scaleFrm_->boot() );
		setEdited();
	}
	reposit();
	repaint();
}


void NMainFrameWidget::voiceChangeDialog() {
	if (listDialog_->boot
	     (currentStaff_->getVoice(),
	      LIST_VOICE,
	      kapp->makeStdCaption(i18n("Voice selection")),
	      i18n("<center>Select your voice here:</center>")
	     )
	   ) this->changeVoice();
}

/*--------------------------- reaction on Ok button of the dialogs above ----------------*/


void NMainFrameWidget::cleanupRests() {
	int smallestRest;

	smallestRest = cleanUpRestsDialog_->item2length();
	currentVoice_->cleanupRests(smallestRest);
	computeMidiTimes(false);
	reposit();
	repaint();
	setEdited();
}

void NMainFrameWidget::insVolChange() {
	NSign *sign;
	sign = new NSign(&main_props_, currentStaff_->getStaffPropsAddr(), VOLUME_SIG);
	sign->setVolume( volChangeDialog_->sel->currentItem(), volChangeDialog_->scal_ed->getValue() );
	tmpElem_ = sign;
	selectedSign_ = VOLUME_SIG;
}

void NMainFrameWidget::changeVoice(int voice) {
	NSign *sign;
	sign = new NSign(&main_props_, currentStaff_->getStaffPropsAddr(), PROGRAM_CHANGE);
	if( voice < 0 )
	    voice = listDialog_->choice->currentItem();
	sign->setProgram(voice);
	tmpElem_ = sign;
	selectedSign_ = PROGRAM_CHANGE;
}

void NMainFrameWidget::changeTempo(int temp) {
	tempo_ = temp * INTERNAL_LENGTH_FAC;
}

void NMainFrameWidget::arrangeStaffs() {
	NStaff *staff_elem;
	lastYHeight_ = Y_STAFF_BASE;
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		staff_elem->setBase(lastYHeight_+staff_elem->overlength_);
		lastYHeight_ += staff_elem->underlength_ + staff_elem->overlength_ + STAFF_HIGHT;
	}
	reposit();
	repaint();
	setEdited();
}


void NMainFrameWidget::changeZoomValue(int zval) {
	main_props_.zoom = zoomselect_->computeZoomVal(zval);
	main_props_.tp->setZoom(main_props_.zoom);
	main_props_.p->setZoom(main_props_.zoom);
	main_props_.directPainter->setZoom(main_props_.zoom);
	main_props_.scaledText_ = QFont ("Times" , (int) (main_props_.zoom * 36.0), QFont::Normal);
	main_props_.scaledItalic_ = QFont ("Times" , (int) (main_props_.zoom * 36.0), QFont::Normal, true);
	main_props_.scaledBold_ = QFont ("Times" , (int) (main_props_.zoom * 48.0), QFont::Bold);
	main_props_.scaledBold2_ = QFont ("Times" , (int) (main_props_.zoom * 36.0), QFont::Bold);
	main_props_.scaledBoldItalic_ = QFont ("Times" , (int) (main_props_.zoom * 40.0), QFont::Bold, true);
	setScrollableNotePage();
	xscrollFromWidget(leftx_);
}

void NMainFrameWidget::setInsertionKey() {
	int i;

	if (tmpKeysig_ == 0) return;
	for (i = 0; i < 7; ++i) {
		offs_list_[i]->setKeysigObj(0);
	}
	tmpElem_ = tmpKeysig_;
	selectedSign_ = T_KEYSIG;
	keyDialog_->hide();
}

void NMainFrameWidget::changeKey(int idx) {
	int i;
	int count, kind;

	if (idx > 6) {
		count = idx - 6;
		kind = STAT_CROSS;
	}
	else {
		count = idx;
		kind = STAT_FLAT;
	}
	tmpKeysig_->setRegular(count, kind);
	for (i = 0; i < 7; ++i) {
		offs_list_[i]->setKeysigObj(0); // avoid feedback
		offs_list_[i]->set(STAT_NATUR);
	}
	for (i = 0; i < 7; ++i) {
		offs_list_[i]->set(tmpKeysig_->getState(i));
	}
	for (i = 0; i < 7; ++i) {
		offs_list_[i]->setKeysigObj(tmpKeysig_); // avoid feedback
	}
}

/*-------------------------- reaction on scroll events ---------------------------------*/

void NMainFrameWidget::xscrollFromWidget(int val) {
	if (playing_) return;
        leftx_ = val;
	main_props_.tp->setXPosition(val-LEFT_RIGHT_BORDER);
	main_props_.directPainter->setXPosition(val-LEFT_RIGHT_BORDER);
	repaintAll();
}

void NMainFrameWidget::xscrollDuringReplay(int val) {
	NStaff *staff_elem;
	bool leftScroll = false;
	if (val < leftx_) {
		leftScroll = true;
	}
        leftx_ = val - turnOverOffset_;
	if (leftScroll) {
        	leftx_ = val;
		main_props_.tp->setXPosition(leftx_ - LEFT_RIGHT_BORDER);
		main_props_.p->begin( notePart_->acWritePixmap() );
		main_props_.p->setBrush(NResource::backgroundBrush);
		main_props_.p->setPen(NResource::noPen);
		main_props_.p->setPen(NResource::blackPen);
		main_props_.p->drawRect(0, 0, paperWidth_, paperHeight_);
		main_props_.p->end();
		for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
			if (staff_elem->getBase() < topy_) continue;
			if (staff_elem->getBase() > boty_) break;
			staff_elem->draw(leftx_, leftx_ + (int) ((float) nettoWidth_ / main_props_.zoom));
		}
		nextStaffElemToBePainted_ = 0;
	}
	if (nextStaffElemToBePainted_) {
		if (staffList_.find(nextStaffElemToBePainted_) == -1) {
			printf("staff not found\n");fflush(stdout);
		}
	}
	while (nextStaffElemToBePainted_) {
		if (nextStaffElemToBePainted_->getBase() < topy_) continue;
		if (nextStaffElemToBePainted_->getBase() > boty_) break;
		nextStaffElemToBePainted_->draw(newLeft_, newRight_);
		nextStaffElemToBePainted_ = staffList_.next();
	}
	newLeft_ = leftx_ + paperScrollWidth_ - turnOverOffset_;
	newRight_ = newLeft_+ paperScrollWidth_;
	notePart_->swap();
	main_props_.tp->setXPosition(newLeft_ - LEFT_RIGHT_BORDER);
	main_props_.tp->setPaintDevice(notePart_->acWritePixmap());
	main_props_.p->setPaintDevice(notePart_->acWritePixmap());
	main_props_.p->begin( notePart_->acWritePixmap() );
	main_props_.p->setBrush(NResource::backgroundBrush);
	main_props_.p->setPen(NResource::noPen);
	main_props_.p->setPen(NResource::blackPen);
	main_props_.p->drawRect(0, 0, paperWidth_, paperHeight_);
	main_props_.p->end();
	nextStaffElemToBePainted_ = staffList_.first();
	if (leftScroll) {
		main_props_.directPainter->setXPosition(val-LEFT_RIGHT_BORDER);
	}
	else {
		main_props_.directPainter->setXPosition(val-LEFT_RIGHT_BORDER- turnOverOffset_);
	}
	repaintAll();
}

void NMainFrameWidget::yscroll(int val) {
        topy_ = val;
	boty_ = val + (int) ((float) paperHeight_ / main_props_.zoom);
	main_props_.tp->setYPosition(val-TOP_BOTTOM_BORDER);
	main_props_.directPainter->setYPosition(val-TOP_BOTTOM_BORDER);
	main_props_.p->setYPosition(val-TOP_BOTTOM_BORDER);
	repaintAll();
}

/*-------------------------- reaction on timer events ---------------------------------*/

void NMainFrameWidget::playNext() {
	NMidiEventStr *m_events;
	QList<NMidiEventStr> *tmp;
	NVoice *voice_elem;
	int min_time = (1 << 30);
	int last_time = myTime_;
	int newtempofactor;
	int pxpos;

	if (playStop_) {
		NResource::mapper_->stopAllNotes(nextToPlay_);
		for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
			voice_elem->stopPlaying();
		}
		playStop_ = playing_ = false;
		NResource::mapper_->isInUse_ = false;
		setScrollableNotePage();
		playButtonReset();
		main_props_.tp->setXPosition(leftx_-LEFT_RIGHT_BORDER);
		main_props_.directPainter->setXPosition(leftx_-LEFT_RIGHT_BORDER);
		repaint();
		notePart_->setMouseTracking(NResource::showAuxLines_);
		return;
	}

	nextToSearch_->clear();
	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		m_events = voice_elem->getNextMidiEvent(myTime_+1, false);
		if (m_events) {
			nextToSearch_->append(m_events);
			if (m_events->ev_time < min_time) min_time = m_events->ev_time;
		}
	}

	for (m_events = stopList_.first(); m_events; m_events = stopList_.next()) {
		nextToSearch_->append(m_events);
		if (m_events->ev_time < min_time) {
			min_time = m_events->ev_time;
		}
	}
	notesToPlay_ = 0;
	newtempofactor = 1;
	for (m_events = nextToSearch_->first(); m_events; m_events = nextToSearch_->next()) {
		if (m_events->ev_time == min_time)  {
			pxpos = m_events->xpos;
			++notesToPlay_;
			if (m_events->midi_cmd == MNOTE_OFF) {
				stopList_.find(m_events);
				stopList_.remove();
			}
			else  {
				if (m_events->fermate) newtempofactor = 2;
				m_events->notehalt->ev_time = min_time + m_events->length;
				stopList_.append(m_events->notehalt);
				m_events->from->skipChord();
			}
		}
	}
	NResource::mapper_->play_list(nextToPlay_, last_time);
	if (!notesToPlay_) {
		for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
			voice_elem->stopPlaying();
		}
		playStop_ = playing_ = false;
		NResource::mapper_->isInUse_ = false;
		setScrollableNotePage();
		main_props_.tp->setXPosition(leftx_-LEFT_RIGHT_BORDER);
		repaint();
		playButtonReset();
		return;
	}
	myTime_ = min_time;
	timer_.start((int) ((double) ((myTime_-last_time) * SECONDS_PER_MINUTE * INTERNAL_LENGTH_FAC * tempofactor_) / (MYPPQN * tempo_)), true);
	tempofactor_ = newtempofactor;
	tmp = nextToSearch_;
	nextToSearch_ = nextToPlay_;
	nextToPlay_ = tmp;
	if (nextStaffElemToBePainted_ && !staffPainting_) {
		staffPainting_ = true;
		paintNextStaff();
	}
		
		
	if (pxpos > leftx_ + paperScrollWidth_ - turnOverOffset_) {
		scrollx_->setValue(leftx_ + paperScrollWidth_);
		xscrollDuringReplay(leftx_ + paperScrollWidth_);
	}
	else if (pxpos <  leftx_) { // repeat
		scrollx_->setValue(pxpos);
		xscrollDuringReplay(pxpos);
	}
}


/*----------------------------- internal reaction on resize --------------------------*/

void NMainFrameWidget::setScrollableNotePage() {
	NVoice *voice_elem;
	if (playing_) return;
	paperHeight_ = height_-MENUBARHEIGHT-TOOLBARHEIGHT-3*BORDER-SCROLLBARHEIGHT;
	paperScrollHeight_ = (int) ((float) paperHeight_ / main_props_.zoom);
	boty_ = topy_ + paperScrollHeight_;
	if (paperScrollHeight_ < lastYHeight_ + Y_SCROLL_DIST) {
		scrolly_->setGeometry(width_ - BORDER - SCROLLBARHEIGHT, MENUBARHEIGHT + TOOLBARHEIGHT,
		   	SCROLLBARHEIGHT, height_-MENUBARHEIGHT-TOOLBARHEIGHT-3*BORDER-SCROLLBARHEIGHT);
		scrolly_->setSteps (10, (int) ((float) height_ / main_props_.zoom));
		scrolly_->setRange(0, lastYHeight_);
		scrolly_->show();
		scrolly_->setValue(0);
		paperWidth_ = width_ - 3*BORDER - SCROLLBARHEIGHT;
	}
	else {
		topy_ = 0;
		boty_ = paperScrollHeight_;
		main_props_.tp->setYPosition(-TOP_BOTTOM_BORDER);
		main_props_.directPainter->setYPosition(-TOP_BOTTOM_BORDER);
		main_props_.p->setYPosition(-TOP_BOTTOM_BORDER);
		scrolly_->hide();
		paperWidth_ = width_ - 2*BORDER;
	}

	nettoWidth_ = paperWidth_ - 2*LEFT_RIGHT_BORDER;
	nettoHeight_ = paperHeight_ - 2*TOP_BOTTOM_BORDER;
	paperScrollWidth_ = (int) ((float) paperWidth_ / main_props_.zoom);

	notePart_->setGeometry(BORDER, height()-3*BORDER-SCROLLBARHEIGHT-paperHeight_, paperWidth_, paperHeight_);
	notePart_->set1backpixmap(paperWidth_, paperHeight_);
	main_props_.tp->setPaintDevice(notePart_->acShowPixmap());
	main_props_.directPainter->setPaintDevice(notePart_);
	main_props_.p->setPaintDevice(notePart_->acShowPixmap());
	main_props_.tp->noticeClipRect ( QRect(LEFT_RIGHT_BORDER, TOP_BOTTOM_BORDER, nettoWidth_, nettoHeight_ ));
	main_props_.directPainter->noticeClipRect ( QRect(LEFT_RIGHT_BORDER, TOP_BOTTOM_BORDER, nettoWidth_, nettoHeight_ ));
	main_props_.p->noticeClipRect ( QRect(LEFT_RIGHT_BORDER, TOP_BOTTOM_BORDER, nettoWidth_, nettoHeight_ ));

	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		voice_elem->paperDimensiones(paperScrollWidth_);
	}
	scrollx_->setSteps (10, (int) ((float) width_ / main_props_.zoom));
	if (!playing_) notePart_->setMouseTracking(NResource::showAuxLines_);

}

void NMainFrameWidget::preparePixmaps() {
	scrolly_->hide();
	notePart_->set2backpixmaps();

	newLeft_ = leftx_ + paperScrollWidth_ - turnOverOffset_;
	newRight_ = newLeft_ + paperScrollWidth_;
	main_props_.tp->setXPosition(newLeft_  - LEFT_RIGHT_BORDER);
	main_props_.tp->setPaintDevice(notePart_->acWritePixmap());
	main_props_.p->setPaintDevice(notePart_->acWritePixmap());
	main_props_.p->begin( notePart_->acWritePixmap() );
	main_props_.p->setBrush(NResource::backgroundBrush);
	main_props_.p->setPen(NResource::noPen);
	main_props_.p->setPen(NResource::blackPen);
	main_props_.p->drawRect(0, 0, paperWidth_, paperHeight_);
	main_props_.p->end();
	nextStaffElemToBePainted_ = staffList_.first();
}

/*----------------------------- update of buttons due to selection ------------------*/

void NMainFrameWidget::setButton(int nr) {
	if (nr >= 0) {
            	note_buttons_[nr]->setChecked( true );
	}
	else {
		note_dymmy_->setChecked(true);
	}
}

void NMainFrameWidget::stateButtonChange(int state) {
	if (state == -1) return;

	// avoid feedback
	// Note from David: you can use blockSignals instead (avoids duplicating code for connect/disconnect)
        // (For the KDE interface, this is not even necessary)
	// Ok! Thank you (J.Anders)

	beambutton_->setOn (state & STAT_BEAMED);
	dotbutton_->setOn (state & STAT_SINGLE_DOT);
	ddotbutton_->setOn (state & STAT_DOUBLE_DOT);
	tiebutton_->setOn (state & STAT_TIED);
	slurbutton_->setOn (state & STAT_SLURED);
	tripletbutton_->setOn (state & STAT_TRIPLET);
	staccatobutton_->setOn (state & STAT_STACC);
	sforzatobutton_->setOn (state & STAT_SFORZ);
	portatobutton_->setOn (state & STAT_PORTA);
	strong_pizzicatobutton_->setOn (state & STAT_STPIZ);
	sforzandobutton_->setOn (state & STAT_SFZND);
	fermatebutton_->setOn (state & STAT_FERMT);

	stemUpbutton_->setOn(state & STAT_STEM_UP);
	stemDownbutton_->setOn(!(state & STAT_STEM_UP));
        offs_buttons_[0]->setOn(state & STAT_CROSS);
	if (state & STAT_CROSS) {
            actualOffs_ = 1;
        }
        offs_buttons_[1]->setOn(state & STAT_FLAT);
	if (state & STAT_FLAT) {
            actualOffs_ = -1;
        }
        offs_buttons_[3]->setOn(state & STAT_DCROSS); 
	if (state & STAT_DCROSS) {
            actualOffs_ = 2;
        }
        offs_buttons_[4]->setOn(state & STAT_DFLAT); 
	if (state & STAT_DFLAT) {
            actualOffs_ = -2;
        }
        offs_buttons_[2]->setOn(state & STAT_NATUR);
	if (state & STAT_NATUR) {
            actualOffs_ = 0;
        }
	if (!(state & ACC_MASK)) {
	    actualOffs_ = UNDEFINED_OFFS;
	}
	main_props_.dotcount = (state & DOT_MASK);
	main_props_.tied = (state & STAT_TIED);
	main_props_.staccato = (state & STAT_STACC);
	main_props_.sforzato = (state & STAT_SFORZ);
	main_props_.portato  = (state & STAT_PORTA);
	main_props_.strong_pizzicato = (state & STAT_STPIZ);
	main_props_.sforzato  = (state & STAT_SFZND);
	main_props_.fermate   = (state & STAT_FERMT);
	if (state & STAT_STEM_UP) {
		main_props_.actualStemDir = STEM_DIR_UP;
	}
	else if (state & STEM_DIR_DOWN) {
		main_props_.actualStemDir = STEM_DIR_DOWN;
	}
	else {
		main_props_.actualStemDir = STEM_DIR_AUTO;
	}
	main_props_.noteBody = (state & BODY_MASK);
	switch (main_props_.noteBody) {
		case STAT_BODY_CROSS: crossDrumBu_->setOn(true); break;
		case STAT_BODY_CROSS2: cross2DrumBu->setOn(true); break;
		case STAT_BODY_CIRCLE_CROSS: crossCricDrumBu_->setOn(true); break;
		case STAT_BODY_RECT: rectDrumBu_->setOn(true); break;
		case STAT_BODY_TRIA: triaDrumBu_->setOn(true); break;
		default: crossDrumBu_->setOn(false);
			 cross2DrumBu->setOn(false);
			 crossCricDrumBu_->setOn(false);
			 rectDrumBu_->setOn(false);
			 triaDrumBu_->setOn(false);
	}
}

void NMainFrameWidget::playButtonReset() {
	playbutton_->setOn(false);
}

/*-----------------------------(re-)storing ----------------------------------------*/


void NMainFrameWidget::writeStaffs(const char *fname) {
	if (playing_) return;
#ifdef WITH_TSE3
	kbbutton_->setOn(false);
#endif
	if (fhandler_->writeStaffs(fname , &staffList_, this, true)) {
		setEdited(false);
	}
}

void NMainFrameWidget::autosave(int nr) {
	if (playing_) return;
	QString savname;
	if (actualFname_.isNull()) {
		savname.sprintf("unnamed%d.not.sav", nr);
		fhandler_->writeStaffs(savname, &staffList_, this, false);
	}
	else {
		fhandler_->writeStaffs(actualFname_ + QString(".sav"), &staffList_, this, false);
	}
}
	

bool NMainFrameWidget::readStaffs(const char *fname) {
	NVoice *voice_elem;
	if (playing_) return false;
#ifdef WITH_TSE3
	kbbutton_->setOn(false);
#endif

	if (!fhandler_->readStaffs(fname , &voiceList_, &staffList_, this)) {
		return false;
		
	}
	setEdited(false);
	staffCount_ = staffList_.count();

	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		voice_elem->paperDimensiones(width());
	}
	currentStaff_ = staffList_.first();
	currentStaff_->setActual(true);
	currentVoice_ = currentStaff_->getVoiceNr(0);
	enableCriticalButtons(true);
	voiceDisplay_->setMax(currentStaff_->voiceCount());
	voiceDisplay_->setVal(0);
	lastYHeight_ = voiceList_.last()->getStaff()->getBase()+voiceList_.last()->getStaff()->underlength_;
	computeMidiTimes(false);
	selectedSign_ = 0;
	NVoice::resetUndo();
	main_props_.actualLength = 3*32;
	setButton(3);
	main_props_.tp->setYPosition(-TOP_BOTTOM_BORDER);
	main_props_.directPainter->setYPosition(-TOP_BOTTOM_BORDER);
	main_props_.p->setYPosition(-TOP_BOTTOM_BORDER);
	cleanupSelections();
	return true;
}

/*---------------------------- positioning ------------------------------------------*/

void NMainFrameWidget::reposit() {
	NPositStr *posit;
	QList<NPositStr> plist;
	int min_time;
	int maxwidth;
	int current_xpos = LEFT_SPACE;
	int num_positions;
	bool only_playables;
	int sequNr = 0;
	NStaff *staff_elem;
	myTime_ = 0;

	if (playing_) return;

	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		staff_elem->startRepositioning();
	}
	while(1) {
		plist.clear();
		num_positions = 0;
		min_time = (1 << 30);
		maxwidth = 0;
		for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
			staff_elem->getElementsAfter(&plist, myTime_, &num_positions, &min_time, &maxwidth);
		}
		if (!num_positions) {
			lastBarNr_ = voiceList_.first()->getLastBarNr();
			oldLastXpos_ = lastXpos_;
			lastXpos_ = current_xpos;
			repaintAll();
			return;
		}
		only_playables = true;
		for (posit = plist.first(); posit; posit = plist.next()) {
			if (posit->ev_time == min_time && !(posit->ev_type & PLAYABLE)) only_playables = false;
		}
		for (posit = plist.first(); posit; posit = plist.next()) {
			if (only_playables) {
				if (posit->ev_time == min_time)  {
					posit->from->placeAt(current_xpos, sequNr);
				}
			}
			else {
				if (posit->ev_time == min_time && !(posit->ev_type & PLAYABLE))   {
					posit->from->placeAt(current_xpos, sequNr);
				}
			}
		}
		current_xpos += maxwidth;
		++sequNr;
		if (only_playables)
			myTime_ = min_time+1;
	}
}

void NMainFrameWidget::computeMidiTimes(bool insertBars) {
	NVoice *voice_elem;
	for (voice_elem = voiceList_.first(); voice_elem; voice_elem = voiceList_.next()) {
		voice_elem->getStaff()->staff_props_.measureLength = 4*3*32;
		voice_elem->computeMidiTime(insertBars);
	}
}

/*------------------------------- selection ---------------------------------------*/

int NMainFrameWidget::checkAllStaffs(const QRect p, const QRect p0, int *state, bool *playable) {
	int val;
	NStaff *staff_elem;
	if (playing_) return -1;
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		if ((val = staff_elem->checkElement(p, state, playable)) > 0) {
			if (staff_elem != currentStaff_) {
				currentStaff_->setActual(false);
				currentVoice_->release();
				currentStaff_->draw(leftx_, leftx_ + nettoWidth_);
				currentStaff_ = staff_elem;
				currentVoice_ = staff_elem->getActualVoice();
				enableCriticalButtons(currentVoice_->isFirstVoice());
				currentStaff_->setActual(true);
				voiceDisplay_->setMax(currentStaff_->voiceCount());
				voiceDisplay_->setVal(staff_elem->getActualVoiceNr()+1);
			}
			manageToolElement();
			return val;
		}
	}
	if (checkStaffIntersection(p0)) {
		return -1;
	}
	return -1;
}

bool NMainFrameWidget::checkStaffIntersection(const QRect p) {
	if (playing_) return false;
	NStaff *staff_elem;
	for (staff_elem = staffList_.first(); staff_elem; staff_elem = staffList_.next()) {
		if (staff_elem->intersects(p)) {
			if (staff_elem != currentStaff_) {
				currentStaff_->setActual(false);
				currentVoice_->release();
				currentStaff_->draw(leftx_, leftx_ + nettoWidth_);
				currentStaff_ = staff_elem;
				currentVoice_ = staff_elem->getActualVoice();
				enableCriticalButtons(currentVoice_->isFirstVoice());
				currentStaff_->setActual(true);
				currentStaff_->draw(leftx_, leftx_ + nettoWidth_);
				voiceDisplay_->setMax(currentStaff_->voiceCount());
				voiceDisplay_->setVal(staff_elem->getActualVoiceNr()+1);
#ifdef WITH_TSE3
				NResource::mapper_->setEchoChannel(currentStaff_->getChannel(), currentStaff_->getVoice());
#endif
				return true;
			}
		}
	}
	return false;
}

void NMainFrameWidget::nextElement() {
	if (playing_) return;
	int val, state;
	val = currentVoice_->makeNextElementActual(&state);
	if (editMode_) {
		setButton(NResource::noteLength2Button_(val));
		stateButtonChange(state);
		main_props_.actualLength = val;
	}
	manageToolElement();
	repaintAll();
}


void NMainFrameWidget::prevElement() {
	if (playing_) return;
	int val, state;
	val = currentVoice_->makePreviousElementActual(&state);
	if (editMode_) {
		setButton(NResource::noteLength2Button_(val));
		stateButtonChange(state);
		main_props_.actualLength = val;
	}
        manageToolElement();
	repaintAll();
}

void NMainFrameWidget::undo() {
	NVoice *ref;
	if ((ref = NVoice::undoPossible()) == 0) return;
	ref->undo();
	computeMidiTimes(false);
	reposit();
	repaint();
}

void NMainFrameWidget::redo() {
	NVoice *ref;
	if ((ref = NVoice::redoPossible()) == 0) return;
	ref->redo();
	computeMidiTimes(false);
	reposit();
	repaint();
}

/*------------------------------ TSE3 -----------------------------------------------*/
void NMainFrameWidget::createTSE3() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	kbbutton_->setOn(false);
	tse3Handler_->createTSE3(&voiceList_);
#endif

}
void NMainFrameWidget::playSong() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	kbbutton_->setOn(false);
	tse3Handler_->playSong();
#endif
}

void NMainFrameWidget::writeTSE3() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	if (playing_) return;
	kbbutton_->setOn(false);
	QString fileName = this->checkFileName(KFileDialog::getSaveFileName( QString::null, tse3_file_pattern, this ),".tse3");
	if (fileName.isNull() )  return;

	if (!tse3Handler_->writeTSE3(QFile::encodeName(fileName)))
		KMessageBox::sorry
		  (this,
		   i18n("Error writing file \"%1\".").arg(fileName),
		   kapp->makeStdCaption(i18n("Write TSE3"))
		  );
#endif
}

void NMainFrameWidget::TSE3Filter() {
#ifdef WITH_TSE3
	tse3Handler_->initFiltering();
#endif
}

void NMainFrameWidget::importRecording() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	if (KMessageBox::warningYesNo
	     (this,
	      i18n("This overrides the current voice! Continue?"),
	      kapp->makeStdCaption(i18n("Import Recording")),
	      i18n("&Import Recording"),
	      i18n("&Cancel")
	     )
	    == KMessageBox::No
	   ) return;
	tse3Handler_->TSE3Rec2Staff(currentStaff_, currentVoice_);
	setEdited();
#endif
}

#ifdef WITH_TSE3
void NMainFrameWidget::createStaffFromPhraseEdit(TSE3::PhraseEdit *phraseEdit) {
	tse3Handler_->TSE3PhraseEditToStaff(phraseEdit, currentVoice_);
	setEdited();
}

bool NMainFrameWidget::stillRecording() {
    return recordButton_->isChecked();
    }
#endif

void NMainFrameWidget::completeRecording(bool ok) {
#ifdef WITH_TSE3
	if (!ok) {
		recordButton_->setOn(false);
		return;
	}
	setEdited();
	computeMidiTimes(false);
        reposit();
	setScrollableNotePage();
	repaint();
#endif
}

void NMainFrameWidget::TSE3toScore() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	if (playing_) return;
	kbbutton_->setOn(false);
	newPaper();
	tse3Handler_->TSE3toScore(&staffList_, &voiceList_);
#endif
}

void NMainFrameWidget::completeTSE3toScore(bool ok) {
#ifdef WITH_TSE3
	NVoice *voice;
	double partTime, readyPart;
	int i;
	int maxmiditime = 0, difftime;
	if (!ok)  {
		KMessageBox::error
		  (this,
		   i18n("Error creating Score"),
		   kapp->makeStdCaption(i18n("Creating Score"))
		  );
		NResource::progress_->hide();
		return;
	}
	for (voice = voiceList_.first(); voice; voice = voiceList_.next()) {
		voice->paperDimensiones(paperWidth_);
	}
	currentVoice_= voiceList_.first();
	currentStaff_ = currentVoice_->getStaff();
	enableCriticalButtons(true);
	staffCount_ = staffList_.count();
	voiceDisplay_->setMax(currentStaff_->voiceCount());
	lastYHeight_ = voiceList_.last()->getStaff()->getBase()+voiceList_.last()->getStaff()->underlength_;
	currentStaff_->setActual(true);
	setEdited(false);
	computeMidiTimes(false);
	for (voice = voiceList_.first(); voice; voice = voiceList_.next()) {
		if (voice->getMidiEndTime() > maxmiditime) maxmiditime = voice->getMidiEndTime();
	}
	maxmiditime += 3*128;

	readyPart = 100.0 * FIRST_PART_TIME;
	partTime = (double) voiceList_.count() / (100.0 * SECOND_PART_TIME) + 1.e-20;
	for (i = 0, voice = voiceList_.first(); voice; voice = voiceList_.next(), i++) {
		NResource::progress_->setValue((int) (readyPart + (double) (i+1) / partTime));
		difftime = maxmiditime - voice->getMidiEndTime();
		voice->handleEndAfterMidiImport(difftime);
	}
	computeMidiTimes(false);
	tse3Handler_->insertTimeAndKeySigs(&voiceList_);
	computeMidiTimes(false);
	setScrollableNotePage();
        reposit();
	scrollx_->setValue(0); // includes repaint()
	cleanupSelections();
	tempo_ = INTERNAL_LENGTH_FAC * DEFAULT_TEMPO;
	NResource::progress_->hide();
#endif
}

void NMainFrameWidget::TSE3MidiOut() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	if (playing_) return;
	kbbutton_->setOn(false);
	QString fileName = this->checkFileName(KFileDialog::getSaveFileName(QString::null, midi_file_pattern, this), ".mid");
	if (fileName.isNull() )  return;
	if (!tse3Handler_->TSE3MidiOut(fileName.ascii()))
		KMessageBox::sorry
		  (this,
		   i18n("Error writing file \"%1\".").arg(fileName),
		   kapp->makeStdCaption(i18n("TSE3 MIDI out"))
		  );
#endif
}

void NMainFrameWidget::TSE3MidiIn() {
#ifdef WITH_TSE3
	if (recordButton_->isChecked()) return;
	if (playing_) return;
	kbbutton_->setOn(false);
	QString fileName = KFileDialog::getOpenFileName( QString::null, midi_file_pattern, this );
	if (fileName.isNull() )  return;
	if (!tse3Handler_->TSE3MidiIn(fileName.ascii()))
		KMessageBox::sorry
		  (this,
		   i18n("File read error \"%1\".").arg(fileName),
		   kapp->makeStdCaption(i18n("TSE3 MIDI in"))
		  );
	repaint();
#endif
}

void NMainFrameWidget::TSE3record(bool on) {
#ifdef WITH_TSE3
	if (!on) {
		importRecording();
		return;
	}
	if (playing_) return;
	kbbutton_->setOn(false);
	if (!tse3Handler_->TSE3record(currentStaff_->getChannel())) {
		disconnect(recordButton_, SIGNAL(toggled(bool)), this, SLOT(TSE3record(bool)));
		recordButton_->setOn(false);
		connect(recordButton_, SIGNAL(toggled(bool)), this, SLOT(TSE3record(bool)));
	}
#endif
}
void NMainFrameWidget::readTSE3() {
#ifdef WITH_TSE3
	if (playing_) return;
	kbbutton_->setOn(false);
	QString fileName = KFileDialog::getOpenFileName( QString::null, tse3_file_pattern, this);
	if (fileName.isNull() )  return;
	if (!tse3Handler_->readTSE3(fileName.ascii()))
		KMessageBox::sorry
		  (this,
		   i18n("File read error \"%1\".").arg(fileName),
		   kapp->makeStdCaption(i18n("Read TSE3"))
		  );
#endif
}

/*----------------------------- modification -------------------------------------*/

void NMainFrameWidget::moveUp() {
	if (playing_) return;
	currentVoice_->moveUp(1);
	reposit();
	setEdited();
}

void NMainFrameWidget::moveDown() {
	if (playing_) return;
	currentVoice_->moveDown(true);
	reposit();
	setEdited();
}

void NMainFrameWidget::deleteElem(bool backspace) {
	int val, state;
	if (playing_) return;
	val = currentVoice_->deleteActualElem(&state, backspace);
	if (editMode_) {
		setButton(NResource::noteLength2Button_(val));
		stateButtonChange(state);
		main_props_.actualLength = val;
	}
	computeMidiTimes(false);
	reposit();
	setEdited(val != -1);
}

/*----------------------------- KDE main window -------------------------------------*/

NMainWindow::NMainWindow(QWidget *parent, const char *name )
    : KMainWindow(parent, name)
{
	QWidget * w = new NMainFrameWidget( actionCollection(), false, this  );
	NResource::windowList_.append(this);
	setCentralWidget( w );
	//w->setFocus();
	connect( w, SIGNAL( caption( const QString & ) ), this, SLOT( slotCaption( const QString & ) ) );

	// Now build the GUI
	createGUI();

}

void NMainWindow::slotCaption( const QString & s )
{
	setCaption( s );
}



NMainFrameWidget * NMainWindow::mainFrameWidget() const
{
	return static_cast<NMainFrameWidget *>( centralWidget() );
}

/*------------------------------------- tools ------------------------------------*/

QString NMainFrameWidget::checkFileName(QString fileName, char *extension)
{
	if (fileName.find(extension, -strlen(extension), true) < 0 && !fileName.isNull())
	    fileName+=extension;
	if (!fileName.isNull() )
		if (!access(fileName, F_OK))  {
			if (KMessageBox::warningYesNo
			     (0,
			      i18n("File \"%1\" already exists! Overwrite?").arg(fileName),
			      kapp->makeStdCaption(i18n("File already exists"))
			     )
		      == KMessageBox::No
			   ) return 0;
		}
	return fileName;
}

/*------------------------------------- help -------------------------------------*/

void NMainFrameWidget::showTipOfTheDay() {
#if KDE_VERSION >= 220
	KTipDialog::KTipDialog::showTip(locate("data", "noteedit/tips"), true);
#else
	tipFrm( 0, NResource::tipNo_ );
#endif
}

#include "mainframewidget.moc"
