/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#include <math.h>
#include <stdio.h>
#include "muselement.h"
#include "resource.h"
#include "transpainter.h"
#include "clef.h"
#include "chord.h"


main_props_str::main_props_str() {
			dotcount = 0;
			tied = staccato = sforzato = portato = strong_pizzicato = 
			sforzando = fermate = hidden = false;
			noteBody = 0;
			actualLength = 3 * 32;
			actualStemDir = STEM_DIR_AUTO;
			tp              = new NTransPainter();
			tp->setXPosition(-LEFT_RIGHT_BORDER);
			tp->setYPosition(-TOP_BOTTOM_BORDER);
			directPainter              = new NTransPainter();
			directPainter->setXPosition(-LEFT_RIGHT_BORDER);
			directPainter->setYPosition(-TOP_BOTTOM_BORDER);
			p                = new NTransPainter();
			p->setYPosition(-TOP_BOTTOM_BORDER);
		}

main_props_str::~main_props_str() {
			delete tp;
			delete p;
			delete directPainter;
		}


NMusElement::NMusElement(main_props_str *main_props, staff_props_str *staff_props) {
	midiTime_ = status_ = trill_ = dynamic_ = slurY_ = 0;
	staff_props_ = staff_props;
	main_props_ = main_props;
}

void NMusElement::change(NMusElement *elem) {
	status_ = trill_ = dynamic_ = slurY_ = 0;
	staff_props_ = elem->staff_props_;
	status_ = elem->status_;
	trill_ = elem->trill_;
	dynamic_ = elem->dynamic_;
	dynamicAlign_ = elem->dynamicAlign_;
	slurY_ = elem->slurY_;
	main_props_ = elem->main_props_;
}


void NMusElement::computeTriplet() {
	computeTriplet(tripletList_);
}

void NMusElement::computeTriplet(QList<NMusElement> *tripletList) {
#define TRIPLET_DIST 24
	NMusElement *elem, *elem0, *first_chord = 0;
	double sumxi2, sumxi, sumxiyi, sumyi;
	int count;
	double m, n, nn, don, x0, xdist;
	int xstart, xend;
	bool isFirst;
	double tritexn, tt;

	sumxi2 = sumxi = sumxiyi = sumyi = count = 0;
	x0 = tripletList->first()->getTopX2();
	for (elem = tripletList->first(); elem; elem = tripletList->next()) {
		if (elem->getType() == T_CHORD) {
			first_chord = elem;
			xdist = elem->getTopX2() - x0;
			sumxi2 += xdist  * xdist;
			sumyi += elem->getTopY2();
			sumxi += xdist;
			sumxiyi += xdist * elem->getTopY2();
			count++;
		}
	}
	
	if (count == 0 || !first_chord) {
		m = 0.0;
		n = tripletList->first()->getTopY2();
	}
	else if (count < 2) {
		m = 0.0;
		n = first_chord->getTopY2();
	}
	else {
		m = -(sumxi*sumyi-sumxiyi*count)/(don = sumxi2*count-sumxi*sumxi);
		n = (-sumxi*sumxiyi+sumyi*sumxi2)/don;
	}
	isFirst = true;
	tritexn = -1.0e30;
	n = 1.0e30;
	for (elem = tripletList->first(); elem; elem = tripletList->next()) {
		xdist = elem->getTopX2();
		nn = (double) elem->getTopY2() - m * (double) xdist - TRIPLET_DIST;
		tt = (double) elem->getBotY() - m * (double) xdist;
		if (isFirst) {
			isFirst = false;
			xstart = elem->getTopX2();
		}
		if (nn < n) n = nn;
		if (tritexn < tt) tritexn = tt;
	}
	xend = tripletList->last()->getTopX2();
	elem0 = tripletList->first();
	for (elem = tripletList->next(); elem; elem = tripletList->next()) {
		elem0->setTripletParams(tripletList, false, m, n, tritexn, xstart, xend);
		elem0->calculateDimensionsAndPixmaps();
		elem0 = elem;
	}
	elem0->setTripletParams(tripletList, true, m, n, tritexn, xstart, xend);
	elem0->calculateDimensionsAndPixmaps();
}

void NMusElement::breakTriplet() {
	NMusElement *elem;
	for (elem = tripletList_->first(); elem; elem = tripletList_->next()) {
		elem->status_ &= (~(STAT_TRIPLET | STAT_LAST_TRIPLET));
		elem->changeLength(elem->getSubType());
	}
}

void NMusElement::unsetTriplet() {
	status_ &= (~(STAT_TRIPLET | STAT_LAST_TRIPLET));
}

void NMusElement::reposit(int xpos, int sequNr) {
	xpos_ = xpos;
	sequNr_ = sequNr;
	calculateDimensionsAndPixmaps();
}


NMusElement::~NMusElement() {
}

bool NMusElement::intersects (const QRect p) const {
	return bbox_.intersects(p);
}



QString *NMusElement::computeTeXTriplet(NClef *clef) {
	NMusElement *elem;
	QString *s;
	int line, delta = 0;
	if (!(status_ & STAT_TRIPLET)) return 0;
	if (tripletList_ == 0) {
		NResource::abort("internal error: NMusElement::computeTeX: tripletList_ == 0");
	}
	if (this == tripletList_->first()) {
		for (elem = tripletList_->first(); delta == 0 && elem; elem = tripletList_->next()) {
			if (elem->getType() != T_CHORD) continue;
			if (elem->getSubType() > 3*32) continue;
			if (!(elem->status_ & STAT_STEM_UP)) delta = -4;
		}
		s = new QString();
		line = (int) (((staff_props_->base - (trim_ * xpos_ + triTeXn_))) * 2.0 + 0.5) / LINE_DIST;
		line += delta;
		if (line < MINLINE) line = MINLINE;
		else if (line > MAXLINE) line = MAXLINE;
		s->sprintf("\\downtrio{%c}{%d}{%d}", clef->line2TexTab_[line+LINE_OVERFLOW],
				tripletList_->last()->sequNr_ - tripletList_->first()->sequNr_, (int) (atan(-trim_) * 180.0 / 3.1415 / 2.0));
		return s;
	}
	return 0;
}

	
