/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#ifndef MUSELEMENT_H

#define MUSELEMENT_H
#include <qpoint.h>
#include <qrect.h>
#include "resource.h"

#define LINE_DIST 21
#define LINE_OVERFLOW 12
#define MINLINE (-LINE_OVERFLOW)
#define MAXLINE (8 + LINE_OVERFLOW)
#define STAFF_HIGHT (4*LINE_DIST)

#define T_CHORD (1 << 0)
#define T_REST (1 << 1)
#define T_SIGN (1 << 2)
#define T_CLEF (1 << 3)
#define T_KEYSIG (1 << 4)
#define T_TIMESIG (1 << 5)

#define PLAYABLE (T_CHORD | T_REST)

// please study uiconnect before you make any clef changes!
#define TREBLE_CLEF (1 << 0)
#define BASS_CLEF (1 << 1)
#define ALTO_CLEF (1 << 2)
#define TENOR_CLEF (1 << 3)
#define DRUM_CLEF (1 << 4)
#define DRUM_BASS_CLEF (1 << 5)

// signs:

#define SIMPLE_BAR (1 << 6)
#define REPEAT_OPEN (1 << 7)
#define REPEAT_CLOSE (1 << 8)
#define REPEAT_OPEN_CLOSE (1 << 9)
#define SPECIAL_ENDING1 (1 << 10)
#define SPECIAL_ENDING2 (1 << 11)
#define TEMPO_SIGNATURE ((1 << 12) + 1)
#define VOLUME_SIG      ((1 << 12) + 2)
#define PROGRAM_CHANGE  ((1 << 12) + 3)
#define TRILL		((1 << 12) + 4)
#define LNTRILL		((1 << 12) + 5)
#define DYNAMIC		((1 << 12) + 6)

// special rest
#define MULTIREST	(1 << 12)

#define SPECIAL_ENDING (SPECIAL_ENDING1 | SPECIAL_ENDING2)
#define BAR_SYMS (REPEAT_OPEN | REPEAT_CLOSE | SIMPLE_BAR | REPEAT_OPEN_CLOSE)

#define TRIPLET_HEIGHT 15
#define THREE_DIST 39

#define DRAW_INDIRECT 0
#define DRAW_DIRECT_BLACK (1 << 0)
#define DRAW_DIRECT_RED (1 << 1)
#define DRAW_INDIRECT_GREY (1 << 2)
#define DRAW_NO_HIDDEN_REST (1 << 3)

#define STEM_POL_UP 0
#define STEM_POL_INDIVIDUAL 1
#define STEM_POL_DOWN 2

class muselement;
class NClef;
class NKeySig;
class NChord;
class NTransPainter;
class NNumberDisplay;

class staff_props_str {
	public:
		int base;
		int lyricsdist;
		bool is_actual;
		int measureLength;
		NKeySig *actual_keysig;
};

class main_props_str {
	public:
		main_props_str();
		~main_props_str();
		int dotcount;
		bool hidden;
		bool tied;
		bool staccato;
		bool sforzato;
		bool portato;
		bool strong_pizzicato;
		bool sforzando;
		bool fermate;
		unsigned int noteBody;
		int actualLength;
		int actualStemDir;
		NTransPainter *p;
		NTransPainter *tp;
		NTransPainter *directPainter;
		NNumberDisplay *voiceDisplay;
		QFont scaledText_;
		QFont scaledItalic_;
		QFont scaledBoldItalic_;
		QFont scaledBold_;
		QFont scaledBold2_;
		float zoom;
};

class NNote {
	public:
		QPixmap *bodyPixmap;
		QPixmap *redBodyPixmap;
		QPixmap *greyBodyPixmap;
		char line;
		char offs;
		QPoint nbase_draw_point;
		QPoint acc_draw_point;
		QRect point_pos1;
		QRect point_pos2;
		NNote *tie_forward, *tie_backward;
		char acc_offs;
		char needed_acc;
		char acc_TeX_pos;
		unsigned int status;
#define STAT_SINGLE_DOT   (1 << 0)
#define STAT_DOUBLE_DOT   (1 << 1)
#define DOT_MASK     (STAT_SINGLE_DOT | STAT_DOUBLE_DOT)

#define STAT_HIDDEN   (1 << 2) /* conflics "STAT_CROSS", but can only occur in rests */

#define STAT_CROSS   (1 << 2)
#define STAT_FLAT    (1 << 3)
#define STAT_DCROSS  (1 << 4)
#define STAT_DFLAT   (1 << 5)
#define STAT_NATUR   (1 << 6)
#define STAT_NO_ACC  0

#define ACC_MASK     (STAT_CROSS | STAT_FLAT | STAT_DCROSS | STAT_DFLAT | STAT_NATUR)

#define STAT_BEAMED  (1 << 7)
#define STAT_SLURED  (1 << 8)
#define STAT_PART_OF_SLUR  (1 << 9)
#define STAT_TRIPLET (1 << 10)
#define STAT_LAST_TRIPLET (1 << 11)
#define STAT_STEM_UP (1 << 12)
#define STAT_STEM_UP_BEFORE_BEAM (1 << 13)
#define STAT_STACC   (1 << 14)
#define STAT_TIED    (1 << 15)
#define STAT_PART_OF_TIE (1 << 16)
#define STAT_FORCE   (1 << 17)
#define STAT_SHIFTED (1 << 18)
#define STAT_VIRTUAL (1 << 19)
// Note: If you want to change the values of sforzato - sforzando, study function setAccent(*) at first!
// sforzato
#define STAT_SFORZ   (1 << 20)
// portato
#define STAT_PORTA   (1 << 21)
// strong pizzicato
#define STAT_STPIZ   (1 << 22)
// sforzando
#define STAT_SFZND   (1 << 23)
// fermate
#define STAT_FERMT   (1 << 24)
// body as cross
#define STAT_BODY_CROSS (1 << 25)
// body as alternative cross
#define STAT_BODY_CROSS2 (1 << 26)
// body as cross with circle
#define STAT_BODY_CIRCLE_CROSS (1 << 27)
// body as rectangle
#define STAT_BODY_RECT (1 << 28)
// body as triangle
#define STAT_BODY_TRIA (1 << 29)



#define CHORD_STAT_PART ( STAT_SINGLE_DOT | STAT_DOUBLE_DOT | STAT_BEAMED | STAT_SLURED | \
	 STAT_PART_OF_SLUR | STAT_TRIPLET | STAT_LAST_TRIPLET | STAT_STEM_UP | STAT_STACC | \
	 STAT_SFORZ | STAT_PORTA | STAT_STPIZ | STAT_SFZND | STAT_FERMT )

#define NOTE_STAT_PART (~CHORD_STAT_PART)

#define BODY_MASK ( STAT_BODY_CROSS | STAT_BODY_CROSS2 | STAT_BODY_CIRCLE_CROSS |\
			STAT_BODY_RECT | STAT_BODY_TRIA )

#define SET_STATUS(condition, stat_var, stat_bit) if (condition) {stat_var |= stat_bit;} else {stat_var &= (~stat_bit);}
		QPoint tie_start_point_up, tie_start_point_down;
		QPoint tie_forward_point_up, tie_forward_point_down;
		QPoint tie_back_point_up, tie_back_point_down;
		NChord *chordref;
		short TeXTieNr;
};

class NMusElement {
	public :
		NMusElement(main_props_str *main_props, staff_props_str *staff_props);
		void change(NMusElement *elem);
		unsigned int status_;
		int trill_;
		int dynamic_;
		bool dynamicAlign_;
		int slurY_;
		virtual NMusElement *clone() = 0;
		virtual ~NMusElement();
		virtual void draw(int flags = 0) = 0;
		virtual void setStaffProps(staff_props_str *staff_props) {staff_props_ = staff_props;}
		virtual void setMainProps(main_props_str *main_props) {main_props_ = main_props;}
		virtual void moveUp(int, int, NKeySig *) {};
		virtual void moveDown(int, int, NKeySig *) {};
		virtual void changeLength(int) {};
		virtual void changeBody(unsigned int) {};
		virtual void changeOffs(int, NKeySig *) {}
		virtual void setTied(bool) {};
		virtual int getSubType() const {return 0;}
		virtual QRect *getBbox () {return &bbox_;} 
		virtual QList<NNote> *getNoteList() {return 0;}
		virtual bool deleteNoteAtLine(int, int) {return false;}
		virtual void deletePart(NNote *) {};
		virtual NNote *searchLine(int , int) { return 0;}
		virtual NNote *insertNewNote(int line, int offs, int voices_stem_policy, unsigned int status) {return 0;}
		virtual void insertNewNote(NNote *) {};
		void reposit(int xpos, int sequNr_);
		int getXpos() {return xpos_;}
		int getSequNr() {return sequNr_;}
		void setActual(bool ac) {actual_ = ac;}
		virtual int getType () const  = 0;
		bool intersects(const QRect p) const;
		virtual void setDotted(int) {}
		virtual int getMidiLength () const {return 0;}
		virtual bool lastBeamed() {return false;}
		void breakTriplet();
		virtual void breakBeames() {}
		virtual QPoint *getTopY() {return 0;}
		virtual int getTopY2() {return 0;}
		virtual int getTopX2() {return 0;}
		virtual double getBotY() {return 0.0;}
		void computeTriplet();
		static void computeTriplet(QList<NMusElement> *tripletList);
		virtual void setTripletParams(QList<NMusElement> *, bool, double, double, double, int, int) {}
		void changeTripletList(QList<NMusElement> *triList) {tripletList_ = triList;}
		void unsetTriplet();
		QList<NMusElement> *getTripletList() {return tripletList_;}
		void resetTripletFlag() { status_ &= (~((STAT_TRIPLET | STAT_LAST_TRIPLET))); tripletList_ = 0;}
		int midiTime_;
		virtual void calculateDimensionsAndPixmaps() = 0;
		QString *computeTeXTriplet(NClef *clef);
		bool isFirstInTriplet() {return tripletList_->first() == this;}
	protected:
		virtual int computeMidiLength() const {return 0;}
		bool actual_;
		int pixmapHeight_;
		int pixmapWidth_;
		QRect bbox_;
		staff_props_str *staff_props_;
		main_props_str *main_props_;
		int xpos_;
		int sequNr_;
		int midiLength_;
		double trim_; double trin_;
		double triTeXn_;
		int xstart_, xend_;
		QList<NMusElement> *tripletList_;
};

#endif // MUSELEMENT_H
