/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#ifndef MUSIXTEX_H

#define MUSIXTEX_H

#include <fstream>
#include "muselement.h"

class NMusElement;
class NVoice;
class NStaff;
class exportFrm;
struct trill_descr_str;
struct dynamics_descr_str {
	char *lastDynSym;
	int dynEndPos;
};

class NExportError {
	public:
		NExportError(int r, int m) {reason = r; measure = m;}
		int reason;
#define ERR_128             1
#define ERR_IRR_KEY         2
#define ERR_TOO_MANY_TIES   3
#define ERR_TOO_MANY_SLURS  4
#define ERR_NESTED_TILLS    5
#define ERR_TOO_MANY_TRILLS 6
#define ERR_BEAM_COUNT      7
		int measure;
};

class NMusiXTeX {
	public:
		NMusiXTeX();
		void exportStaffs(QString filen, QList<NStaff> *stafflist, exportFrm *form, NMainFrameWidget *mainWidget);
		void doExport();
	private:
		void generate(int staff_nr, int real_staff_nr, int staffs_to_export, const char *extraDelimiter,
			 NMusElement *elem, NStaff *staff_elem, NVoice *ac_voice);
		ofstream out_;
		unsigned int staffCount_;
		NStaff **listOfStaffsToExport_;
		bool lastSignWasBar_;
		QString lyrics2TeX(QString *lyrics);
		int *beamNrs_;
		bool elemsWritten_;
		unsigned int slurTiePool_;
		int barNr_;
		int maxTies_;
		int maxSlurs_;
		int lastBarNr_;
		bool spare_;
		bool limitMeasures_;
		int maxMeasuresPerLine_;
		int maxFlags_;
		unsigned int maxStaffsAndBeams_;
		unsigned int trillPool_;
		unsigned int beamPool_;
		struct trill_descr_str *trill_descr_;
		QList<NExportError> badMeasures_;
		exportFrm *exportDialog_;
		bool madeSeek_;
		QList<NStaff> *staffList_;
		QList<NNote> shiftes_notes_, non_shifted_notes_;
		long lastBarPos_;
		int newTempo_;
		struct dynamics_descr_str *dysymDescr_;
		QString fileName;
		NMainFrameWidget *mainWidget_;
};

#endif // MUSIXTEX_H
