/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#ifndef RESOURCE_H

#define RESOURCE_H

#include <kapp.h>

#include <qcolor.h>
#include <qlist.h>
#include <qstring.h>

#define AUTOSAVE_INTERVAL_MIN  1
#define AUTOSAVE_INTERVAL_MAX 64
#define TURN_OVER_MIN 0
#define TURN_OVER_MAX 300

#define DEFAULT_OVERLENGTH 60
#define DEFAULT_UNDERLENGTH DEFAULT_OVERLENGTH
#define DEFAULT_LYRICSDIST 60
#define LEFT_RIGHT_BORDER 10
#define TOP_BOTTOM_BORDER 10
#define LEFT_SPACE 25

#define NULL_LINE -111
#define COUNT_CHORDBUTTONS 9

#define STEM_DIR_AUTO 0
#define STEM_DIR_UP 1
#define STEM_DIR_DOWN 2

#define PREFERRED_ZOOM_VAL 8
#define START_WIDTH 600
#define START_HEIGHT 400
#define WINDOWXY_INCR 20
#define MAXWINDOWXY 400

#define FIRST_PART_TIME 0.6
#define SECOND_PART_TIME 0.2
#define THIRD_PART_TIME 0.1
#define FOURTH_PART_TIME 0.1

class staff_props_str;
class NMainWindow;
class NMidiMapper;
class NVoice;
class NMainFrameWidget;
class NMusElement;
class NKeySig;
class NClef;
class expWrn;

class KProgress;
class QPen;
class QBrush;
class QPixmap;
class QFont;

class QLineEdit;
class QMultiLineEdit;

class main_props_str;
class NResource : public QObject  {
	Q_OBJECT
	public:
		NResource();
		~NResource();
		static NResource *nresourceobj_;
		static QBrush backgroundBrush;
		static QBrush selectionBackgroundBrush;
		static QBrush tempoSignatureBrush;
		static QBrush selectedTempoSignatureBrush;
		static QBrush blackBrush;
		static QBrush redBrush;
		static QPen noPen;
		static QPen staffPen;
		static QPen selectedStaffPen;
		static QPen barPen;
		static QPen selectedBarPen;
		static QPen barNumberPen;
		static QPen selectedBarNumberPen;
		static QPen tempoSignaturePen;
		static QPen selectedTempoSignaturePen;
		static QPen volumeSignaturePen;
		static QPen selectedVolumeSignaturePen;
		static QPen programChangePen;
		static QPen selectedProgramChangePen;
		static QPen specialEndingPen;
		static QPen selectedSpecialEndingPen;
		static QPen staffNamePen;
		static QPen selectedStaffNamePen;
		static QPen lyricPen;
		static QPen whiteWidePen;
		static QPen blackWidePen;
		static QPen redWidePen;
		static QPen greyWidePen;
		static QPen greenPen;
		static QPen redPen;
		static QPen whitePen;
		static QPen blackPen;
		static QPen greyPen;
		static QString resourceDir_;

		static QPixmap *stopIcon_;
		static QPixmap *threePixmap_;

		static QPixmap *time_24Icon_;
		static QPixmap *time_44Icon_;
		static QPixmap *time_34Icon_;
		static QPixmap *time_38Icon_;
		static QPixmap *time_68Icon_;

		static QPixmap *crossIcon_;
		static QPixmap *flatIcon_;
		static QPixmap *naturIcon_;

		static QPixmap *fullPixmap_;
		static QPixmap *fullRedPixmap_;
		static QPixmap *fullGreyPixmap_;
		static QPixmap *fullMagPixmap_;
		static QPixmap *flagPixmap_;
		static QPixmap *flagRedPixmap_;
		static QPixmap *flagGreyPixmap_;
		static int flagPixmapWidth_;
		static QPixmap *flagDownPixmap_;
		static QPixmap *flagDownRedPixmap_;
		static QPixmap *flagDownGreyPixmap_;
		static int flagDownPixmapHeight_;
		static QPixmap *nbasePixmap_;
		static QPixmap *nbaseRedPixmap_;
		static QPixmap *nbaseGreyPixmap_;

		static QPixmap *r128Pixmap_;
		static QPixmap *r128RedPixmap_;
		static QPixmap *r128GreyPixmap_;
		static QPixmap *r128MagPixmap_;
		static QPixmap *r64Pixmap_;
		static QPixmap *r64RedPixmap_;
		static QPixmap *r64GreyPixmap_;
		static QPixmap *r64MagPixmap_;
		static QPixmap *r32Pixmap_;
		static QPixmap *r32RedPixmap_;
		static QPixmap *r32GreyPixmap_;
		static QPixmap *r32MagPixmap_;
		static QPixmap *r16Pixmap_;
		static QPixmap *r16RedPixmap_;
		static QPixmap *r16GreyPixmap_;
		static QPixmap *r16MagPixmap_;
		static QPixmap *r8Pixmap_;
		static QPixmap *r8RedPixmap_;
		static QPixmap *r8GreyPixmap_;
		static QPixmap *r8MagPixmap_;
		static QPixmap *rquarterPixmap_;
		static QPixmap *rquarterRedPixmap_;
		static QPixmap *rquarterGreyPixmap_;
		static QPixmap *rquarterMagPixmap_;
		static QPixmap *rhalfPixmap_;
		static QPixmap *rhalfRedPixmap_;
		static QPixmap *rhalfGreyPixmap_;
		static QPixmap *rhalfMagPixmap_;
		static QPixmap *rfullPixmap_;
		static QPixmap *rfullRedPixmap_;
		static QPixmap *rfullGreyPixmap_;
		static QPixmap *rfullMagPixmap_;
		static QPixmap *brevePixmap_;
		static QPixmap *breveRedPixmap_;
		static QPixmap *breveGreyPixmap_;

		static QPixmap *crossPixmap_;
		static QPixmap *crossRedPixmap_;
		static QPixmap *crossGreyPixmap_;
		static int crossPixWidth_;
		static QPixmap *dcrossPixmap_;
		static QPixmap *dcrossRedPixmap_;
		static QPixmap *dcrossGreyPixmap_;
		static int dcrossPixWidth_;
		static QPixmap *flatPixmap_;
		static QPixmap *flatRedPixmap_;
		static QPixmap *flatGreyPixmap_;
		static int flatPixWidth_;
		static QPixmap *dflatPixmap_;
		static QPixmap *dflatRedPixmap_;
		static QPixmap *dflatGreyPixmap_;
		static int dflatPixWidth_;
		static QPixmap *naturPixmap_;
		static QPixmap *naturRedPixmap_;
		static QPixmap *naturGreyPixmap_;
		static int naturPixWidth_;

		static QPixmap *repOpenPixmap_;
		static QPixmap *repOpenRedPixMap_;
		static QPixmap *repClosePixmap_;
		static QPixmap *repCloseRedPixMap_;
		static QPixmap *repOpenClosePixmap_;
		static QPixmap *repOpenCloseRedPixMap_;

		static QPixmap  *treblePixmap_;
		static QPixmap  *trebleRedPixmap_;
		static QPixmap  *treblepPixmap_;
		static QPixmap  *treblepRedPixmap_;
		static QPixmap  *treblemPixmap_;
		static QPixmap  *treblemRedPixmap_;
		static QPixmap  *bassPixmap_;
		static QPixmap  *bassRedPixmap_;
		static QPixmap  *basspPixmap_;
		static QPixmap  *basspRedPixmap_;
		static QPixmap  *bassmPixmap_;
		static QPixmap  *bassmRedPixmap_;

		static QPixmap  *altoPixmap_;
		static QPixmap  *altoRedPixmap_;
		static QPixmap  *altopPixmap_;
		static QPixmap  *altopRedPixmap_;
		static QPixmap  *altomPixmap_;
		static QPixmap  *altomRedPixmap_;

		static QPixmap  *drumClefPixmap_;
		static QPixmap  *drumClefRedPixmap_;

		static QPixmap  *drumBassClefPixmap_;
		static QPixmap  *drumBassClefRedPixmap_;

		static QPixmap *sforzatoAbPixmap_;
		static QPixmap *sforzatoAbRedPixmap_;
		static QPixmap *sforzatoBePixmap_;
		static QPixmap *sforzatoBeRedPixmap_;
		static QPixmap *portatoPixmap_;
		static QPixmap *portatoRedPixmap_;
		static QPixmap *strong_pizzicatoAbPixmap_;
		static QPixmap *strong_pizzicatoAbRedPixmap_;
		static QPixmap *strong_pizzicatoBePixmap_;
		static QPixmap *strong_pizzicatoBeRedPixmap_;
		static QPixmap *sforzandoPixmap_;
		static QPixmap *sforzandoRedPixmap_;
		static QPixmap *fermateAbPixmap_;
		static QPixmap *fermateAbRedPixmap_;
		static QPixmap *fermateBePixmap_;
		static QPixmap *fermateBeRedPixmap_;
		static QPixmap *trillPixmap_;
		static QPixmap *trillRedPixmap_;

		static QPixmap *perCrossPixmap_;
		static QPixmap *perCrossRedPixmap_;
		static QPixmap *perCrossGreyPixmap_;

		static QPixmap *perCross2Pixmap_;
		static QPixmap *perCross2RedPixmap_;
		static QPixmap *perCross2GreyPixmap_;

		static QPixmap *perCrossCircPixmap_;
		static QPixmap *perCrossCircRedPixmap_;
		static QPixmap *perCrossCircGreyPixmap_;

		static QPixmap *perRectPixmap_;
		static QPixmap *perRectRedPixmap_;
		static QPixmap *perRectGreyPixmap_;

		static QPixmap *perTrianPixmap_;
		static QPixmap *perTrianRedPixmap_;
		static QPixmap *perTrianGreyPixmap_;

		static QFont *text_;
		static expWrn *exportWarning_;

#if KDE_VERSION >= 220
		//  Temporary kludge to be used until the equivalent is implemented in
		//  kdelibs.                                                   -Erik Sigra
		static void detailedWarningDontShowAgain
		 (QWidget *, const QString &, const QString &, const QString &,
		  const QString &, bool notify=true
		 );
#endif

		static void printError(QString s);
		static void printWarning(QString s);
		static void abort( QString s, signed char no = -1 );
		static void setAutosave(bool enable, int intervall);
		static int nbasePixmapHeight_;
		static int nbasePixmapWidth_;
		static int nbasePixmapWidth2_;
		static int noteButton2Ength_[COUNT_CHORDBUTTONS];
		static int noteLength2Button_(int length);



		//  GENERAL

		//  Autosave
		static bool autosaveEnable;
		static unsigned int autosaveInterval;
		static unsigned int turnOverPoint;

		//  Startup
		static bool startupLogo;
		static bool startupLoadLastScore;


		//  showStaffNrs_, showStaffNames_, showAuxLines_ and showDrumToolbar_ are cached versions
		//  kapp->config()->readBoolEntry(QString("ShowStaffNrs"), true),
		//  kapp->config()->readBoolEntry(QString("ShowStaffNames"), true),
		//  kapp->config()->readBoolEntry(QString("ShowAuxLines"), true) and
		//  kapp->config()->readBoolEntry(QString("ShowDrumToolbar"), true)
		//  respectively. They are cached because acces to booleans ought to be
		//  faster than the access method above. This extra cache layer is used
		//  because the values are read in drawing code that is executed many
		//  times. It may not make a difference unless used on slow computers with
		//  large datasets.
		//
		//  Note that the calls to kapp->config()->readBoolEntry are already
		//  cached by KConfig so there is no disk access involved in every call.
		//  still this extra chache layer shold increase performance.
		//
		//  Some other values than the three above may be chached as well.
		static bool showStaffNrs_, showStaffNames_;
		static bool allowMixedBeames_, allowKeyboardInsert_, allowInsertEcho_;
		static bool moveAccKeysig_, showAuxLines_, showDrumToolbar_;
		static bool automaticBarInsertion_;
		static int defMidiPort_;
		static bool midiPortSet_;
		static int underlength_, overlength_;
		static int schedulerRequest_;
		static int defZoomval_; 
		static staff_props_str nullprops_;
		static main_props_str nullMainProps_;
		static NKeySig *nullKeySig_;
		static NClef *nullClef_;
		static QList<NMainWindow> windowList_;
		static NMidiMapper *mapper_;
		static int lastWindowX_, lastWindowY_;
		static NVoice *voiceWithSelectedRegion_;
		static NMainFrameWidget *windowWithSelectedRegion_;
		static bool isGrabbed_;
		static int numOfMultiStaffs_;
		static KProgress *progress_;
		static QString lyrics_[5];
		static bool dontShowMupWarnings_;
#if KDE_VERSION < 220
		static int tipNo_;
		static QList<QString> theTips_;
		void loadTips(QString fname);
#endif
		static bool *staffSelMute_;
		static bool *staffSelAutobar_;
		static bool *staffSelAutobeam_;
		static bool *staffSelMerge_;
		static bool *staffSelTrack_;
		static bool *staffSelExport_;
		static bool *staffSelMulti_;
		static QTimer autoSaveTimer_;
		static bool lilyAvailable_;
		static bool commandLine_;
		
	private:
		bool loadPixmaps(QPixmap **black_pixmap, QPixmap **red_pixmap, QString fname);
		bool loadAlternativePixmap(QPixmap **grey_pixmap, QString fname, QString suffix);
	private slots:
		void autosave();
};


#endif // RESOURCE_H
