/****************************************************************************************/
/*											*/
/* This program is free software; you can redistribute it and/or modify it under the	*/
/* terms of the GNU General Public License as published by the Free Software		*/
/* Foundation; either version 2 of the License, or (at your option) any later version.	*/
/*											*/
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY	*/
/* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A	*/
/* PARTICULAR PURPOSE. See the GNU General Public License for more details.		*/
/*											*/
/* You should have received a copy of the GNU General Public License along with this	*/
/* program; (See "LICENSE.GPL"). If not, write to the Free Software Foundation, Inc.,	*/
/* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.				*/
/*											*/
/*--------------------------------------------------------------------------------------*/
/*											*/
/*		Joerg Anders, TU Chemnitz, Fakultaet fuer Informatik, GERMANY		*/
/*		ja@informatik.tu-chemnitz.de						*/
/*											*/
/*											*/
/****************************************************************************************/

#include <qpixmap.h>
#include <stdio.h>
#include "rest.h"
#include "resource.h"
#include "transpainter.h"

#define POINT_DIST 6
#define POINT_RAD 5

NRest::NRest(main_props_str *main_props, staff_props_str *staff_props, int *y_voice_offs, int length, unsigned int status) :
		 NMusElement(main_props, staff_props) {
	length_ = length;
	switch (length) {
		case MULTIREST: status_ = 0;
				multiRestLength_ = status;
				break;
		default: status_ = status;
			 multiRestLength_ = 0;
			 break;
	}
	midiLength_ = computeMidiLength();
	actual_ = false;
	xpos_ = 0;
	yRestOffs_ = y_voice_offs;
	calculateDimensionsAndPixmaps();
}

NRest *NRest::clone() {
	NRest *crest;
	crest = new NRest(main_props_, staff_props_, yRestOffs_, length_, false);
	*crest = *this;
	crest->actual_ = false;
	return crest;
}

int NRest::computeMidiLength() const {
	if (length_ == MULTIREST) {
		return staff_props_->measureLength * multiRestLength_;
	}
	if (status_ & STAT_TRIPLET) {
		return 2 * length_ / 3;
	}
	switch (status_ & DOT_MASK) {
		case 1: return 3 * length_ / 2;
		case 2: return 7 * length_ / 4;
	}
	return length_;
}

void NRest::changeLength(int length) {
	if (length > 3*128) return;
	length_ = length;
	midiLength_ = computeMidiLength();
}

void NRest::setTripletParams(QList<NMusElement> *tripletList, 
			bool last, double m, double n, double tritexn, int xstart, int xend) {
	tripletList_ = tripletList;
	SET_STATUS(last, status_, STAT_LAST_TRIPLET);
	status_ |= STAT_TRIPLET;
	triTeXn_ = tritexn;
	trim_ = m; trin_ = n; xstart_ = xstart; xend_ = xend;
	midiLength_ = computeMidiLength();
}

void NRest::calculateDimensionsAndPixmaps() {
	int ypos;
	int lineoffs = 0;
	int yoffs = 0;
	int pointyoffs = 30;
	switch (length_) {
	      case MULTIREST:
			ypos = staff_props_->base-3*LINE_DIST;
			pointPos1_ = QRect(xpos_ + 3, staff_props_->base + 3*LINE_DIST/2, 200, 12);
			lenString_.sprintf("%d", multiRestLength_);
			nbaseDrawPoint_ = QPoint(xpos_ + 80, staff_props_->base + 1*LINE_DIST);
			break;
	      case 3*128: blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::rfullMagPixmap_ : NResource::rfullPixmap_ ;
			redPixmap_   = NResource::rfullRedPixmap_;
			greyPixmap_   = NResource::rfullGreyPixmap_;
			lineoffs = -2;
			pointyoffs = -2;
			break;
	      case 3*64: blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::rhalfMagPixmap_ : NResource::rhalfPixmap_;
			redPixmap_   = NResource::rhalfRedPixmap_;
			greyPixmap_ = NResource::rhalfGreyPixmap_;
			lineoffs = -3;
			pointyoffs = -2;
			yoffs = 5;
			break;
	      case 3*32: blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::rquarterMagPixmap_ : NResource::rquarterPixmap_;
			redPixmap_   = NResource::rquarterRedPixmap_;
			greyPixmap_   = NResource::rquarterGreyPixmap_;
			lineoffs = -1;
			pointyoffs = 30;
			break;
	      case 3*16: blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::r8MagPixmap_ : NResource::r8Pixmap_;
			redPixmap_   = NResource::r8RedPixmap_;
			greyPixmap_   = NResource::r8GreyPixmap_;
			lineoffs = -3;
			pointyoffs = 30;
			break;
	      case 3*8 : blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::r16MagPixmap_ : NResource::r16Pixmap_;
			redPixmap_   = NResource::r16RedPixmap_;
			greyPixmap_   = NResource::r16GreyPixmap_;
			lineoffs = -2;
			pointyoffs = 40;
			break;
	      case 3*4  :blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::r32MagPixmap_ : NResource::r32Pixmap_;
			redPixmap_   = NResource::r32RedPixmap_;
			greyPixmap_   = NResource::r32GreyPixmap_;
			lineoffs = -1;
			pointyoffs = 50;
			break;
		case 3*2 : blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::r64MagPixmap_ : NResource::r64Pixmap_;
			redPixmap_   = NResource::r64RedPixmap_;
			greyPixmap_   = NResource::r64GreyPixmap_;
			lineoffs = -1;
			pointyoffs = 60;
			break;
		case 3*1  :
			blackPixmap_ = (status_ & STAT_HIDDEN ) ? NResource::r128MagPixmap_ : NResource::r128Pixmap_;
			redPixmap_   = NResource::r128RedPixmap_;
			greyPixmap_   = NResource::r128GreyPixmap_;
			lineoffs = 1;
			pointyoffs = 70;
			break;
                default :
                        qWarning("Unknown rest %d", length_);
                        blackPixmap_ = NResource::r128Pixmap_;
                        redPixmap_   = NResource::r128RedPixmap_;
                        greyPixmap_   = NResource::r128RedPixmap_;
                        lineoffs = 1;
                        break;
	}
	if (status_ & STAT_LAST_TRIPLET) {
		triplet0_ = QPoint(xstart_, xstart_ * trim_ + trin_);
		triplet1_ = QPoint(xend_, xend_ * trim_ + trin_);
		triplet00_ = triplet0_ + QPoint(0, TRIPLET_HEIGHT);
		triplet01_ = triplet1_ + QPoint(0, TRIPLET_HEIGHT);
		tripletThree_ = triplet0_ + (triplet1_ - triplet0_) / 2 + QPoint(0, -THREE_DIST);
	}
	ypos = staff_props_->base - lineoffs * LINE_DIST/2 + yoffs;
	if (length_ == MULTIREST) {
		pixmapHeight_ = 3*LINE_DIST;
		pixmapWidth_ = 200;
		bbox_ = QRect(xpos_, staff_props_->base + 3*LINE_DIST/2 , pixmapWidth_, pixmapHeight_);
	}
	else {
		pixmapHeight_ = blackPixmap_->height();
		pixmapWidth_ = blackPixmap_->width();
		pointPos1_ = QRect(xpos_ + pixmapWidth_ + POINT_DIST, ypos+pointyoffs+ ((*yRestOffs_) * LINE_DIST), 2*POINT_RAD, 2*POINT_RAD);
		pointPos2_ = QRect(xpos_ + pixmapWidth_ + 2*POINT_DIST+2*POINT_RAD, ypos+pointyoffs+ ((*yRestOffs_) * LINE_DIST), 2*POINT_RAD, 2*POINT_RAD);
		nbaseDrawPoint_ = QPoint (xpos_, ypos + ((*yRestOffs_) * LINE_DIST));
		switch(status_ & DOT_MASK) {
			case 1: pixmapWidth_ += 3*POINT_DIST + 2*POINT_RAD; break;
			case 2: pixmapWidth_ += 4*POINT_DIST+  4*POINT_RAD; break;
		}
		bbox_ = QRect(xpos_, ypos+(*yRestOffs_ * LINE_DIST) , pixmapWidth_, pixmapHeight_);
	}
}

QPoint *NRest::getTopY() {
	return new QPoint(xpos_ + pixmapWidth_, staff_props_->base + 5);
}

int NRest::getTopY2() {
	return staff_props_->base + 5;
}

int NRest::getTopX2() {
	return xpos_;
}

double NRest::getBotY() {
	return staff_props_->base - (double) (-2 * LINE_DIST) / 2.0;
}

void NRest::draw(int flags) {
	if ((status_ & STAT_HIDDEN) && (flags & DRAW_NO_HIDDEN_REST)) return;
	main_props_->tp->beginTranslated();
	if (length_ == MULTIREST) {
		main_props_->tp->setPen(actual_ ? NResource::redPen : NResource::blackPen);
		main_props_->tp->fillRect(pointPos1_, actual_ ? NResource::redBrush : NResource::blackBrush);
		main_props_->tp->toggleToScaledText(true);
		main_props_->tp->setFont( main_props_->scaledBold_ );
		main_props_->tp->drawScaledText(nbaseDrawPoint_, lenString_);
		main_props_->tp->end();
		return;
	}
	main_props_->tp->drawPixmap (nbaseDrawPoint_, actual_ ?
					 *redPixmap_ : ((flags & DRAW_INDIRECT_GREY) ?  *greyPixmap_ : *blackPixmap_));
	if (status_ & DOT_MASK) { //  draw the dot for dotted rests
		main_props_->tp->setPen(actual_ ? NResource::redPen : NResource::blackPen);
		main_props_->tp->setBrush(actual_ ? NResource::redBrush : NResource::blackBrush);
		main_props_->tp->drawPie(pointPos1_ , 0, 320*16);
		if ((status_ & DOT_MASK) > 1) {
			main_props_->tp->drawPie(pointPos2_, 0, 320*16);
		}
	}
	if (status_ & STAT_LAST_TRIPLET) {
			main_props_->tp->setPen(NResource::blackWidePen);
			main_props_->tp->drawPixmap(tripletThree_, *NResource::threePixmap_);
			main_props_->tp->drawLine(triplet00_, triplet0_);
			main_props_->tp->drawLine(triplet0_, triplet1_);
			main_props_->tp->drawLine(triplet1_, triplet01_);
	}
	main_props_->tp->end();
}

void NRest::setDotted(int dotcount) {
	if (length_ == MULTIREST) return;
	status_ &= (~DOT_MASK);
	status_ |= (dotcount & DOT_MASK);
	midiLength_ = computeMidiLength();
}
